use pokerust::*;

mod common;

fn test_from_id<T: FromId>(id: i16, json: &str) {
    let _m = common::make_mock(&format!("/api/v2/{}/{}/", T::ENDPOINT, id), json);
    T::from_id(id).unwrap();
}

fn test_from_name<T: FromName>(name: &str, json: &str) {
    let _m = common::make_mock(&format!("/api/v2/{}/{}/", T::ENDPOINT, name), json);
    T::from_name(name).unwrap();
}

// https://pokeapi.co/api/v2/berry/2/
static CHESTO_JSON: &str = r#"{"id":2,"name":"chesto","growth_time":3,"max_harvest":5,"natural_gift_power":60,"size":80,"smoothness":25,"soil_dryness":15,"firmness":{"name":"super-hard","url":"http:\/\/localhost:8000\/api\/v2\/berry-firmness\/5\/"},"flavors":[{"potency":0,"flavor":{"name":"spicy","url":"http:\/\/localhost:8000\/api\/v2\/berry-flavor\/1\/"}},{"potency":10,"flavor":{"name":"dry","url":"http:\/\/localhost:8000\/api\/v2\/berry-flavor\/2\/"}},{"potency":0,"flavor":{"name":"sweet","url":"http:\/\/localhost:8000\/api\/v2\/berry-flavor\/3\/"}},{"potency":0,"flavor":{"name":"bitter","url":"http:\/\/localhost:8000\/api\/v2\/berry-flavor\/4\/"}},{"potency":0,"flavor":{"name":"sour","url":"http:\/\/localhost:8000\/api\/v2\/berry-flavor\/5\/"}}],"item":{"name":"chesto-berry","url":"http:\/\/localhost:8000\/api\/v2\/item\/127\/"},"natural_gift_type":{"name":"water","url":"http:\/\/localhost:8000\/api\/v2\/type\/11\/"}}"#;

#[test]
fn berry_from_id() {
    test_from_id::<Berry>(2, CHESTO_JSON);
}

#[test]
fn berry_from_name() {
    test_from_name::<Berry>("chesto", CHESTO_JSON);
}

// https://pokeapi.co/api/v2/berry-firmness/5/
static SUPER_HARD_JSON: &str = r#"{"berries":[{"name":"chesto","url":"https://pokeapi.co/api/v2/berry/2/"},{"name":"aspear","url":"https://pokeapi.co/api/v2/berry/5/"},{"name":"oran","url":"https://pokeapi.co/api/v2/berry/7/"},{"name":"lum","url":"https://pokeapi.co/api/v2/berry/9/"},{"name":"aguav","url":"https://pokeapi.co/api/v2/berry/14/"},{"name":"wepear","url":"https://pokeapi.co/api/v2/berry/19/"},{"name":"nomel","url":"https://pokeapi.co/api/v2/berry/30/"},{"name":"occa","url":"https://pokeapi.co/api/v2/berry/36/"},{"name":"colbur","url":"https://pokeapi.co/api/v2/berry/50/"},{"name":"babiri","url":"https://pokeapi.co/api/v2/berry/51/"},{"name":"starf","url":"https://pokeapi.co/api/v2/berry/59/"},{"name":"custap","url":"https://pokeapi.co/api/v2/berry/62/"}],"id":5,"name":"super-hard","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Super ferme"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Super Hard"}]}"#;

#[test]
fn berry_firmness_from_id() {
    test_from_id::<BerryFirmness>(5, SUPER_HARD_JSON);
}

#[test]
fn berry_firmness_from_name() {
    test_from_name::<BerryFirmness>("super-hard", SUPER_HARD_JSON);
}

// https://pokeapi.co/api/v2/berry-flavor/1/
static SPICY_JSON: &str = r#"{"berries":[{"berry":{"name":"rowap","url":"https://pokeapi.co/api/v2/berry/64/"},"potency":10},{"berry":{"name":"leppa","url":"https://pokeapi.co/api/v2/berry/6/"},"potency":10},{"berry":{"name":"oran","url":"https://pokeapi.co/api/v2/berry/7/"},"potency":10},{"berry":{"name":"persim","url":"https://pokeapi.co/api/v2/berry/8/"},"potency":10},{"berry":{"name":"lum","url":"https://pokeapi.co/api/v2/berry/9/"},"potency":10},{"berry":{"name":"razz","url":"https://pokeapi.co/api/v2/berry/16/"},"potency":10},{"berry":{"name":"pinap","url":"https://pokeapi.co/api/v2/berry/20/"},"potency":10},{"berry":{"name":"pomeg","url":"https://pokeapi.co/api/v2/berry/21/"},"potency":10},{"berry":{"name":"qualot","url":"https://pokeapi.co/api/v2/berry/23/"},"potency":10},{"berry":{"name":"hondew","url":"https://pokeapi.co/api/v2/berry/24/"},"potency":10},{"berry":{"name":"nomel","url":"https://pokeapi.co/api/v2/berry/30/"},"potency":10},{"berry":{"name":"belue","url":"https://pokeapi.co/api/v2/berry/35/"},"potency":10},{"berry":{"name":"rindo","url":"https://pokeapi.co/api/v2/berry/39/"},"potency":10},{"berry":{"name":"shuca","url":"https://pokeapi.co/api/v2/berry/43/"},"potency":10},{"berry":{"name":"charti","url":"https://pokeapi.co/api/v2/berry/47/"},"potency":10},{"berry":{"name":"apicot","url":"https://pokeapi.co/api/v2/berry/57/"},"potency":10},{"berry":{"name":"cheri","url":"https://pokeapi.co/api/v2/berry/1/"},"potency":10},{"berry":{"name":"chople","url":"https://pokeapi.co/api/v2/berry/41/"},"potency":15},{"berry":{"name":"figy","url":"https://pokeapi.co/api/v2/berry/11/"},"potency":15},{"berry":{"name":"occa","url":"https://pokeapi.co/api/v2/berry/36/"},"potency":15},{"berry":{"name":"tamato","url":"https://pokeapi.co/api/v2/berry/26/"},"potency":20},{"berry":{"name":"tanga","url":"https://pokeapi.co/api/v2/berry/46/"},"potency":20},{"berry":{"name":"babiri","url":"https://pokeapi.co/api/v2/berry/51/"},"potency":25},{"berry":{"name":"starf","url":"https://pokeapi.co/api/v2/berry/59/"},"potency":30},{"berry":{"name":"liechi","url":"https://pokeapi.co/api/v2/berry/53/"},"potency":30},{"berry":{"name":"spelon","url":"https://pokeapi.co/api/v2/berry/31/"},"potency":30},{"berry":{"name":"petaya","url":"https://pokeapi.co/api/v2/berry/56/"},"potency":30},{"berry":{"name":"lansat","url":"https://pokeapi.co/api/v2/berry/58/"},"potency":30},{"berry":{"name":"enigma","url":"https://pokeapi.co/api/v2/berry/60/"},"potency":40}],"contest_type":{"name":"cool","url":"https://pokeapi.co/api/v2/contest-type/1/"},"id":1,"name":"spicy","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Épicé"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Spicy"}]}"#;

#[test]
fn berry_flavor_from_id() {
    test_from_id::<BerryFlavor>(1, SPICY_JSON);
}

#[test]
fn berry_flavor_from_name() {
    test_from_name::<BerryFlavor>("spicy", SPICY_JSON);
}

// https://pokeapi.co/api/v2/contest-type/5/
static COOL_JSON: &str = r#"{"berry_flavor":{"name":"spicy","url":"https://pokeapi.co/api/v2/berry-flavor/1/"},"id":1,"name":"cool","names":[{"color":"Red","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Cool"},{"color":"Rouge","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Sang-froid"}]}"#;

#[test]
fn contest_type_from_id() {
    test_from_id::<ContestType>(1, COOL_JSON);
}

#[test]
fn contest_type_from_name() {
    test_from_name::<ContestType>("cool", COOL_JSON);
}

// https://pokeapi.co/api/v2/contest-effect/9/
static CONTEST_EFFECT_9_JSON: &str = r#"{"appeal":2,"effect_entries":[{"effect":"Attempts to jam the Pokémon that appealed before the user.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"flavor_text_entries":[{"flavor_text":"Startles the Pokémon that appealed before the user.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"id":9,"jam":3}"#;

#[test]
fn contest_effect_from_id() {
    test_from_id::<ContestEffect>(9, CONTEST_EFFECT_9_JSON);
}

// https://pokeapi.co/api/v2/super-contest-effect/4/
static SUPER_CONTEST_EFFECT_4_JSON: &str = r#"{"appeal":2,"flavor_text_entries":[{"flavor_text":"Earn +2 if the Judge's Voltage goes up.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"id":4,"moves":[{"name":"octazooka","url":"https://pokeapi.co/api/v2/move/190/"},{"name":"zap-cannon","url":"https://pokeapi.co/api/v2/move/192/"},{"name":"poison-fang","url":"https://pokeapi.co/api/v2/move/305/"},{"name":"weather-ball","url":"https://pokeapi.co/api/v2/move/311/"},{"name":"silver-wind","url":"https://pokeapi.co/api/v2/move/318/"},{"name":"signal-beam","url":"https://pokeapi.co/api/v2/move/324/"},{"name":"poison-tail","url":"https://pokeapi.co/api/v2/move/342/"},{"name":"brine","url":"https://pokeapi.co/api/v2/move/362/"},{"name":"poison-jab","url":"https://pokeapi.co/api/v2/move/398/"},{"name":"bug-buzz","url":"https://pokeapi.co/api/v2/move/405/"},{"name":"dragon-pulse","url":"https://pokeapi.co/api/v2/move/406/"},{"name":"stone-edge","url":"https://pokeapi.co/api/v2/move/444/"},{"name":"grass-knot","url":"https://pokeapi.co/api/v2/move/447/"},{"name":"attack-order","url":"https://pokeapi.co/api/v2/move/454/"},{"name":"spacial-rend","url":"https://pokeapi.co/api/v2/move/460/"}]}"#;

#[test]
fn super_contest_effect_from_id() {
    test_from_id::<SuperContestEffect>(4, SUPER_CONTEST_EFFECT_4_JSON);
}

// https://pokeapi.co/api/v2/encounter-method/7/
static HEADBUTT_JSON: &str = r#"{"id":7,"name":"headbutt","names":[{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Bäumen Kopfnüsse geben"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Headbutting trees"}],"order":17}"#;

#[test]
fn encounter_method_from_id() {
    test_from_id::<EncounterMethod>(7, HEADBUTT_JSON);
}

#[test]
fn encounter_method_from_name() {
    test_from_name::<EncounterMethod>("headbutt", HEADBUTT_JSON);
}

// https://pokeapi.co/api/v2/encounter-condition/5/
static RADIO_JSON: &str = r#"{"id":5,"name":"radio","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Radio"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Radio"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Radio"}],"values":[{"name":"radio-off","url":"https://pokeapi.co/api/v2/encounter-condition-value/14/"},{"name":"radio-hoenn","url":"https://pokeapi.co/api/v2/encounter-condition-value/15/"},{"name":"radio-sinnoh","url":"https://pokeapi.co/api/v2/encounter-condition-value/16/"}]}"#;

#[test]
fn encounter_condition_from_id() {
    test_from_id::<EncounterCondition>(5, RADIO_JSON);
}

#[test]
fn encounter_condition_from_name() {
    test_from_name::<EncounterCondition>("radio", RADIO_JSON);
}

// https://pokeapi.co/api/v2/encounter-condition-value/14/
static RADIO_OFF_JSON: &str = r#"{"condition":{"name":"radio","url":"https://pokeapi.co/api/v2/encounter-condition/5/"},"id":14,"name":"radio-off","names":[{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Radio aus"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Radio off"}]}"#;

#[test]
fn encounter_condition_value_from_id() {
    test_from_id::<EncounterConditionValue>(14, RADIO_OFF_JSON);
}

#[test]
fn encounter_condition_value_from_name() {
    test_from_name::<EncounterConditionValue>("radio-off", RADIO_OFF_JSON);
}

// https://pokeapi.co/api/v2/evolution-chain/145/
static EVOLUTION_CHAIN_145_JSON: &str = r#"{"baby_trigger_item":null,"chain":{"evolution_details":[],"evolves_to":[{"evolution_details":[{"gender":null,"held_item":null,"item":null,"known_move":null,"known_move_type":null,"location":null,"min_affection":null,"min_beauty":null,"min_happiness":null,"min_level":20,"needs_overworld_rain":false,"party_species":null,"party_type":null,"relative_physical_stats":null,"time_of_day":"","trade_species":null,"trigger":{"name":"level-up","url":"https://pokeapi.co/api/v2/evolution-trigger/1/"},"turn_upside_down":false}],"evolves_to":[{"evolution_details":[{"gender":null,"held_item":null,"item":null,"known_move":null,"known_move_type":null,"location":null,"min_affection":null,"min_beauty":null,"min_happiness":null,"min_level":40,"needs_overworld_rain":false,"party_species":null,"party_type":null,"relative_physical_stats":null,"time_of_day":"","trade_species":null,"trigger":{"name":"level-up","url":"https://pokeapi.co/api/v2/evolution-trigger/1/"},"turn_upside_down":false}],"evolves_to":[],"is_baby":false,"species":{"name":"exploud","url":"https://pokeapi.co/api/v2/pokemon-species/295/"}}],"is_baby":false,"species":{"name":"loudred","url":"https://pokeapi.co/api/v2/pokemon-species/294/"}}],"is_baby":false,"species":{"name":"whismur","url":"https://pokeapi.co/api/v2/pokemon-species/293/"}},"id":145}"#;

#[test]
fn evolution_chain_value_from_id() {
    test_from_id::<EvolutionChain>(145, EVOLUTION_CHAIN_145_JSON);
}

// https://pokeapi.co/api/v2/evolution-trgger/4/
static SHED_JSON: &str = r#"{"id":4,"name":"shed","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Place dans l'équipe et une Poké Ball"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Platz im Team und ein Pokéball"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Shed"}],"pokemon_species":[{"name":"shedinja","url":"https://pokeapi.co/api/v2/pokemon-species/292/"}]}"#;

#[test]
fn evolution_trigger_from_id() {
    test_from_id::<EvolutionTrigger>(4, SHED_JSON);
}

#[test]
fn evolution_trigger_from_name() {
    test_from_name::<EvolutionTrigger>("shed", SHED_JSON);
}

// https://pokeapi.co/api/v2/generation/3/
static GENERATION_III_JSON: &str = r#"{"abilities":[{"name":"stench","url":"https://pokeapi.co/api/v2/ability/1/"},{"name":"drizzle","url":"https://pokeapi.co/api/v2/ability/2/"},{"name":"speed-boost","url":"https://pokeapi.co/api/v2/ability/3/"},{"name":"battle-armor","url":"https://pokeapi.co/api/v2/ability/4/"},{"name":"sturdy","url":"https://pokeapi.co/api/v2/ability/5/"},{"name":"damp","url":"https://pokeapi.co/api/v2/ability/6/"},{"name":"limber","url":"https://pokeapi.co/api/v2/ability/7/"},{"name":"sand-veil","url":"https://pokeapi.co/api/v2/ability/8/"},{"name":"static","url":"https://pokeapi.co/api/v2/ability/9/"},{"name":"volt-absorb","url":"https://pokeapi.co/api/v2/ability/10/"},{"name":"water-absorb","url":"https://pokeapi.co/api/v2/ability/11/"},{"name":"oblivious","url":"https://pokeapi.co/api/v2/ability/12/"},{"name":"cloud-nine","url":"https://pokeapi.co/api/v2/ability/13/"},{"name":"compound-eyes","url":"https://pokeapi.co/api/v2/ability/14/"},{"name":"insomnia","url":"https://pokeapi.co/api/v2/ability/15/"},{"name":"color-change","url":"https://pokeapi.co/api/v2/ability/16/"},{"name":"immunity","url":"https://pokeapi.co/api/v2/ability/17/"},{"name":"flash-fire","url":"https://pokeapi.co/api/v2/ability/18/"},{"name":"shield-dust","url":"https://pokeapi.co/api/v2/ability/19/"},{"name":"own-tempo","url":"https://pokeapi.co/api/v2/ability/20/"},{"name":"suction-cups","url":"https://pokeapi.co/api/v2/ability/21/"},{"name":"intimidate","url":"https://pokeapi.co/api/v2/ability/22/"},{"name":"shadow-tag","url":"https://pokeapi.co/api/v2/ability/23/"},{"name":"rough-skin","url":"https://pokeapi.co/api/v2/ability/24/"},{"name":"wonder-guard","url":"https://pokeapi.co/api/v2/ability/25/"},{"name":"levitate","url":"https://pokeapi.co/api/v2/ability/26/"},{"name":"effect-spore","url":"https://pokeapi.co/api/v2/ability/27/"},{"name":"synchronize","url":"https://pokeapi.co/api/v2/ability/28/"},{"name":"clear-body","url":"https://pokeapi.co/api/v2/ability/29/"},{"name":"natural-cure","url":"https://pokeapi.co/api/v2/ability/30/"},{"name":"lightning-rod","url":"https://pokeapi.co/api/v2/ability/31/"},{"name":"serene-grace","url":"https://pokeapi.co/api/v2/ability/32/"},{"name":"swift-swim","url":"https://pokeapi.co/api/v2/ability/33/"},{"name":"chlorophyll","url":"https://pokeapi.co/api/v2/ability/34/"},{"name":"illuminate","url":"https://pokeapi.co/api/v2/ability/35/"},{"name":"trace","url":"https://pokeapi.co/api/v2/ability/36/"},{"name":"huge-power","url":"https://pokeapi.co/api/v2/ability/37/"},{"name":"poison-point","url":"https://pokeapi.co/api/v2/ability/38/"},{"name":"inner-focus","url":"https://pokeapi.co/api/v2/ability/39/"},{"name":"magma-armor","url":"https://pokeapi.co/api/v2/ability/40/"},{"name":"water-veil","url":"https://pokeapi.co/api/v2/ability/41/"},{"name":"magnet-pull","url":"https://pokeapi.co/api/v2/ability/42/"},{"name":"soundproof","url":"https://pokeapi.co/api/v2/ability/43/"},{"name":"rain-dish","url":"https://pokeapi.co/api/v2/ability/44/"},{"name":"sand-stream","url":"https://pokeapi.co/api/v2/ability/45/"},{"name":"pressure","url":"https://pokeapi.co/api/v2/ability/46/"},{"name":"thick-fat","url":"https://pokeapi.co/api/v2/ability/47/"},{"name":"early-bird","url":"https://pokeapi.co/api/v2/ability/48/"},{"name":"flame-body","url":"https://pokeapi.co/api/v2/ability/49/"},{"name":"run-away","url":"https://pokeapi.co/api/v2/ability/50/"},{"name":"keen-eye","url":"https://pokeapi.co/api/v2/ability/51/"},{"name":"hyper-cutter","url":"https://pokeapi.co/api/v2/ability/52/"},{"name":"pickup","url":"https://pokeapi.co/api/v2/ability/53/"},{"name":"truant","url":"https://pokeapi.co/api/v2/ability/54/"},{"name":"hustle","url":"https://pokeapi.co/api/v2/ability/55/"},{"name":"cute-charm","url":"https://pokeapi.co/api/v2/ability/56/"},{"name":"plus","url":"https://pokeapi.co/api/v2/ability/57/"},{"name":"minus","url":"https://pokeapi.co/api/v2/ability/58/"},{"name":"forecast","url":"https://pokeapi.co/api/v2/ability/59/"},{"name":"sticky-hold","url":"https://pokeapi.co/api/v2/ability/60/"},{"name":"shed-skin","url":"https://pokeapi.co/api/v2/ability/61/"},{"name":"guts","url":"https://pokeapi.co/api/v2/ability/62/"},{"name":"marvel-scale","url":"https://pokeapi.co/api/v2/ability/63/"},{"name":"liquid-ooze","url":"https://pokeapi.co/api/v2/ability/64/"},{"name":"overgrow","url":"https://pokeapi.co/api/v2/ability/65/"},{"name":"blaze","url":"https://pokeapi.co/api/v2/ability/66/"},{"name":"torrent","url":"https://pokeapi.co/api/v2/ability/67/"},{"name":"swarm","url":"https://pokeapi.co/api/v2/ability/68/"},{"name":"rock-head","url":"https://pokeapi.co/api/v2/ability/69/"},{"name":"drought","url":"https://pokeapi.co/api/v2/ability/70/"},{"name":"arena-trap","url":"https://pokeapi.co/api/v2/ability/71/"},{"name":"vital-spirit","url":"https://pokeapi.co/api/v2/ability/72/"},{"name":"white-smoke","url":"https://pokeapi.co/api/v2/ability/73/"},{"name":"pure-power","url":"https://pokeapi.co/api/v2/ability/74/"},{"name":"shell-armor","url":"https://pokeapi.co/api/v2/ability/75/"},{"name":"air-lock","url":"https://pokeapi.co/api/v2/ability/76/"}],"id":3,"main_region":{"name":"hoenn","url":"https://pokeapi.co/api/v2/region/3/"},"moves":[{"name":"shadow-sky","url":"https://pokeapi.co/api/v2/move/10018/"},{"name":"shadow-shed","url":"https://pokeapi.co/api/v2/move/10017/"},{"name":"shadow-panic","url":"https://pokeapi.co/api/v2/move/10016/"},{"name":"shadow-mist","url":"https://pokeapi.co/api/v2/move/10015/"},{"name":"shadow-hold","url":"https://pokeapi.co/api/v2/move/10014/"},{"name":"shadow-half","url":"https://pokeapi.co/api/v2/move/10013/"},{"name":"shadow-down","url":"https://pokeapi.co/api/v2/move/10012/"},{"name":"shadow-wave","url":"https://pokeapi.co/api/v2/move/10011/"},{"name":"shadow-storm","url":"https://pokeapi.co/api/v2/move/10010/"},{"name":"shadow-rave","url":"https://pokeapi.co/api/v2/move/10009/"},{"name":"shadow-fire","url":"https://pokeapi.co/api/v2/move/10008/"},{"name":"shadow-end","url":"https://pokeapi.co/api/v2/move/10007/"},{"name":"shadow-chill","url":"https://pokeapi.co/api/v2/move/10006/"},{"name":"shadow-break","url":"https://pokeapi.co/api/v2/move/10005/"},{"name":"shadow-bolt","url":"https://pokeapi.co/api/v2/move/10004/"},{"name":"shadow-blitz","url":"https://pokeapi.co/api/v2/move/10003/"},{"name":"shadow-blast","url":"https://pokeapi.co/api/v2/move/10002/"},{"name":"shadow-rush","url":"https://pokeapi.co/api/v2/move/10001/"},{"name":"psycho-boost","url":"https://pokeapi.co/api/v2/move/354/"},{"name":"doom-desire","url":"https://pokeapi.co/api/v2/move/353/"},{"name":"water-pulse","url":"https://pokeapi.co/api/v2/move/352/"},{"name":"shock-wave","url":"https://pokeapi.co/api/v2/move/351/"},{"name":"rock-blast","url":"https://pokeapi.co/api/v2/move/350/"},{"name":"dragon-dance","url":"https://pokeapi.co/api/v2/move/349/"},{"name":"leaf-blade","url":"https://pokeapi.co/api/v2/move/348/"},{"name":"calm-mind","url":"https://pokeapi.co/api/v2/move/347/"},{"name":"water-sport","url":"https://pokeapi.co/api/v2/move/346/"},{"name":"magical-leaf","url":"https://pokeapi.co/api/v2/move/345/"},{"name":"volt-tackle","url":"https://pokeapi.co/api/v2/move/344/"},{"name":"covet","url":"https://pokeapi.co/api/v2/move/343/"},{"name":"poison-tail","url":"https://pokeapi.co/api/v2/move/342/"},{"name":"mud-shot","url":"https://pokeapi.co/api/v2/move/341/"},{"name":"bounce","url":"https://pokeapi.co/api/v2/move/340/"},{"name":"bulk-up","url":"https://pokeapi.co/api/v2/move/339/"},{"name":"frenzy-plant","url":"https://pokeapi.co/api/v2/move/338/"},{"name":"dragon-claw","url":"https://pokeapi.co/api/v2/move/337/"},{"name":"howl","url":"https://pokeapi.co/api/v2/move/336/"},{"name":"block","url":"https://pokeapi.co/api/v2/move/335/"},{"name":"iron-defense","url":"https://pokeapi.co/api/v2/move/334/"},{"name":"icicle-spear","url":"https://pokeapi.co/api/v2/move/333/"},{"name":"aerial-ace","url":"https://pokeapi.co/api/v2/move/332/"},{"name":"bullet-seed","url":"https://pokeapi.co/api/v2/move/331/"},{"name":"muddy-water","url":"https://pokeapi.co/api/v2/move/330/"},{"name":"sheer-cold","url":"https://pokeapi.co/api/v2/move/329/"},{"name":"sand-tomb","url":"https://pokeapi.co/api/v2/move/328/"},{"name":"sky-uppercut","url":"https://pokeapi.co/api/v2/move/327/"},{"name":"extrasensory","url":"https://pokeapi.co/api/v2/move/326/"},{"name":"shadow-punch","url":"https://pokeapi.co/api/v2/move/325/"},{"name":"signal-beam","url":"https://pokeapi.co/api/v2/move/324/"},{"name":"water-spout","url":"https://pokeapi.co/api/v2/move/323/"},{"name":"cosmic-power","url":"https://pokeapi.co/api/v2/move/322/"},{"name":"tickle","url":"https://pokeapi.co/api/v2/move/321/"},{"name":"grass-whistle","url":"https://pokeapi.co/api/v2/move/320/"},{"name":"metal-sound","url":"https://pokeapi.co/api/v2/move/319/"},{"name":"silver-wind","url":"https://pokeapi.co/api/v2/move/318/"},{"name":"rock-tomb","url":"https://pokeapi.co/api/v2/move/317/"},{"name":"odor-sleuth","url":"https://pokeapi.co/api/v2/move/316/"},{"name":"overheat","url":"https://pokeapi.co/api/v2/move/315/"},{"name":"air-cutter","url":"https://pokeapi.co/api/v2/move/314/"},{"name":"fake-tears","url":"https://pokeapi.co/api/v2/move/313/"},{"name":"aromatherapy","url":"https://pokeapi.co/api/v2/move/312/"},{"name":"weather-ball","url":"https://pokeapi.co/api/v2/move/311/"},{"name":"astonish","url":"https://pokeapi.co/api/v2/move/310/"},{"name":"meteor-mash","url":"https://pokeapi.co/api/v2/move/309/"},{"name":"hydro-cannon","url":"https://pokeapi.co/api/v2/move/308/"},{"name":"blast-burn","url":"https://pokeapi.co/api/v2/move/307/"},{"name":"crush-claw","url":"https://pokeapi.co/api/v2/move/306/"},{"name":"poison-fang","url":"https://pokeapi.co/api/v2/move/305/"},{"name":"hyper-voice","url":"https://pokeapi.co/api/v2/move/304/"},{"name":"slack-off","url":"https://pokeapi.co/api/v2/move/303/"},{"name":"needle-arm","url":"https://pokeapi.co/api/v2/move/302/"},{"name":"ice-ball","url":"https://pokeapi.co/api/v2/move/301/"},{"name":"mud-sport","url":"https://pokeapi.co/api/v2/move/300/"},{"name":"blaze-kick","url":"https://pokeapi.co/api/v2/move/299/"},{"name":"teeter-dance","url":"https://pokeapi.co/api/v2/move/298/"},{"name":"feather-dance","url":"https://pokeapi.co/api/v2/move/297/"},{"name":"mist-ball","url":"https://pokeapi.co/api/v2/move/296/"},{"name":"luster-purge","url":"https://pokeapi.co/api/v2/move/295/"},{"name":"tail-glow","url":"https://pokeapi.co/api/v2/move/294/"},{"name":"camouflage","url":"https://pokeapi.co/api/v2/move/293/"},{"name":"arm-thrust","url":"https://pokeapi.co/api/v2/move/292/"},{"name":"dive","url":"https://pokeapi.co/api/v2/move/291/"},{"name":"secret-power","url":"https://pokeapi.co/api/v2/move/290/"},{"name":"snatch","url":"https://pokeapi.co/api/v2/move/289/"},{"name":"grudge","url":"https://pokeapi.co/api/v2/move/288/"},{"name":"refresh","url":"https://pokeapi.co/api/v2/move/287/"},{"name":"imprison","url":"https://pokeapi.co/api/v2/move/286/"},{"name":"skill-swap","url":"https://pokeapi.co/api/v2/move/285/"},{"name":"eruption","url":"https://pokeapi.co/api/v2/move/284/"},{"name":"endeavor","url":"https://pokeapi.co/api/v2/move/283/"},{"name":"knock-off","url":"https://pokeapi.co/api/v2/move/282/"},{"name":"yawn","url":"https://pokeapi.co/api/v2/move/281/"},{"name":"brick-break","url":"https://pokeapi.co/api/v2/move/280/"},{"name":"revenge","url":"https://pokeapi.co/api/v2/move/279/"},{"name":"recycle","url":"https://pokeapi.co/api/v2/move/278/"},{"name":"magic-coat","url":"https://pokeapi.co/api/v2/move/277/"},{"name":"superpower","url":"https://pokeapi.co/api/v2/move/276/"},{"name":"ingrain","url":"https://pokeapi.co/api/v2/move/275/"},{"name":"assist","url":"https://pokeapi.co/api/v2/move/274/"},{"name":"wish","url":"https://pokeapi.co/api/v2/move/273/"},{"name":"role-play","url":"https://pokeapi.co/api/v2/move/272/"},{"name":"trick","url":"https://pokeapi.co/api/v2/move/271/"},{"name":"helping-hand","url":"https://pokeapi.co/api/v2/move/270/"},{"name":"taunt","url":"https://pokeapi.co/api/v2/move/269/"},{"name":"charge","url":"https://pokeapi.co/api/v2/move/268/"},{"name":"nature-power","url":"https://pokeapi.co/api/v2/move/267/"},{"name":"follow-me","url":"https://pokeapi.co/api/v2/move/266/"},{"name":"smelling-salts","url":"https://pokeapi.co/api/v2/move/265/"},{"name":"focus-punch","url":"https://pokeapi.co/api/v2/move/264/"},{"name":"facade","url":"https://pokeapi.co/api/v2/move/263/"},{"name":"memento","url":"https://pokeapi.co/api/v2/move/262/"},{"name":"will-o-wisp","url":"https://pokeapi.co/api/v2/move/261/"},{"name":"flatter","url":"https://pokeapi.co/api/v2/move/260/"},{"name":"torment","url":"https://pokeapi.co/api/v2/move/259/"},{"name":"hail","url":"https://pokeapi.co/api/v2/move/258/"},{"name":"heat-wave","url":"https://pokeapi.co/api/v2/move/257/"},{"name":"swallow","url":"https://pokeapi.co/api/v2/move/256/"},{"name":"spit-up","url":"https://pokeapi.co/api/v2/move/255/"},{"name":"stockpile","url":"https://pokeapi.co/api/v2/move/254/"},{"name":"uproar","url":"https://pokeapi.co/api/v2/move/253/"},{"name":"fake-out","url":"https://pokeapi.co/api/v2/move/252/"}],"name":"generation-iii","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"だいさんせだい"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Génération III"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Generation III"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Generation III"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"第三世代"}],"pokemon_species":[{"name":"metagross","url":"https://pokeapi.co/api/v2/pokemon-species/376/"},{"name":"metang","url":"https://pokeapi.co/api/v2/pokemon-species/375/"},{"name":"salamence","url":"https://pokeapi.co/api/v2/pokemon-species/373/"},{"name":"shelgon","url":"https://pokeapi.co/api/v2/pokemon-species/372/"},{"name":"gorebyss","url":"https://pokeapi.co/api/v2/pokemon-species/368/"},{"name":"huntail","url":"https://pokeapi.co/api/v2/pokemon-species/367/"},{"name":"walrein","url":"https://pokeapi.co/api/v2/pokemon-species/365/"},{"name":"sealeo","url":"https://pokeapi.co/api/v2/pokemon-species/364/"},{"name":"glalie","url":"https://pokeapi.co/api/v2/pokemon-species/362/"},{"name":"chimecho","url":"https://pokeapi.co/api/v2/pokemon-species/358/"},{"name":"dusclops","url":"https://pokeapi.co/api/v2/pokemon-species/356/"},{"name":"banette","url":"https://pokeapi.co/api/v2/pokemon-species/354/"},{"name":"milotic","url":"https://pokeapi.co/api/v2/pokemon-species/350/"},{"name":"armaldo","url":"https://pokeapi.co/api/v2/pokemon-species/348/"},{"name":"cradily","url":"https://pokeapi.co/api/v2/pokemon-species/346/"},{"name":"claydol","url":"https://pokeapi.co/api/v2/pokemon-species/344/"},{"name":"crawdaunt","url":"https://pokeapi.co/api/v2/pokemon-species/342/"},{"name":"whiscash","url":"https://pokeapi.co/api/v2/pokemon-species/340/"},{"name":"altaria","url":"https://pokeapi.co/api/v2/pokemon-species/334/"},{"name":"cacturne","url":"https://pokeapi.co/api/v2/pokemon-species/332/"},{"name":"flygon","url":"https://pokeapi.co/api/v2/pokemon-species/330/"},{"name":"vibrava","url":"https://pokeapi.co/api/v2/pokemon-species/329/"},{"name":"grumpig","url":"https://pokeapi.co/api/v2/pokemon-species/326/"},{"name":"camerupt","url":"https://pokeapi.co/api/v2/pokemon-species/323/"},{"name":"wailord","url":"https://pokeapi.co/api/v2/pokemon-species/321/"},{"name":"sharpedo","url":"https://pokeapi.co/api/v2/pokemon-species/319/"},{"name":"swalot","url":"https://pokeapi.co/api/v2/pokemon-species/317/"},{"name":"roselia","url":"https://pokeapi.co/api/v2/pokemon-species/315/"},{"name":"manectric","url":"https://pokeapi.co/api/v2/pokemon-species/310/"},{"name":"medicham","url":"https://pokeapi.co/api/v2/pokemon-species/308/"},{"name":"aggron","url":"https://pokeapi.co/api/v2/pokemon-species/306/"},{"name":"lairon","url":"https://pokeapi.co/api/v2/pokemon-species/305/"},{"name":"delcatty","url":"https://pokeapi.co/api/v2/pokemon-species/301/"},{"name":"hariyama","url":"https://pokeapi.co/api/v2/pokemon-species/297/"},{"name":"exploud","url":"https://pokeapi.co/api/v2/pokemon-species/295/"},{"name":"loudred","url":"https://pokeapi.co/api/v2/pokemon-species/294/"},{"name":"shedinja","url":"https://pokeapi.co/api/v2/pokemon-species/292/"},{"name":"ninjask","url":"https://pokeapi.co/api/v2/pokemon-species/291/"},{"name":"slaking","url":"https://pokeapi.co/api/v2/pokemon-species/289/"},{"name":"vigoroth","url":"https://pokeapi.co/api/v2/pokemon-species/288/"},{"name":"breloom","url":"https://pokeapi.co/api/v2/pokemon-species/286/"},{"name":"masquerain","url":"https://pokeapi.co/api/v2/pokemon-species/284/"},{"name":"gardevoir","url":"https://pokeapi.co/api/v2/pokemon-species/282/"},{"name":"kirlia","url":"https://pokeapi.co/api/v2/pokemon-species/281/"},{"name":"pelipper","url":"https://pokeapi.co/api/v2/pokemon-species/279/"},{"name":"swellow","url":"https://pokeapi.co/api/v2/pokemon-species/277/"},{"name":"shiftry","url":"https://pokeapi.co/api/v2/pokemon-species/275/"},{"name":"nuzleaf","url":"https://pokeapi.co/api/v2/pokemon-species/274/"},{"name":"ludicolo","url":"https://pokeapi.co/api/v2/pokemon-species/272/"},{"name":"lombre","url":"https://pokeapi.co/api/v2/pokemon-species/271/"},{"name":"dustox","url":"https://pokeapi.co/api/v2/pokemon-species/269/"},{"name":"cascoon","url":"https://pokeapi.co/api/v2/pokemon-species/268/"},{"name":"beautifly","url":"https://pokeapi.co/api/v2/pokemon-species/267/"},{"name":"silcoon","url":"https://pokeapi.co/api/v2/pokemon-species/266/"},{"name":"linoone","url":"https://pokeapi.co/api/v2/pokemon-species/264/"},{"name":"mightyena","url":"https://pokeapi.co/api/v2/pokemon-species/262/"},{"name":"swampert","url":"https://pokeapi.co/api/v2/pokemon-species/260/"},{"name":"marshtomp","url":"https://pokeapi.co/api/v2/pokemon-species/259/"},{"name":"blaziken","url":"https://pokeapi.co/api/v2/pokemon-species/257/"},{"name":"combusken","url":"https://pokeapi.co/api/v2/pokemon-species/256/"},{"name":"sceptile","url":"https://pokeapi.co/api/v2/pokemon-species/254/"},{"name":"grovyle","url":"https://pokeapi.co/api/v2/pokemon-species/253/"},{"name":"deoxys","url":"https://pokeapi.co/api/v2/pokemon-species/386/"},{"name":"jirachi","url":"https://pokeapi.co/api/v2/pokemon-species/385/"},{"name":"rayquaza","url":"https://pokeapi.co/api/v2/pokemon-species/384/"},{"name":"groudon","url":"https://pokeapi.co/api/v2/pokemon-species/383/"},{"name":"kyogre","url":"https://pokeapi.co/api/v2/pokemon-species/382/"},{"name":"latios","url":"https://pokeapi.co/api/v2/pokemon-species/381/"},{"name":"latias","url":"https://pokeapi.co/api/v2/pokemon-species/380/"},{"name":"registeel","url":"https://pokeapi.co/api/v2/pokemon-species/379/"},{"name":"regice","url":"https://pokeapi.co/api/v2/pokemon-species/378/"},{"name":"regirock","url":"https://pokeapi.co/api/v2/pokemon-species/377/"},{"name":"beldum","url":"https://pokeapi.co/api/v2/pokemon-species/374/"},{"name":"bagon","url":"https://pokeapi.co/api/v2/pokemon-species/371/"},{"name":"luvdisc","url":"https://pokeapi.co/api/v2/pokemon-species/370/"},{"name":"relicanth","url":"https://pokeapi.co/api/v2/pokemon-species/369/"},{"name":"clamperl","url":"https://pokeapi.co/api/v2/pokemon-species/366/"},{"name":"spheal","url":"https://pokeapi.co/api/v2/pokemon-species/363/"},{"name":"snorunt","url":"https://pokeapi.co/api/v2/pokemon-species/361/"},{"name":"wynaut","url":"https://pokeapi.co/api/v2/pokemon-species/360/"},{"name":"absol","url":"https://pokeapi.co/api/v2/pokemon-species/359/"},{"name":"tropius","url":"https://pokeapi.co/api/v2/pokemon-species/357/"},{"name":"duskull","url":"https://pokeapi.co/api/v2/pokemon-species/355/"},{"name":"shuppet","url":"https://pokeapi.co/api/v2/pokemon-species/353/"},{"name":"kecleon","url":"https://pokeapi.co/api/v2/pokemon-species/352/"},{"name":"castform","url":"https://pokeapi.co/api/v2/pokemon-species/351/"},{"name":"feebas","url":"https://pokeapi.co/api/v2/pokemon-species/349/"},{"name":"anorith","url":"https://pokeapi.co/api/v2/pokemon-species/347/"},{"name":"lileep","url":"https://pokeapi.co/api/v2/pokemon-species/345/"},{"name":"baltoy","url":"https://pokeapi.co/api/v2/pokemon-species/343/"},{"name":"corphish","url":"https://pokeapi.co/api/v2/pokemon-species/341/"},{"name":"barboach","url":"https://pokeapi.co/api/v2/pokemon-species/339/"},{"name":"solrock","url":"https://pokeapi.co/api/v2/pokemon-species/338/"},{"name":"lunatone","url":"https://pokeapi.co/api/v2/pokemon-species/337/"},{"name":"seviper","url":"https://pokeapi.co/api/v2/pokemon-species/336/"},{"name":"zangoose","url":"https://pokeapi.co/api/v2/pokemon-species/335/"},{"name":"swablu","url":"https://pokeapi.co/api/v2/pokemon-species/333/"},{"name":"cacnea","url":"https://pokeapi.co/api/v2/pokemon-species/331/"},{"name":"trapinch","url":"https://pokeapi.co/api/v2/pokemon-species/328/"},{"name":"spinda","url":"https://pokeapi.co/api/v2/pokemon-species/327/"},{"name":"spoink","url":"https://pokeapi.co/api/v2/pokemon-species/325/"},{"name":"torkoal","url":"https://pokeapi.co/api/v2/pokemon-species/324/"},{"name":"numel","url":"https://pokeapi.co/api/v2/pokemon-species/322/"},{"name":"wailmer","url":"https://pokeapi.co/api/v2/pokemon-species/320/"},{"name":"carvanha","url":"https://pokeapi.co/api/v2/pokemon-species/318/"},{"name":"gulpin","url":"https://pokeapi.co/api/v2/pokemon-species/316/"},{"name":"illumise","url":"https://pokeapi.co/api/v2/pokemon-species/314/"},{"name":"volbeat","url":"https://pokeapi.co/api/v2/pokemon-species/313/"},{"name":"minun","url":"https://pokeapi.co/api/v2/pokemon-species/312/"},{"name":"plusle","url":"https://pokeapi.co/api/v2/pokemon-species/311/"},{"name":"electrike","url":"https://pokeapi.co/api/v2/pokemon-species/309/"},{"name":"meditite","url":"https://pokeapi.co/api/v2/pokemon-species/307/"},{"name":"aron","url":"https://pokeapi.co/api/v2/pokemon-species/304/"},{"name":"mawile","url":"https://pokeapi.co/api/v2/pokemon-species/303/"},{"name":"sableye","url":"https://pokeapi.co/api/v2/pokemon-species/302/"},{"name":"skitty","url":"https://pokeapi.co/api/v2/pokemon-species/300/"},{"name":"nosepass","url":"https://pokeapi.co/api/v2/pokemon-species/299/"},{"name":"azurill","url":"https://pokeapi.co/api/v2/pokemon-species/298/"},{"name":"makuhita","url":"https://pokeapi.co/api/v2/pokemon-species/296/"},{"name":"whismur","url":"https://pokeapi.co/api/v2/pokemon-species/293/"},{"name":"nincada","url":"https://pokeapi.co/api/v2/pokemon-species/290/"},{"name":"slakoth","url":"https://pokeapi.co/api/v2/pokemon-species/287/"},{"name":"shroomish","url":"https://pokeapi.co/api/v2/pokemon-species/285/"},{"name":"surskit","url":"https://pokeapi.co/api/v2/pokemon-species/283/"},{"name":"ralts","url":"https://pokeapi.co/api/v2/pokemon-species/280/"},{"name":"wingull","url":"https://pokeapi.co/api/v2/pokemon-species/278/"},{"name":"taillow","url":"https://pokeapi.co/api/v2/pokemon-species/276/"},{"name":"seedot","url":"https://pokeapi.co/api/v2/pokemon-species/273/"},{"name":"lotad","url":"https://pokeapi.co/api/v2/pokemon-species/270/"},{"name":"wurmple","url":"https://pokeapi.co/api/v2/pokemon-species/265/"},{"name":"zigzagoon","url":"https://pokeapi.co/api/v2/pokemon-species/263/"},{"name":"poochyena","url":"https://pokeapi.co/api/v2/pokemon-species/261/"},{"name":"mudkip","url":"https://pokeapi.co/api/v2/pokemon-species/258/"},{"name":"torchic","url":"https://pokeapi.co/api/v2/pokemon-species/255/"},{"name":"treecko","url":"https://pokeapi.co/api/v2/pokemon-species/252/"}],"types":[{"name":"shadow","url":"https://pokeapi.co/api/v2/type/10002/"}],"version_groups":[{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"},{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"},{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"},{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"},{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}]}"#;

#[test]
fn generation_from_id() {
    test_from_id::<Generation>(3, GENERATION_III_JSON);
}

#[test]
fn generation_from_name() {
    test_from_name::<Generation>("generation-iii", GENERATION_III_JSON);
}

// https://pokeapi.co/api/v2/pokedex/11/
static CONQUEST_GALLERY_JSON: &str = r#"{"descriptions":[{"description":"Pokémon Conquest gallery order","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"description":"Pokémon Conquest Galerie Reihenfolge","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"}},{"description":"Ordre de la Galerie dans Pokémon Conquest","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"}}],"id":11,"is_main_series":false,"name":"conquest-gallery","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Galerie"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Gallerie"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Gallery"}],"pokemon_entries":[{"entry_number":1,"pokemon_species":{"name":"eevee","url":"https://pokeapi.co/api/v2/pokemon-species/133/"}},{"entry_number":2,"pokemon_species":{"name":"vaporeon","url":"https://pokeapi.co/api/v2/pokemon-species/134/"}},{"entry_number":3,"pokemon_species":{"name":"jolteon","url":"https://pokeapi.co/api/v2/pokemon-species/135/"}},{"entry_number":4,"pokemon_species":{"name":"flareon","url":"https://pokeapi.co/api/v2/pokemon-species/136/"}},{"entry_number":5,"pokemon_species":{"name":"espeon","url":"https://pokeapi.co/api/v2/pokemon-species/196/"}},{"entry_number":6,"pokemon_species":{"name":"umbreon","url":"https://pokeapi.co/api/v2/pokemon-species/197/"}},{"entry_number":7,"pokemon_species":{"name":"leafeon","url":"https://pokeapi.co/api/v2/pokemon-species/470/"}},{"entry_number":8,"pokemon_species":{"name":"glaceon","url":"https://pokeapi.co/api/v2/pokemon-species/471/"}},{"entry_number":9,"pokemon_species":{"name":"ralts","url":"https://pokeapi.co/api/v2/pokemon-species/280/"}},{"entry_number":10,"pokemon_species":{"name":"kirlia","url":"https://pokeapi.co/api/v2/pokemon-species/281/"}},{"entry_number":11,"pokemon_species":{"name":"gardevoir","url":"https://pokeapi.co/api/v2/pokemon-species/282/"}},{"entry_number":12,"pokemon_species":{"name":"gallade","url":"https://pokeapi.co/api/v2/pokemon-species/475/"}},{"entry_number":13,"pokemon_species":{"name":"magikarp","url":"https://pokeapi.co/api/v2/pokemon-species/129/"}},{"entry_number":14,"pokemon_species":{"name":"gyarados","url":"https://pokeapi.co/api/v2/pokemon-species/130/"}},{"entry_number":15,"pokemon_species":{"name":"pichu","url":"https://pokeapi.co/api/v2/pokemon-species/172/"}},{"entry_number":16,"pokemon_species":{"name":"pikachu","url":"https://pokeapi.co/api/v2/pokemon-species/25/"}},{"entry_number":17,"pokemon_species":{"name":"raichu","url":"https://pokeapi.co/api/v2/pokemon-species/26/"}},{"entry_number":18,"pokemon_species":{"name":"wooper","url":"https://pokeapi.co/api/v2/pokemon-species/194/"}},{"entry_number":19,"pokemon_species":{"name":"quagsire","url":"https://pokeapi.co/api/v2/pokemon-species/195/"}},{"entry_number":20,"pokemon_species":{"name":"igglybuff","url":"https://pokeapi.co/api/v2/pokemon-species/174/"}},{"entry_number":21,"pokemon_species":{"name":"jigglypuff","url":"https://pokeapi.co/api/v2/pokemon-species/39/"}},{"entry_number":22,"pokemon_species":{"name":"wigglytuff","url":"https://pokeapi.co/api/v2/pokemon-species/40/"}},{"entry_number":23,"pokemon_species":{"name":"zubat","url":"https://pokeapi.co/api/v2/pokemon-species/41/"}},{"entry_number":24,"pokemon_species":{"name":"golbat","url":"https://pokeapi.co/api/v2/pokemon-species/42/"}},{"entry_number":25,"pokemon_species":{"name":"crobat","url":"https://pokeapi.co/api/v2/pokemon-species/169/"}},{"entry_number":26,"pokemon_species":{"name":"starly","url":"https://pokeapi.co/api/v2/pokemon-species/396/"}},{"entry_number":27,"pokemon_species":{"name":"staravia","url":"https://pokeapi.co/api/v2/pokemon-species/397/"}},{"entry_number":28,"pokemon_species":{"name":"staraptor","url":"https://pokeapi.co/api/v2/pokemon-species/398/"}},{"entry_number":29,"pokemon_species":{"name":"bidoof","url":"https://pokeapi.co/api/v2/pokemon-species/399/"}},{"entry_number":30,"pokemon_species":{"name":"bibarel","url":"https://pokeapi.co/api/v2/pokemon-species/400/"}},{"entry_number":31,"pokemon_species":{"name":"venipede","url":"https://pokeapi.co/api/v2/pokemon-species/543/"}},{"entry_number":32,"pokemon_species":{"name":"whirlipede","url":"https://pokeapi.co/api/v2/pokemon-species/544/"}},{"entry_number":33,"pokemon_species":{"name":"scolipede","url":"https://pokeapi.co/api/v2/pokemon-species/545/"}},{"entry_number":34,"pokemon_species":{"name":"shinx","url":"https://pokeapi.co/api/v2/pokemon-species/403/"}},{"entry_number":35,"pokemon_species":{"name":"luxio","url":"https://pokeapi.co/api/v2/pokemon-species/404/"}},{"entry_number":36,"pokemon_species":{"name":"luxray","url":"https://pokeapi.co/api/v2/pokemon-species/405/"}},{"entry_number":37,"pokemon_species":{"name":"litwick","url":"https://pokeapi.co/api/v2/pokemon-species/607/"}},{"entry_number":38,"pokemon_species":{"name":"lampent","url":"https://pokeapi.co/api/v2/pokemon-species/608/"}},{"entry_number":39,"pokemon_species":{"name":"chandelure","url":"https://pokeapi.co/api/v2/pokemon-species/609/"}},{"entry_number":40,"pokemon_species":{"name":"roggenrola","url":"https://pokeapi.co/api/v2/pokemon-species/524/"}},{"entry_number":41,"pokemon_species":{"name":"boldore","url":"https://pokeapi.co/api/v2/pokemon-species/525/"}},{"entry_number":42,"pokemon_species":{"name":"gigalith","url":"https://pokeapi.co/api/v2/pokemon-species/526/"}},{"entry_number":43,"pokemon_species":{"name":"petilil","url":"https://pokeapi.co/api/v2/pokemon-species/548/"}},{"entry_number":44,"pokemon_species":{"name":"lilligant","url":"https://pokeapi.co/api/v2/pokemon-species/549/"}},{"entry_number":45,"pokemon_species":{"name":"mareep","url":"https://pokeapi.co/api/v2/pokemon-species/179/"}},{"entry_number":46,"pokemon_species":{"name":"flaaffy","url":"https://pokeapi.co/api/v2/pokemon-species/180/"}},{"entry_number":47,"pokemon_species":{"name":"ampharos","url":"https://pokeapi.co/api/v2/pokemon-species/181/"}},{"entry_number":48,"pokemon_species":{"name":"cottonee","url":"https://pokeapi.co/api/v2/pokemon-species/546/"}},{"entry_number":49,"pokemon_species":{"name":"whimsicott","url":"https://pokeapi.co/api/v2/pokemon-species/547/"}},{"entry_number":50,"pokemon_species":{"name":"riolu","url":"https://pokeapi.co/api/v2/pokemon-species/447/"}},{"entry_number":51,"pokemon_species":{"name":"lucario","url":"https://pokeapi.co/api/v2/pokemon-species/448/"}},{"entry_number":52,"pokemon_species":{"name":"chingling","url":"https://pokeapi.co/api/v2/pokemon-species/433/"}},{"entry_number":53,"pokemon_species":{"name":"chimecho","url":"https://pokeapi.co/api/v2/pokemon-species/358/"}},{"entry_number":54,"pokemon_species":{"name":"ekans","url":"https://pokeapi.co/api/v2/pokemon-species/23/"}},{"entry_number":55,"pokemon_species":{"name":"arbok","url":"https://pokeapi.co/api/v2/pokemon-species/24/"}},{"entry_number":56,"pokemon_species":{"name":"pineco","url":"https://pokeapi.co/api/v2/pokemon-species/204/"}},{"entry_number":57,"pokemon_species":{"name":"forretress","url":"https://pokeapi.co/api/v2/pokemon-species/205/"}},{"entry_number":58,"pokemon_species":{"name":"meowth","url":"https://pokeapi.co/api/v2/pokemon-species/52/"}},{"entry_number":59,"pokemon_species":{"name":"persian","url":"https://pokeapi.co/api/v2/pokemon-species/53/"}},{"entry_number":60,"pokemon_species":{"name":"spheal","url":"https://pokeapi.co/api/v2/pokemon-species/363/"}},{"entry_number":61,"pokemon_species":{"name":"sealeo","url":"https://pokeapi.co/api/v2/pokemon-species/364/"}},{"entry_number":62,"pokemon_species":{"name":"walrein","url":"https://pokeapi.co/api/v2/pokemon-species/365/"}},{"entry_number":63,"pokemon_species":{"name":"gothita","url":"https://pokeapi.co/api/v2/pokemon-species/574/"}},{"entry_number":64,"pokemon_species":{"name":"gothorita","url":"https://pokeapi.co/api/v2/pokemon-species/575/"}},{"entry_number":65,"pokemon_species":{"name":"gothitelle","url":"https://pokeapi.co/api/v2/pokemon-species/576/"}},{"entry_number":66,"pokemon_species":{"name":"sandile","url":"https://pokeapi.co/api/v2/pokemon-species/551/"}},{"entry_number":67,"pokemon_species":{"name":"krokorok","url":"https://pokeapi.co/api/v2/pokemon-species/552/"}},{"entry_number":68,"pokemon_species":{"name":"krookodile","url":"https://pokeapi.co/api/v2/pokemon-species/553/"}},{"entry_number":69,"pokemon_species":{"name":"duskull","url":"https://pokeapi.co/api/v2/pokemon-species/355/"}},{"entry_number":70,"pokemon_species":{"name":"dusclops","url":"https://pokeapi.co/api/v2/pokemon-species/356/"}},{"entry_number":71,"pokemon_species":{"name":"dusknoir","url":"https://pokeapi.co/api/v2/pokemon-species/477/"}},{"entry_number":72,"pokemon_species":{"name":"munna","url":"https://pokeapi.co/api/v2/pokemon-species/517/"}},{"entry_number":73,"pokemon_species":{"name":"musharna","url":"https://pokeapi.co/api/v2/pokemon-species/518/"}},{"entry_number":74,"pokemon_species":{"name":"blitzle","url":"https://pokeapi.co/api/v2/pokemon-species/522/"}},{"entry_number":75,"pokemon_species":{"name":"zebstrika","url":"https://pokeapi.co/api/v2/pokemon-species/523/"}},{"entry_number":76,"pokemon_species":{"name":"dratini","url":"https://pokeapi.co/api/v2/pokemon-species/147/"}},{"entry_number":77,"pokemon_species":{"name":"dragonair","url":"https://pokeapi.co/api/v2/pokemon-species/148/"}},{"entry_number":78,"pokemon_species":{"name":"dragonite","url":"https://pokeapi.co/api/v2/pokemon-species/149/"}},{"entry_number":79,"pokemon_species":{"name":"larvitar","url":"https://pokeapi.co/api/v2/pokemon-species/246/"}},{"entry_number":80,"pokemon_species":{"name":"pupitar","url":"https://pokeapi.co/api/v2/pokemon-species/247/"}},{"entry_number":81,"pokemon_species":{"name":"tyranitar","url":"https://pokeapi.co/api/v2/pokemon-species/248/"}},{"entry_number":82,"pokemon_species":{"name":"beldum","url":"https://pokeapi.co/api/v2/pokemon-species/374/"}},{"entry_number":83,"pokemon_species":{"name":"metang","url":"https://pokeapi.co/api/v2/pokemon-species/375/"}},{"entry_number":84,"pokemon_species":{"name":"metagross","url":"https://pokeapi.co/api/v2/pokemon-species/376/"}},{"entry_number":85,"pokemon_species":{"name":"gible","url":"https://pokeapi.co/api/v2/pokemon-species/443/"}},{"entry_number":86,"pokemon_species":{"name":"gabite","url":"https://pokeapi.co/api/v2/pokemon-species/444/"}},{"entry_number":87,"pokemon_species":{"name":"garchomp","url":"https://pokeapi.co/api/v2/pokemon-species/445/"}},{"entry_number":88,"pokemon_species":{"name":"croagunk","url":"https://pokeapi.co/api/v2/pokemon-species/453/"}},{"entry_number":89,"pokemon_species":{"name":"toxicroak","url":"https://pokeapi.co/api/v2/pokemon-species/454/"}},{"entry_number":90,"pokemon_species":{"name":"deino","url":"https://pokeapi.co/api/v2/pokemon-species/633/"}},{"entry_number":91,"pokemon_species":{"name":"zweilous","url":"https://pokeapi.co/api/v2/pokemon-species/634/"}},{"entry_number":92,"pokemon_species":{"name":"hydreigon","url":"https://pokeapi.co/api/v2/pokemon-species/635/"}},{"entry_number":93,"pokemon_species":{"name":"snorunt","url":"https://pokeapi.co/api/v2/pokemon-species/361/"}},{"entry_number":94,"pokemon_species":{"name":"glalie","url":"https://pokeapi.co/api/v2/pokemon-species/362/"}},{"entry_number":95,"pokemon_species":{"name":"froslass","url":"https://pokeapi.co/api/v2/pokemon-species/478/"}},{"entry_number":96,"pokemon_species":{"name":"minccino","url":"https://pokeapi.co/api/v2/pokemon-species/572/"}},{"entry_number":97,"pokemon_species":{"name":"cinccino","url":"https://pokeapi.co/api/v2/pokemon-species/573/"}},{"entry_number":98,"pokemon_species":{"name":"machop","url":"https://pokeapi.co/api/v2/pokemon-species/66/"}},{"entry_number":99,"pokemon_species":{"name":"machoke","url":"https://pokeapi.co/api/v2/pokemon-species/67/"}},{"entry_number":100,"pokemon_species":{"name":"machamp","url":"https://pokeapi.co/api/v2/pokemon-species/68/"}},{"entry_number":101,"pokemon_species":{"name":"timburr","url":"https://pokeapi.co/api/v2/pokemon-species/532/"}},{"entry_number":102,"pokemon_species":{"name":"gurdurr","url":"https://pokeapi.co/api/v2/pokemon-species/533/"}},{"entry_number":103,"pokemon_species":{"name":"conkeldurr","url":"https://pokeapi.co/api/v2/pokemon-species/534/"}},{"entry_number":104,"pokemon_species":{"name":"cubchoo","url":"https://pokeapi.co/api/v2/pokemon-species/613/"}},{"entry_number":105,"pokemon_species":{"name":"beartic","url":"https://pokeapi.co/api/v2/pokemon-species/614/"}},{"entry_number":106,"pokemon_species":{"name":"oshawott","url":"https://pokeapi.co/api/v2/pokemon-species/501/"}},{"entry_number":107,"pokemon_species":{"name":"dewott","url":"https://pokeapi.co/api/v2/pokemon-species/502/"}},{"entry_number":108,"pokemon_species":{"name":"samurott","url":"https://pokeapi.co/api/v2/pokemon-species/503/"}},{"entry_number":109,"pokemon_species":{"name":"charmander","url":"https://pokeapi.co/api/v2/pokemon-species/4/"}},{"entry_number":110,"pokemon_species":{"name":"charmeleon","url":"https://pokeapi.co/api/v2/pokemon-species/5/"}},{"entry_number":111,"pokemon_species":{"name":"charizard","url":"https://pokeapi.co/api/v2/pokemon-species/6/"}},{"entry_number":112,"pokemon_species":{"name":"gastly","url":"https://pokeapi.co/api/v2/pokemon-species/92/"}},{"entry_number":113,"pokemon_species":{"name":"haunter","url":"https://pokeapi.co/api/v2/pokemon-species/93/"}},{"entry_number":114,"pokemon_species":{"name":"gengar","url":"https://pokeapi.co/api/v2/pokemon-species/94/"}},{"entry_number":115,"pokemon_species":{"name":"chimchar","url":"https://pokeapi.co/api/v2/pokemon-species/390/"}},{"entry_number":116,"pokemon_species":{"name":"monferno","url":"https://pokeapi.co/api/v2/pokemon-species/391/"}},{"entry_number":117,"pokemon_species":{"name":"infernape","url":"https://pokeapi.co/api/v2/pokemon-species/392/"}},{"entry_number":118,"pokemon_species":{"name":"snivy","url":"https://pokeapi.co/api/v2/pokemon-species/495/"}},{"entry_number":119,"pokemon_species":{"name":"servine","url":"https://pokeapi.co/api/v2/pokemon-species/496/"}},{"entry_number":120,"pokemon_species":{"name":"serperior","url":"https://pokeapi.co/api/v2/pokemon-species/497/"}},{"entry_number":121,"pokemon_species":{"name":"tepig","url":"https://pokeapi.co/api/v2/pokemon-species/498/"}},{"entry_number":122,"pokemon_species":{"name":"pignite","url":"https://pokeapi.co/api/v2/pokemon-species/499/"}},{"entry_number":123,"pokemon_species":{"name":"emboar","url":"https://pokeapi.co/api/v2/pokemon-species/500/"}},{"entry_number":124,"pokemon_species":{"name":"sewaddle","url":"https://pokeapi.co/api/v2/pokemon-species/540/"}},{"entry_number":125,"pokemon_species":{"name":"swadloon","url":"https://pokeapi.co/api/v2/pokemon-species/541/"}},{"entry_number":126,"pokemon_species":{"name":"leavanny","url":"https://pokeapi.co/api/v2/pokemon-species/542/"}},{"entry_number":127,"pokemon_species":{"name":"abra","url":"https://pokeapi.co/api/v2/pokemon-species/63/"}},{"entry_number":128,"pokemon_species":{"name":"kadabra","url":"https://pokeapi.co/api/v2/pokemon-species/64/"}},{"entry_number":129,"pokemon_species":{"name":"alakazam","url":"https://pokeapi.co/api/v2/pokemon-species/65/"}},{"entry_number":130,"pokemon_species":{"name":"treecko","url":"https://pokeapi.co/api/v2/pokemon-species/252/"}},{"entry_number":131,"pokemon_species":{"name":"grovyle","url":"https://pokeapi.co/api/v2/pokemon-species/253/"}},{"entry_number":132,"pokemon_species":{"name":"sceptile","url":"https://pokeapi.co/api/v2/pokemon-species/254/"}},{"entry_number":133,"pokemon_species":{"name":"piplup","url":"https://pokeapi.co/api/v2/pokemon-species/393/"}},{"entry_number":134,"pokemon_species":{"name":"prinplup","url":"https://pokeapi.co/api/v2/pokemon-species/394/"}},{"entry_number":135,"pokemon_species":{"name":"empoleon","url":"https://pokeapi.co/api/v2/pokemon-species/395/"}},{"entry_number":136,"pokemon_species":{"name":"pansage","url":"https://pokeapi.co/api/v2/pokemon-species/511/"}},{"entry_number":137,"pokemon_species":{"name":"simisage","url":"https://pokeapi.co/api/v2/pokemon-species/512/"}},{"entry_number":138,"pokemon_species":{"name":"pansear","url":"https://pokeapi.co/api/v2/pokemon-species/513/"}},{"entry_number":139,"pokemon_species":{"name":"simisear","url":"https://pokeapi.co/api/v2/pokemon-species/514/"}},{"entry_number":140,"pokemon_species":{"name":"panpour","url":"https://pokeapi.co/api/v2/pokemon-species/515/"}},{"entry_number":141,"pokemon_species":{"name":"simipour","url":"https://pokeapi.co/api/v2/pokemon-species/516/"}},{"entry_number":142,"pokemon_species":{"name":"darumaka","url":"https://pokeapi.co/api/v2/pokemon-species/554/"}},{"entry_number":143,"pokemon_species":{"name":"darmanitan","url":"https://pokeapi.co/api/v2/pokemon-species/555/"}},{"entry_number":144,"pokemon_species":{"name":"axew","url":"https://pokeapi.co/api/v2/pokemon-species/610/"}},{"entry_number":145,"pokemon_species":{"name":"fraxure","url":"https://pokeapi.co/api/v2/pokemon-species/611/"}},{"entry_number":146,"pokemon_species":{"name":"haxorus","url":"https://pokeapi.co/api/v2/pokemon-species/612/"}},{"entry_number":147,"pokemon_species":{"name":"joltik","url":"https://pokeapi.co/api/v2/pokemon-species/595/"}},{"entry_number":148,"pokemon_species":{"name":"galvantula","url":"https://pokeapi.co/api/v2/pokemon-species/596/"}},{"entry_number":149,"pokemon_species":{"name":"aron","url":"https://pokeapi.co/api/v2/pokemon-species/304/"}},{"entry_number":150,"pokemon_species":{"name":"lairon","url":"https://pokeapi.co/api/v2/pokemon-species/305/"}},{"entry_number":151,"pokemon_species":{"name":"aggron","url":"https://pokeapi.co/api/v2/pokemon-species/306/"}},{"entry_number":152,"pokemon_species":{"name":"drilbur","url":"https://pokeapi.co/api/v2/pokemon-species/529/"}},{"entry_number":153,"pokemon_species":{"name":"excadrill","url":"https://pokeapi.co/api/v2/pokemon-species/530/"}},{"entry_number":154,"pokemon_species":{"name":"zorua","url":"https://pokeapi.co/api/v2/pokemon-species/570/"}},{"entry_number":155,"pokemon_species":{"name":"zoroark","url":"https://pokeapi.co/api/v2/pokemon-species/571/"}},{"entry_number":156,"pokemon_species":{"name":"skorupi","url":"https://pokeapi.co/api/v2/pokemon-species/451/"}},{"entry_number":157,"pokemon_species":{"name":"drapion","url":"https://pokeapi.co/api/v2/pokemon-species/452/"}},{"entry_number":158,"pokemon_species":{"name":"pawniard","url":"https://pokeapi.co/api/v2/pokemon-species/624/"}},{"entry_number":159,"pokemon_species":{"name":"bisharp","url":"https://pokeapi.co/api/v2/pokemon-species/625/"}},{"entry_number":160,"pokemon_species":{"name":"rhyhorn","url":"https://pokeapi.co/api/v2/pokemon-species/111/"}},{"entry_number":161,"pokemon_species":{"name":"rhydon","url":"https://pokeapi.co/api/v2/pokemon-species/112/"}},{"entry_number":162,"pokemon_species":{"name":"rhyperior","url":"https://pokeapi.co/api/v2/pokemon-species/464/"}},{"entry_number":163,"pokemon_species":{"name":"shieldon","url":"https://pokeapi.co/api/v2/pokemon-species/410/"}},{"entry_number":164,"pokemon_species":{"name":"bastiodon","url":"https://pokeapi.co/api/v2/pokemon-species/411/"}},{"entry_number":165,"pokemon_species":{"name":"scraggy","url":"https://pokeapi.co/api/v2/pokemon-species/559/"}},{"entry_number":166,"pokemon_species":{"name":"scrafty","url":"https://pokeapi.co/api/v2/pokemon-species/560/"}},{"entry_number":167,"pokemon_species":{"name":"drifloon","url":"https://pokeapi.co/api/v2/pokemon-species/425/"}},{"entry_number":168,"pokemon_species":{"name":"drifblim","url":"https://pokeapi.co/api/v2/pokemon-species/426/"}},{"entry_number":169,"pokemon_species":{"name":"rufflet","url":"https://pokeapi.co/api/v2/pokemon-species/627/"}},{"entry_number":170,"pokemon_species":{"name":"braviary","url":"https://pokeapi.co/api/v2/pokemon-species/628/"}},{"entry_number":171,"pokemon_species":{"name":"anorith","url":"https://pokeapi.co/api/v2/pokemon-species/347/"}},{"entry_number":172,"pokemon_species":{"name":"armaldo","url":"https://pokeapi.co/api/v2/pokemon-species/348/"}},{"entry_number":173,"pokemon_species":{"name":"larvesta","url":"https://pokeapi.co/api/v2/pokemon-species/636/"}},{"entry_number":174,"pokemon_species":{"name":"volcarona","url":"https://pokeapi.co/api/v2/pokemon-species/637/"}},{"entry_number":175,"pokemon_species":{"name":"onix","url":"https://pokeapi.co/api/v2/pokemon-species/95/"}},{"entry_number":176,"pokemon_species":{"name":"steelix","url":"https://pokeapi.co/api/v2/pokemon-species/208/"}},{"entry_number":177,"pokemon_species":{"name":"beedrill","url":"https://pokeapi.co/api/v2/pokemon-species/15/"}},{"entry_number":178,"pokemon_species":{"name":"munchlax","url":"https://pokeapi.co/api/v2/pokemon-species/446/"}},{"entry_number":179,"pokemon_species":{"name":"snorlax","url":"https://pokeapi.co/api/v2/pokemon-species/143/"}},{"entry_number":180,"pokemon_species":{"name":"emolga","url":"https://pokeapi.co/api/v2/pokemon-species/587/"}},{"entry_number":181,"pokemon_species":{"name":"sneasel","url":"https://pokeapi.co/api/v2/pokemon-species/215/"}},{"entry_number":182,"pokemon_species":{"name":"weavile","url":"https://pokeapi.co/api/v2/pokemon-species/461/"}},{"entry_number":183,"pokemon_species":{"name":"misdreavus","url":"https://pokeapi.co/api/v2/pokemon-species/200/"}},{"entry_number":184,"pokemon_species":{"name":"mismagius","url":"https://pokeapi.co/api/v2/pokemon-species/429/"}},{"entry_number":185,"pokemon_species":{"name":"audino","url":"https://pokeapi.co/api/v2/pokemon-species/531/"}},{"entry_number":186,"pokemon_species":{"name":"carnivine","url":"https://pokeapi.co/api/v2/pokemon-species/455/"}},{"entry_number":187,"pokemon_species":{"name":"spiritomb","url":"https://pokeapi.co/api/v2/pokemon-species/442/"}},{"entry_number":188,"pokemon_species":{"name":"scyther","url":"https://pokeapi.co/api/v2/pokemon-species/123/"}},{"entry_number":189,"pokemon_species":{"name":"scizor","url":"https://pokeapi.co/api/v2/pokemon-species/212/"}},{"entry_number":190,"pokemon_species":{"name":"lapras","url":"https://pokeapi.co/api/v2/pokemon-species/131/"}},{"entry_number":191,"pokemon_species":{"name":"terrakion","url":"https://pokeapi.co/api/v2/pokemon-species/639/"}},{"entry_number":192,"pokemon_species":{"name":"articuno","url":"https://pokeapi.co/api/v2/pokemon-species/144/"}},{"entry_number":193,"pokemon_species":{"name":"registeel","url":"https://pokeapi.co/api/v2/pokemon-species/379/"}},{"entry_number":194,"pokemon_species":{"name":"groudon","url":"https://pokeapi.co/api/v2/pokemon-species/383/"}},{"entry_number":195,"pokemon_species":{"name":"dialga","url":"https://pokeapi.co/api/v2/pokemon-species/483/"}},{"entry_number":196,"pokemon_species":{"name":"mewtwo","url":"https://pokeapi.co/api/v2/pokemon-species/150/"}},{"entry_number":197,"pokemon_species":{"name":"reshiram","url":"https://pokeapi.co/api/v2/pokemon-species/643/"}},{"entry_number":198,"pokemon_species":{"name":"zekrom","url":"https://pokeapi.co/api/v2/pokemon-species/644/"}},{"entry_number":199,"pokemon_species":{"name":"arceus","url":"https://pokeapi.co/api/v2/pokemon-species/493/"}},{"entry_number":200,"pokemon_species":{"name":"rayquaza","url":"https://pokeapi.co/api/v2/pokemon-species/384/"}}],"region":null,"version_groups":[]}"#;

#[test]
fn pokedex_from_id() {
    test_from_id::<Pokedex>(11, CONQUEST_GALLERY_JSON);
}

#[test]
fn pokedex_from_name() {
    test_from_name::<Pokedex>("conquest-gallery", CONQUEST_GALLERY_JSON);
}

// https://pokeapi.co/api/v2/version/9/
static EMERALD_JSON: &str = r#"{"id":9,"name":"emerald","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"エメラルド"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Émeraude"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Smaragd"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Smeraldo"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Emerald"}],"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}}"#;

#[test]
fn version_from_id() {
    test_from_id::<Version>(9, EMERALD_JSON);
}

#[test]
fn version_from_name() {
    test_from_name::<Version>("emerald", EMERALD_JSON);
}

// https://pokeapi.co/api/v2/version-group/15/
static X_Y_JSON: &str = r#"{"generation":{"name":"generation-vi","url":"https://pokeapi.co/api/v2/generation/6/"},"id":15,"move_learn_methods":[{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},{"name":"egg","url":"https://pokeapi.co/api/v2/move-learn-method/2/"},{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},{"name":"light-ball-egg","url":"https://pokeapi.co/api/v2/move-learn-method/6/"},{"name":"form-change","url":"https://pokeapi.co/api/v2/move-learn-method/10/"}],"name":"x-y","order":15,"pokedexes":[{"name":"kalos-central","url":"https://pokeapi.co/api/v2/pokedex/12/"},{"name":"kalos-coastal","url":"https://pokeapi.co/api/v2/pokedex/13/"},{"name":"kalos-mountain","url":"https://pokeapi.co/api/v2/pokedex/14/"}],"regions":[{"name":"kalos","url":"https://pokeapi.co/api/v2/region/6/"}],"versions":[{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"},{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}]}"#;

#[test]
fn version_group_from_id() {
    test_from_id::<VersionGroup>(15, X_Y_JSON);
}

#[test]
fn version_group_from_name() {
    test_from_name::<VersionGroup>("x-y", X_Y_JSON);
}

// https://pokeapi.co/api/v2/item/655/
static RESET_URGE_JSON: &str = r#"{"attributes":[],"baby_trigger_for":null,"category":{"name":"miracle-shooter","url":"https://pokeapi.co/api/v2/item-category/43/"},"cost":0,"effect_entries":[{"effect":"Used on a party Pokémon in battle\n:   Selects another friendly Pokémon at random.  Removes all of that Pokémon's stat changes.\n\nThis item can only be obtained or used via the Wonder Launcher.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"short_effect":"Resets a friendly Pokémon's stat changes."}],"flavor_text_entries":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"text":"使用后，我方宝可梦的\n能力变化将消失。","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"text":"使うと　味方の　ポケモンの\n能力変化が　元に　戻る。\n","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"text":"When used, it restores any stat changes of an\nally Pokémon.","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"text":"Usandolo, tutte le modifiche alle statistiche di un\nPokémon della squadra saranno annullate.","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"text":"Cuando se usa, se anulan los cambios en las\ncaracterísticas de un Pokémon de tu equipo.","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"text":"Hebt bei Gebrauch Statusveränderungen deines\nMitstreiters auf.","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"text":"Permet d’annuler les variations de stats d’un\nPokémon de l’équipe.","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"text":"使用後，我方寶可夢的\n能力變化將復原。","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"text":"사용하면 같은 편 포켓몬의\n능력 변화가 원래대로 되돌아간다.","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"text":"つかうと　みかたの　ポケモンの\nのうりょく　へんかが　もとにもどる。\n","version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"text":"使用后，我方宝可梦的\n能力变化将消失。","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"text":"使うと　味方の　ポケモンの\n能力変化が　元に　戻る。\n","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"text":"When used, it restores any stat changes of an\nally Pokémon.","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"text":"Usandolo, tutte le modifiche alle statistiche di un\nPokémon della squadra saranno annullate.","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"text":"Cuando se usa, se anulan los cambios en las\ncaracterísticas de un Pokémon de tu equipo.","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"text":"Hebt bei Gebrauch Statusveränderungen deines\nMitstreiters auf.","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"text":"Permet d’annuler les variations de stats d’un\nPokémon de l’équipe.","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"text":"使用後，我方寶可夢的\n能力變化將復原。","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"text":"사용하면 같은 편 포켓몬의\n능력 변화가 원래대로 되돌아간다.","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"text":"つかうと　みかたの　ポケモンの\nのうりょく　へんかが　もとにもどる。\n","version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"text":"使うと　味方の　ポケモンの\n能力変化が　元に　戻る。\n","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"text":"When used, it restores any stat\nchanges of an ally Pokémon.","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"text":"Usandolo, tutte le modifiche alle statistiche\ndi un Pokémon della squadra saranno annullate.","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"text":"Tarjeta necesaria para introducir la contraseña en\nla Fragata Plasma.","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"text":"Hebt bei Gebrauch Statusveränderungen\ndeines Mitstreiters auf.","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"text":"Permet d’annuler les variations de stats d’un\nPokémon de l’équipe.","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"text":"사용하면 같은 편 포켓몬의\n능력 변화가 원래대로 되돌아간다.","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"text":"つかうと　みかたの　ポケモンの\nのうりょく　へんかが　もとにもどる。\n","version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"text":"When used, it restores any stat\nchanges of an ally Pokémon.","version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"text":"When used, it restores any stat\nchanges of an ally Pokémon.","version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"text":"Permet d’annuler les variations de\nstats d’un Pokémon de l’équipe.","version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}],"fling_effect":null,"fling_power":null,"game_indices":[{"game_index":614,"generation":{"name":"generation-vii","url":"https://pokeapi.co/api/v2/generation/7/"}},{"game_index":614,"generation":{"name":"generation-vi","url":"https://pokeapi.co/api/v2/generation/6/"}},{"game_index":614,"generation":{"name":"generation-v","url":"https://pokeapi.co/api/v2/generation/5/"}}],"held_by_pokemon":[],"id":655,"machines":[],"name":"reset-urge","names":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"name":"？？？"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"フラットコール"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Reset Urge"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Ripristino"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Quitaestado"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Umkehrappell"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Réamorçage"},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"name":"？？？"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"플랫콜"},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"フラットコール"}],"sprites":{"default":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/items/reset-urge.png"}}"#;

#[test]
fn item_from_id() {
    test_from_id::<Item>(655, RESET_URGE_JSON);
}

#[test]
fn item_from_name() {
    test_from_name::<Item>("reset-urge", RESET_URGE_JSON);
}

// https://pokeapi.co/api/v2/item-attribute/5/
static HOLDABLE_JSON: &str = r#"{"descriptions":[{"description":"Can be held by a Pokémon","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"id":5,"items":[{"name":"master-ball","url":"https://pokeapi.co/api/v2/item/1/"},{"name":"ultra-ball","url":"https://pokeapi.co/api/v2/item/2/"},{"name":"great-ball","url":"https://pokeapi.co/api/v2/item/3/"},{"name":"poke-ball","url":"https://pokeapi.co/api/v2/item/4/"},{"name":"safari-ball","url":"https://pokeapi.co/api/v2/item/5/"},{"name":"net-ball","url":"https://pokeapi.co/api/v2/item/6/"},{"name":"dive-ball","url":"https://pokeapi.co/api/v2/item/7/"},{"name":"nest-ball","url":"https://pokeapi.co/api/v2/item/8/"},{"name":"repeat-ball","url":"https://pokeapi.co/api/v2/item/9/"},{"name":"timer-ball","url":"https://pokeapi.co/api/v2/item/10/"},{"name":"luxury-ball","url":"https://pokeapi.co/api/v2/item/11/"},{"name":"premier-ball","url":"https://pokeapi.co/api/v2/item/12/"},{"name":"dusk-ball","url":"https://pokeapi.co/api/v2/item/13/"},{"name":"heal-ball","url":"https://pokeapi.co/api/v2/item/14/"},{"name":"quick-ball","url":"https://pokeapi.co/api/v2/item/15/"},{"name":"cherish-ball","url":"https://pokeapi.co/api/v2/item/16/"},{"name":"potion","url":"https://pokeapi.co/api/v2/item/17/"},{"name":"antidote","url":"https://pokeapi.co/api/v2/item/18/"},{"name":"burn-heal","url":"https://pokeapi.co/api/v2/item/19/"},{"name":"ice-heal","url":"https://pokeapi.co/api/v2/item/20/"},{"name":"awakening","url":"https://pokeapi.co/api/v2/item/21/"},{"name":"paralyze-heal","url":"https://pokeapi.co/api/v2/item/22/"},{"name":"full-restore","url":"https://pokeapi.co/api/v2/item/23/"},{"name":"max-potion","url":"https://pokeapi.co/api/v2/item/24/"},{"name":"hyper-potion","url":"https://pokeapi.co/api/v2/item/25/"},{"name":"super-potion","url":"https://pokeapi.co/api/v2/item/26/"},{"name":"full-heal","url":"https://pokeapi.co/api/v2/item/27/"},{"name":"revive","url":"https://pokeapi.co/api/v2/item/28/"},{"name":"max-revive","url":"https://pokeapi.co/api/v2/item/29/"},{"name":"fresh-water","url":"https://pokeapi.co/api/v2/item/30/"},{"name":"soda-pop","url":"https://pokeapi.co/api/v2/item/31/"},{"name":"lemonade","url":"https://pokeapi.co/api/v2/item/32/"},{"name":"moomoo-milk","url":"https://pokeapi.co/api/v2/item/33/"},{"name":"energy-powder","url":"https://pokeapi.co/api/v2/item/34/"},{"name":"energy-root","url":"https://pokeapi.co/api/v2/item/35/"},{"name":"heal-powder","url":"https://pokeapi.co/api/v2/item/36/"},{"name":"revival-herb","url":"https://pokeapi.co/api/v2/item/37/"},{"name":"ether","url":"https://pokeapi.co/api/v2/item/38/"},{"name":"max-ether","url":"https://pokeapi.co/api/v2/item/39/"},{"name":"elixir","url":"https://pokeapi.co/api/v2/item/40/"},{"name":"max-elixir","url":"https://pokeapi.co/api/v2/item/41/"},{"name":"lava-cookie","url":"https://pokeapi.co/api/v2/item/42/"},{"name":"berry-juice","url":"https://pokeapi.co/api/v2/item/43/"},{"name":"sacred-ash","url":"https://pokeapi.co/api/v2/item/44/"},{"name":"hp-up","url":"https://pokeapi.co/api/v2/item/45/"},{"name":"protein","url":"https://pokeapi.co/api/v2/item/46/"},{"name":"iron","url":"https://pokeapi.co/api/v2/item/47/"},{"name":"carbos","url":"https://pokeapi.co/api/v2/item/48/"},{"name":"calcium","url":"https://pokeapi.co/api/v2/item/49/"},{"name":"rare-candy","url":"https://pokeapi.co/api/v2/item/50/"},{"name":"pp-up","url":"https://pokeapi.co/api/v2/item/51/"},{"name":"zinc","url":"https://pokeapi.co/api/v2/item/52/"},{"name":"pp-max","url":"https://pokeapi.co/api/v2/item/53/"},{"name":"old-gateau","url":"https://pokeapi.co/api/v2/item/54/"},{"name":"guard-spec","url":"https://pokeapi.co/api/v2/item/55/"},{"name":"dire-hit","url":"https://pokeapi.co/api/v2/item/56/"},{"name":"x-attack","url":"https://pokeapi.co/api/v2/item/57/"},{"name":"x-defense","url":"https://pokeapi.co/api/v2/item/58/"},{"name":"x-speed","url":"https://pokeapi.co/api/v2/item/59/"},{"name":"x-accuracy","url":"https://pokeapi.co/api/v2/item/60/"},{"name":"x-sp-atk","url":"https://pokeapi.co/api/v2/item/61/"},{"name":"x-sp-def","url":"https://pokeapi.co/api/v2/item/62/"},{"name":"poke-doll","url":"https://pokeapi.co/api/v2/item/63/"},{"name":"fluffy-tail","url":"https://pokeapi.co/api/v2/item/64/"},{"name":"blue-flute","url":"https://pokeapi.co/api/v2/item/65/"},{"name":"yellow-flute","url":"https://pokeapi.co/api/v2/item/66/"},{"name":"red-flute","url":"https://pokeapi.co/api/v2/item/67/"},{"name":"black-flute","url":"https://pokeapi.co/api/v2/item/68/"},{"name":"white-flute","url":"https://pokeapi.co/api/v2/item/69/"},{"name":"adamant-orb","url":"https://pokeapi.co/api/v2/item/112/"},{"name":"lustrous-orb","url":"https://pokeapi.co/api/v2/item/113/"},{"name":"bright-powder","url":"https://pokeapi.co/api/v2/item/190/"},{"name":"white-herb","url":"https://pokeapi.co/api/v2/item/191/"},{"name":"macho-brace","url":"https://pokeapi.co/api/v2/item/192/"},{"name":"exp-share","url":"https://pokeapi.co/api/v2/item/193/"},{"name":"quick-claw","url":"https://pokeapi.co/api/v2/item/194/"},{"name":"soothe-bell","url":"https://pokeapi.co/api/v2/item/195/"},{"name":"mental-herb","url":"https://pokeapi.co/api/v2/item/196/"},{"name":"choice-band","url":"https://pokeapi.co/api/v2/item/197/"},{"name":"kings-rock","url":"https://pokeapi.co/api/v2/item/198/"},{"name":"silver-powder","url":"https://pokeapi.co/api/v2/item/199/"},{"name":"amulet-coin","url":"https://pokeapi.co/api/v2/item/200/"},{"name":"cleanse-tag","url":"https://pokeapi.co/api/v2/item/201/"},{"name":"soul-dew","url":"https://pokeapi.co/api/v2/item/202/"},{"name":"deep-sea-tooth","url":"https://pokeapi.co/api/v2/item/203/"},{"name":"deep-sea-scale","url":"https://pokeapi.co/api/v2/item/204/"},{"name":"smoke-ball","url":"https://pokeapi.co/api/v2/item/205/"},{"name":"everstone","url":"https://pokeapi.co/api/v2/item/206/"},{"name":"focus-band","url":"https://pokeapi.co/api/v2/item/207/"},{"name":"lucky-egg","url":"https://pokeapi.co/api/v2/item/208/"},{"name":"scope-lens","url":"https://pokeapi.co/api/v2/item/209/"},{"name":"metal-coat","url":"https://pokeapi.co/api/v2/item/210/"},{"name":"leftovers","url":"https://pokeapi.co/api/v2/item/211/"},{"name":"light-ball","url":"https://pokeapi.co/api/v2/item/213/"},{"name":"soft-sand","url":"https://pokeapi.co/api/v2/item/214/"},{"name":"hard-stone","url":"https://pokeapi.co/api/v2/item/215/"},{"name":"miracle-seed","url":"https://pokeapi.co/api/v2/item/216/"},{"name":"black-glasses","url":"https://pokeapi.co/api/v2/item/217/"},{"name":"black-belt","url":"https://pokeapi.co/api/v2/item/218/"},{"name":"magnet","url":"https://pokeapi.co/api/v2/item/219/"},{"name":"mystic-water","url":"https://pokeapi.co/api/v2/item/220/"},{"name":"sharp-beak","url":"https://pokeapi.co/api/v2/item/221/"},{"name":"poison-barb","url":"https://pokeapi.co/api/v2/item/222/"},{"name":"never-melt-ice","url":"https://pokeapi.co/api/v2/item/223/"},{"name":"spell-tag","url":"https://pokeapi.co/api/v2/item/224/"},{"name":"twisted-spoon","url":"https://pokeapi.co/api/v2/item/225/"},{"name":"charcoal","url":"https://pokeapi.co/api/v2/item/226/"},{"name":"dragon-fang","url":"https://pokeapi.co/api/v2/item/227/"},{"name":"silk-scarf","url":"https://pokeapi.co/api/v2/item/228/"},{"name":"shell-bell","url":"https://pokeapi.co/api/v2/item/230/"},{"name":"sea-incense","url":"https://pokeapi.co/api/v2/item/231/"},{"name":"lax-incense","url":"https://pokeapi.co/api/v2/item/232/"},{"name":"lucky-punch","url":"https://pokeapi.co/api/v2/item/233/"},{"name":"metal-powder","url":"https://pokeapi.co/api/v2/item/234/"},{"name":"thick-club","url":"https://pokeapi.co/api/v2/item/235/"},{"name":"stick","url":"https://pokeapi.co/api/v2/item/236/"},{"name":"wide-lens","url":"https://pokeapi.co/api/v2/item/242/"},{"name":"muscle-band","url":"https://pokeapi.co/api/v2/item/243/"},{"name":"wise-glasses","url":"https://pokeapi.co/api/v2/item/244/"},{"name":"expert-belt","url":"https://pokeapi.co/api/v2/item/245/"},{"name":"light-clay","url":"https://pokeapi.co/api/v2/item/246/"},{"name":"life-orb","url":"https://pokeapi.co/api/v2/item/247/"},{"name":"power-herb","url":"https://pokeapi.co/api/v2/item/248/"},{"name":"toxic-orb","url":"https://pokeapi.co/api/v2/item/249/"},{"name":"flame-orb","url":"https://pokeapi.co/api/v2/item/250/"},{"name":"quick-powder","url":"https://pokeapi.co/api/v2/item/251/"},{"name":"focus-sash","url":"https://pokeapi.co/api/v2/item/252/"},{"name":"zoom-lens","url":"https://pokeapi.co/api/v2/item/253/"},{"name":"metronome","url":"https://pokeapi.co/api/v2/item/254/"},{"name":"iron-ball","url":"https://pokeapi.co/api/v2/item/255/"},{"name":"lagging-tail","url":"https://pokeapi.co/api/v2/item/256/"},{"name":"destiny-knot","url":"https://pokeapi.co/api/v2/item/257/"},{"name":"black-sludge","url":"https://pokeapi.co/api/v2/item/258/"},{"name":"icy-rock","url":"https://pokeapi.co/api/v2/item/259/"},{"name":"smooth-rock","url":"https://pokeapi.co/api/v2/item/260/"},{"name":"heat-rock","url":"https://pokeapi.co/api/v2/item/261/"},{"name":"damp-rock","url":"https://pokeapi.co/api/v2/item/262/"},{"name":"grip-claw","url":"https://pokeapi.co/api/v2/item/263/"},{"name":"choice-scarf","url":"https://pokeapi.co/api/v2/item/264/"},{"name":"sticky-barb","url":"https://pokeapi.co/api/v2/item/265/"},{"name":"power-bracer","url":"https://pokeapi.co/api/v2/item/266/"},{"name":"power-belt","url":"https://pokeapi.co/api/v2/item/267/"},{"name":"power-lens","url":"https://pokeapi.co/api/v2/item/268/"},{"name":"power-band","url":"https://pokeapi.co/api/v2/item/269/"},{"name":"power-anklet","url":"https://pokeapi.co/api/v2/item/270/"},{"name":"power-weight","url":"https://pokeapi.co/api/v2/item/271/"},{"name":"shed-shell","url":"https://pokeapi.co/api/v2/item/272/"},{"name":"big-root","url":"https://pokeapi.co/api/v2/item/273/"},{"name":"choice-specs","url":"https://pokeapi.co/api/v2/item/274/"},{"name":"flame-plate","url":"https://pokeapi.co/api/v2/item/275/"},{"name":"splash-plate","url":"https://pokeapi.co/api/v2/item/276/"},{"name":"zap-plate","url":"https://pokeapi.co/api/v2/item/277/"},{"name":"meadow-plate","url":"https://pokeapi.co/api/v2/item/278/"},{"name":"icicle-plate","url":"https://pokeapi.co/api/v2/item/279/"},{"name":"fist-plate","url":"https://pokeapi.co/api/v2/item/280/"},{"name":"toxic-plate","url":"https://pokeapi.co/api/v2/item/281/"},{"name":"earth-plate","url":"https://pokeapi.co/api/v2/item/282/"},{"name":"sky-plate","url":"https://pokeapi.co/api/v2/item/283/"},{"name":"mind-plate","url":"https://pokeapi.co/api/v2/item/284/"},{"name":"insect-plate","url":"https://pokeapi.co/api/v2/item/285/"},{"name":"stone-plate","url":"https://pokeapi.co/api/v2/item/286/"},{"name":"spooky-plate","url":"https://pokeapi.co/api/v2/item/287/"},{"name":"draco-plate","url":"https://pokeapi.co/api/v2/item/288/"},{"name":"dread-plate","url":"https://pokeapi.co/api/v2/item/289/"},{"name":"iron-plate","url":"https://pokeapi.co/api/v2/item/290/"},{"name":"odd-incense","url":"https://pokeapi.co/api/v2/item/291/"},{"name":"rock-incense","url":"https://pokeapi.co/api/v2/item/292/"},{"name":"full-incense","url":"https://pokeapi.co/api/v2/item/293/"},{"name":"wave-incense","url":"https://pokeapi.co/api/v2/item/294/"},{"name":"rose-incense","url":"https://pokeapi.co/api/v2/item/295/"},{"name":"luck-incense","url":"https://pokeapi.co/api/v2/item/296/"},{"name":"pure-incense","url":"https://pokeapi.co/api/v2/item/297/"},{"name":"razor-claw","url":"https://pokeapi.co/api/v2/item/303/"},{"name":"razor-fang","url":"https://pokeapi.co/api/v2/item/304/"}],"name":"holdable","names":[{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Holdable"}]}"#;

#[test]
fn item_attribute_from_id() {
    test_from_id::<ItemAttribute>(5, HOLDABLE_JSON);
}

#[test]
fn item_attribute_from_name() {
    test_from_name::<ItemAttribute>("holdable", HOLDABLE_JSON);
}

// https://pokeapi.co/api/v2/item-category/5/
static IN_A_PINCH_JSON: &str = r#"{"id":5,"items":[{"name":"liechi-berry","url":"https://pokeapi.co/api/v2/item/178/"},{"name":"ganlon-berry","url":"https://pokeapi.co/api/v2/item/179/"},{"name":"salac-berry","url":"https://pokeapi.co/api/v2/item/180/"},{"name":"petaya-berry","url":"https://pokeapi.co/api/v2/item/181/"},{"name":"apicot-berry","url":"https://pokeapi.co/api/v2/item/182/"},{"name":"lansat-berry","url":"https://pokeapi.co/api/v2/item/183/"},{"name":"starf-berry","url":"https://pokeapi.co/api/v2/item/184/"},{"name":"micle-berry","url":"https://pokeapi.co/api/v2/item/186/"},{"name":"custap-berry","url":"https://pokeapi.co/api/v2/item/187/"}],"name":"in-a-pinch","names":[{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"In a pinch"}],"pocket":{"name":"berries","url":"https://pokeapi.co/api/v2/item-pocket/5/"}}"#;

#[test]
fn item_category_from_id() {
    test_from_id::<ItemCategory>(5, IN_A_PINCH_JSON);
}

#[test]
fn item_category_from_name() {
    test_from_name::<ItemCategory>("in-a-pinch", IN_A_PINCH_JSON);
}

// https://pokeapi.co/api/v2/item-fling-effect/1/
static BADLY_POISON_JSON: &str = r#"{"effect_entries":[{"effect":"Badly poisons the target.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"id":1,"items":[{"name":"toxic-orb","url":"https://pokeapi.co/api/v2/item/249/"}],"name":"badly-poison"}"#;

#[test]
fn item_fling_effect_from_id() {
    test_from_id::<ItemFlingEffect>(1, BADLY_POISON_JSON);
}

#[test]
fn item_fling_effect_from_name() {
    test_from_name::<ItemFlingEffect>("badly-poison", BADLY_POISON_JSON);
}

// https://pokeapi.co/api/v2/item-pocket/7/
static BATTLE_JSON: &str = r#"{"categories":[{"name":"stat-boosts","url":"https://pokeapi.co/api/v2/item-category/1/"},{"name":"flutes","url":"https://pokeapi.co/api/v2/item-category/38/"},{"name":"miracle-shooter","url":"https://pokeapi.co/api/v2/item-category/43/"}],"id":7,"name":"battle","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Objets de combat"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Battle Items"}]}"#;

#[test]
fn item_pocket_from_id() {
    test_from_id::<ItemPocket>(7, BATTLE_JSON);
}

#[test]
fn item_pocket_from_name() {
    test_from_name::<ItemPocket>("battle", BATTLE_JSON);
}

// https://pokeapi.co/api/v2/location/792/
static RUINS_OF_HOPE_JSON: &str = r#"{"areas":[],"game_indices":[{"game_index":180,"generation":{"name":"generation-vii","url":"https://pokeapi.co/api/v2/generation/7/"}}],"id":792,"name":"ruins-of-hope","names":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"name":"彼岸遗迹"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"彼岸の遺跡"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Ruins of Hope"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Tempio del Passaggio"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Ruinas del Tránsito"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Ruinen der Heimkehr"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Ruines de l’Au-Delà"},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"name":"彼岸遺跡"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"피안의 유적"},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"ひがんのいせき"}],"region":{"name":"alola","url":"https://pokeapi.co/api/v2/region/7/"}}"#;

#[test]
fn location_from_id() {
    test_from_id::<Location>(792, RUINS_OF_HOPE_JSON);
}

#[test]
fn location_from_name() {
    test_from_name::<Location>("ruins-of-hope", RUINS_OF_HOPE_JSON);
}

// https://pokeapi.co/api/v2/location-area/649/
static ABUNDANT_SHRINE_AREA_JSON: &str = r#"{"encounter_method_rates":[{"encounter_method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"version_details":[{"rate":1,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"rate":1,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"rate":8,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"rate":8,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}}]},{"encounter_method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"version_details":[{"rate":50,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"rate":50,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"rate":50,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"rate":50,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}}]},{"encounter_method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"version_details":[{"rate":10,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"rate":10,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"rate":3,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"rate":3,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}}]},{"encounter_method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"version_details":[{"rate":3,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"rate":3,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}}]}],"game_index":105,"id":649,"location":{"name":"abundant-shrine","url":"https://pokeapi.co/api/v2/location/412/"},"name":"abundant-shrine-area","names":[{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":""}],"pokemon_encounters":[{"pokemon":{"name":"vulpix","url":"https://pokeapi.co/api/v2/pokemon/37/"},"version_details":[{"encounter_details":[{"chance":20,"condition_values":[],"max_level":57,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":57},{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":47}],"max_chance":40,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":57,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":57},{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":47}],"max_chance":40,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":37,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":37},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":34}],"max_chance":30,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":37,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":37},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":34}],"max_chance":30,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"ninetales","url":"https://pokeapi.co/api/v2/pokemon/38/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":5,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":5,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36},{"chance":1,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36}],"max_chance":5,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36},{"chance":1,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36}],"max_chance":5,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"golduck","url":"https://pokeapi.co/api/v2/pokemon/55/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":38,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":38},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":34}],"max_chance":20,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":38,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":38},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":34}],"max_chance":20,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"slowpoke","url":"https://pokeapi.co/api/v2/pokemon/79/"},"version_details":[{"encounter_details":[{"chance":30,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":60,"condition_values":[],"max_level":60,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25}],"max_chance":90,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":30,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":60,"condition_values":[],"max_level":60,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25}],"max_chance":90,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"slowbro","url":"https://pokeapi.co/api/v2/pokemon/80/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":70,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":35}],"max_chance":5,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":70,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":35}],"max_chance":5,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"goldeen","url":"https://pokeapi.co/api/v2/pokemon/118/"},"version_details":[{"encounter_details":[{"chance":40,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":15,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":4,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":1,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":35}],"max_chance":100,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":15,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":4,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":1,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":35}],"max_chance":100,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":40},{"chance":15,"condition_values":[],"max_level":70,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":50},{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":50},{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":40}],"max_chance":100,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":40},{"chance":15,"condition_values":[],"max_level":70,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":50},{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":50},{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":40}],"max_chance":100,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"seaking","url":"https://pokeapi.co/api/v2/pokemon/119/"},"version_details":[{"encounter_details":[{"chance":15,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":35},{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":45},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":45}],"max_chance":20,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":15,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":35},{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":45},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":45}],"max_chance":20,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":15,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":50},{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":50}],"max_chance":20,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":15,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":50},{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":50}],"max_chance":20,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"noctowl","url":"https://pokeapi.co/api/v2/pokemon/164/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":4,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":1,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48},{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50}],"max_chance":30,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":4,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":1,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48},{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50}],"max_chance":30,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"marill","url":"https://pokeapi.co/api/v2/pokemon/183/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":38,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":38},{"chance":30,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":60,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":34}],"max_chance":110,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":38,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":38},{"chance":30,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":60,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":34}],"max_chance":110,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"azumarill","url":"https://pokeapi.co/api/v2/pokemon/184/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":30},{"chance":1,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":30}],"max_chance":5,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36},{"chance":4,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":30},{"chance":1,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":30}],"max_chance":10,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"murkrow","url":"https://pokeapi.co/api/v2/pokemon/198/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":5,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48}],"max_chance":10,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}}]},{"pokemon":{"name":"slowking","url":"https://pokeapi.co/api/v2/pokemon/199/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":35},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":35}],"max_chance":5,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":70,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":35},{"chance":1,"condition_values":[],"max_level":70,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":35}],"max_chance":5,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"misdreavus","url":"https://pokeapi.co/api/v2/pokemon/200/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":5,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48}],"max_chance":10,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"stantler","url":"https://pokeapi.co/api/v2/pokemon/234/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48}],"max_chance":20,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48}],"max_chance":20,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"swablu","url":"https://pokeapi.co/api/v2/pokemon/333/"},"version_details":[{"encounter_details":[{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":33}],"max_chance":20,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":33}],"max_chance":20,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"altaria","url":"https://pokeapi.co/api/v2/pokemon/334/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":38,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":38},{"chance":4,"condition_values":[],"max_level":40,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":40},{"chance":1,"condition_values":[],"max_level":40,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":40},{"chance":4,"condition_values":[],"max_level":36,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":36},{"chance":1,"condition_values":[],"max_level":36,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":36}],"max_chance":20,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":38,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":38},{"chance":4,"condition_values":[],"max_level":40,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":40},{"chance":1,"condition_values":[],"max_level":40,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":40},{"chance":4,"condition_values":[],"max_level":36,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":36},{"chance":1,"condition_values":[],"max_level":36,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":36}],"max_chance":20,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"chimecho","url":"https://pokeapi.co/api/v2/pokemon/358/"},"version_details":[{"encounter_details":[{"chance":20,"condition_values":[],"max_level":57,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":57},{"chance":10,"condition_values":[],"max_level":59,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":59},{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":47},{"chance":10,"condition_values":[],"max_level":49,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":49}],"max_chance":60,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":57,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":57},{"chance":10,"condition_values":[],"max_level":59,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":59},{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":47},{"chance":10,"condition_values":[],"max_level":49,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":49}],"max_chance":60,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"mismagius","url":"https://pokeapi.co/api/v2/pokemon/429/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":5,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}}]},{"pokemon":{"name":"honchkrow","url":"https://pokeapi.co/api/v2/pokemon/430/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":5,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}}]},{"pokemon":{"name":"bronzor","url":"https://pokeapi.co/api/v2/pokemon/436/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":32,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":32},{"chance":5,"condition_values":[],"max_level":32,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":32}],"max_chance":15,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":32,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":32},{"chance":5,"condition_values":[],"max_level":32,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":32}],"max_chance":15,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"bronzong","url":"https://pokeapi.co/api/v2/pokemon/437/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":4,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":1,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":5,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48},{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50}],"max_chance":20,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":4,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":1,"condition_values":[],"max_level":60,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":60},{"chance":5,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48},{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":50}],"max_chance":20,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":36},{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":36}],"max_chance":10,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":36},{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":36}],"max_chance":10,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"audino","url":"https://pokeapi.co/api/v2/pokemon/531/"},"version_details":[{"encounter_details":[{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":47},{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":47},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":48},{"chance":10,"condition_values":[],"max_level":49,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":49},{"chance":10,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50},{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":75,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":47},{"chance":20,"condition_values":[],"max_level":47,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":47},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":48},{"chance":10,"condition_values":[],"max_level":49,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":49},{"chance":10,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50},{"chance":4,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50},{"chance":1,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":75,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":33},{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":33},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":34},{"chance":10,"condition_values":[],"max_level":35,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":35},{"chance":10,"condition_values":[],"max_level":35,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":35},{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36},{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36}],"max_chance":80,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":33},{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":33},{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":34},{"chance":10,"condition_values":[],"max_level":35,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":35},{"chance":10,"condition_values":[],"max_level":35,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":35},{"chance":5,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36}],"max_chance":75,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"cottonee","url":"https://pokeapi.co/api/v2/pokemon/546/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48}],"max_chance":20,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":37,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":37},{"chance":10,"condition_values":[],"max_level":39,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":39},{"chance":4,"condition_values":[],"max_level":39,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":39},{"chance":1,"condition_values":[],"max_level":39,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":39},{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":33},{"chance":5,"condition_values":[],"max_level":35,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":35},{"chance":4,"condition_values":[],"max_level":35,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":35},{"chance":1,"condition_values":[],"max_level":35,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":35}],"max_chance":65,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}}]},{"pokemon":{"name":"whimsicott","url":"https://pokeapi.co/api/v2/pokemon/547/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":5,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36},{"chance":1,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36}],"max_chance":5,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}}]},{"pokemon":{"name":"petilil","url":"https://pokeapi.co/api/v2/pokemon/548/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":58,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":58},{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":48}],"max_chance":20,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":20,"condition_values":[],"max_level":37,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":37},{"chance":10,"condition_values":[],"max_level":39,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":39},{"chance":4,"condition_values":[],"max_level":39,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":39},{"chance":1,"condition_values":[],"max_level":39,"method":{"name":"dark-grass","url":"https://pokeapi.co/api/v2/encounter-method/8/"},"min_level":39},{"chance":20,"condition_values":[],"max_level":33,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":33},{"chance":5,"condition_values":[],"max_level":35,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":35},{"chance":4,"condition_values":[],"max_level":35,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":35},{"chance":1,"condition_values":[],"max_level":35,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":35}],"max_chance":65,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"lilligant","url":"https://pokeapi.co/api/v2/pokemon/549/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":50,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":50}],"max_chance":5,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36},{"chance":1,"condition_values":[],"max_level":36,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":36}],"max_chance":5,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"basculin-red-striped","url":"https://pokeapi.co/api/v2/pokemon/550/"},"version_details":[{"encounter_details":[{"chance":40,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":60,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":5,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":4,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25}],"max_chance":110,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":35},{"chance":30,"condition_values":[],"max_level":60,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25}],"max_chance":70,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":40},{"chance":60,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":5,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":30},{"chance":4,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":30},{"chance":1,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":30}],"max_chance":110,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":40},{"chance":30,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25},{"chance":5,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":30}],"max_chance":75,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"emolga","url":"https://pokeapi.co/api/v2/pokemon/587/"},"version_details":[{"encounter_details":[{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":48}],"max_chance":10,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":48,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":48}],"max_chance":10,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":34}],"max_chance":10,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":34,"method":{"name":"grass-spots","url":"https://pokeapi.co/api/v2/encounter-method/9/"},"min_level":34}],"max_chance":10,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]},{"pokemon":{"name":"basculin-blue-striped","url":"https://pokeapi.co/api/v2/pokemon/10016/"},"version_details":[{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":35},{"chance":30,"condition_values":[],"max_level":60,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25}],"max_chance":70,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":55,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":35},{"chance":60,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":5,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":4,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":55,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25}],"max_chance":110,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod-spots","url":"https://pokeapi.co/api/v2/encounter-method/12/"},"min_level":40},{"chance":30,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":25},{"chance":5,"condition_values":[],"max_level":40,"method":{"name":"surf-spots","url":"https://pokeapi.co/api/v2/encounter-method/13/"},"min_level":30}],"max_chance":75,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"encounter_details":[{"chance":40,"condition_values":[],"max_level":60,"method":{"name":"super-rod","url":"https://pokeapi.co/api/v2/encounter-method/4/"},"min_level":40},{"chance":60,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":25},{"chance":5,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":30},{"chance":4,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":30},{"chance":1,"condition_values":[],"max_level":40,"method":{"name":"surf","url":"https://pokeapi.co/api/v2/encounter-method/5/"},"min_level":30}],"max_chance":110,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}}]}]}"#;

#[test]
fn location_area_from_id() {
    test_from_id::<LocationArea>(649, ABUNDANT_SHRINE_AREA_JSON);
}

#[test]
fn location_area_from_name() {
    test_from_name::<LocationArea>("abundant-shrine-area", ABUNDANT_SHRINE_AREA_JSON);
}

// https://pokeapi.co/api/v2/pal-park-area/4/
static POND_JSON: &str = r#"{"id":4,"name":"pond","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Étang"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Pond"}],"pokemon_encounters":[{"base_score":50,"pokemon_species":{"name":"squirtle","url":"https://pokeapi.co/api/v2/pokemon-species/7/"},"rate":30},{"base_score":80,"pokemon_species":{"name":"wartortle","url":"https://pokeapi.co/api/v2/pokemon-species/8/"},"rate":10},{"base_score":90,"pokemon_species":{"name":"blastoise","url":"https://pokeapi.co/api/v2/pokemon-species/9/"},"rate":3},{"base_score":50,"pokemon_species":{"name":"psyduck","url":"https://pokeapi.co/api/v2/pokemon-species/54/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"golduck","url":"https://pokeapi.co/api/v2/pokemon-species/55/"},"rate":20},{"base_score":50,"pokemon_species":{"name":"poliwag","url":"https://pokeapi.co/api/v2/pokemon-species/60/"},"rate":30},{"base_score":80,"pokemon_species":{"name":"poliwhirl","url":"https://pokeapi.co/api/v2/pokemon-species/61/"},"rate":10},{"base_score":90,"pokemon_species":{"name":"poliwrath","url":"https://pokeapi.co/api/v2/pokemon-species/62/"},"rate":3},{"base_score":50,"pokemon_species":{"name":"slowpoke","url":"https://pokeapi.co/api/v2/pokemon-species/79/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"slowbro","url":"https://pokeapi.co/api/v2/pokemon-species/80/"},"rate":20},{"base_score":50,"pokemon_species":{"name":"goldeen","url":"https://pokeapi.co/api/v2/pokemon-species/118/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"seaking","url":"https://pokeapi.co/api/v2/pokemon-species/119/"},"rate":20},{"base_score":30,"pokemon_species":{"name":"magikarp","url":"https://pokeapi.co/api/v2/pokemon-species/129/"},"rate":50},{"base_score":90,"pokemon_species":{"name":"gyarados","url":"https://pokeapi.co/api/v2/pokemon-species/130/"},"rate":3},{"base_score":70,"pokemon_species":{"name":"dratini","url":"https://pokeapi.co/api/v2/pokemon-species/147/"},"rate":20},{"base_score":80,"pokemon_species":{"name":"dragonair","url":"https://pokeapi.co/api/v2/pokemon-species/148/"},"rate":10},{"base_score":50,"pokemon_species":{"name":"totodile","url":"https://pokeapi.co/api/v2/pokemon-species/158/"},"rate":30},{"base_score":80,"pokemon_species":{"name":"croconaw","url":"https://pokeapi.co/api/v2/pokemon-species/159/"},"rate":10},{"base_score":50,"pokemon_species":{"name":"marill","url":"https://pokeapi.co/api/v2/pokemon-species/183/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"azumarill","url":"https://pokeapi.co/api/v2/pokemon-species/184/"},"rate":20},{"base_score":80,"pokemon_species":{"name":"politoed","url":"https://pokeapi.co/api/v2/pokemon-species/186/"},"rate":10},{"base_score":50,"pokemon_species":{"name":"wooper","url":"https://pokeapi.co/api/v2/pokemon-species/194/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"quagsire","url":"https://pokeapi.co/api/v2/pokemon-species/195/"},"rate":20},{"base_score":80,"pokemon_species":{"name":"slowking","url":"https://pokeapi.co/api/v2/pokemon-species/199/"},"rate":10},{"base_score":50,"pokemon_species":{"name":"mudkip","url":"https://pokeapi.co/api/v2/pokemon-species/258/"},"rate":30},{"base_score":80,"pokemon_species":{"name":"marshtomp","url":"https://pokeapi.co/api/v2/pokemon-species/259/"},"rate":10},{"base_score":90,"pokemon_species":{"name":"swampert","url":"https://pokeapi.co/api/v2/pokemon-species/260/"},"rate":3},{"base_score":30,"pokemon_species":{"name":"lotad","url":"https://pokeapi.co/api/v2/pokemon-species/270/"},"rate":50},{"base_score":50,"pokemon_species":{"name":"lombre","url":"https://pokeapi.co/api/v2/pokemon-species/271/"},"rate":30},{"base_score":80,"pokemon_species":{"name":"ludicolo","url":"https://pokeapi.co/api/v2/pokemon-species/272/"},"rate":10},{"base_score":50,"pokemon_species":{"name":"surskit","url":"https://pokeapi.co/api/v2/pokemon-species/283/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"masquerain","url":"https://pokeapi.co/api/v2/pokemon-species/284/"},"rate":20},{"base_score":80,"pokemon_species":{"name":"azurill","url":"https://pokeapi.co/api/v2/pokemon-species/298/"},"rate":10},{"base_score":50,"pokemon_species":{"name":"barboach","url":"https://pokeapi.co/api/v2/pokemon-species/339/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"whiscash","url":"https://pokeapi.co/api/v2/pokemon-species/340/"},"rate":20},{"base_score":50,"pokemon_species":{"name":"corphish","url":"https://pokeapi.co/api/v2/pokemon-species/341/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"crawdaunt","url":"https://pokeapi.co/api/v2/pokemon-species/342/"},"rate":20},{"base_score":50,"pokemon_species":{"name":"piplup","url":"https://pokeapi.co/api/v2/pokemon-species/393/"},"rate":30},{"base_score":80,"pokemon_species":{"name":"prinplup","url":"https://pokeapi.co/api/v2/pokemon-species/394/"},"rate":10},{"base_score":90,"pokemon_species":{"name":"empoleon","url":"https://pokeapi.co/api/v2/pokemon-species/395/"},"rate":3},{"base_score":50,"pokemon_species":{"name":"ambipom","url":"https://pokeapi.co/api/v2/pokemon-species/424/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"drifloon","url":"https://pokeapi.co/api/v2/pokemon-species/425/"},"rate":20},{"base_score":50,"pokemon_species":{"name":"garchomp","url":"https://pokeapi.co/api/v2/pokemon-species/445/"},"rate":30},{"base_score":70,"pokemon_species":{"name":"munchlax","url":"https://pokeapi.co/api/v2/pokemon-species/446/"},"rate":20}]}"#;

#[test]
fn pal_park_area_from_id() {
    test_from_id::<PalParkArea>(4, POND_JSON);
}

#[test]
fn pal_park_area_from_name() {
    test_from_name::<PalParkArea>("pond", POND_JSON);
}

// https://pokeapi.co/api/v2/region/7/
static ALOLA_JSON: &str = r#"{"id":7,"locations":[{"name":"alola-route-1--hauoli-outskirts","url":"https://pokeapi.co/api/v2/location/710/"},{"name":"alola-route-1","url":"https://pokeapi.co/api/v2/location/711/"},{"name":"alola-route-3","url":"https://pokeapi.co/api/v2/location/712/"},{"name":"alola-route-2","url":"https://pokeapi.co/api/v2/location/713/"},{"name":"kalae-bay","url":"https://pokeapi.co/api/v2/location/714/"},{"name":"melemele-sea","url":"https://pokeapi.co/api/v2/location/715/"},{"name":"hauoli-city--beachfront","url":"https://pokeapi.co/api/v2/location/716/"},{"name":"hauoli-city--shopping-district","url":"https://pokeapi.co/api/v2/location/717/"},{"name":"hauoli-city--marina","url":"https://pokeapi.co/api/v2/location/718/"},{"name":"iki-town","url":"https://pokeapi.co/api/v2/location/719/"},{"name":"mahalo-trail","url":"https://pokeapi.co/api/v2/location/720/"},{"name":"mahalo-trail--plank-bridge","url":"https://pokeapi.co/api/v2/location/721/"},{"name":"ruins-of-conflict","url":"https://pokeapi.co/api/v2/location/722/"},{"name":"ten-carat-hill","url":"https://pokeapi.co/api/v2/location/723/"},{"name":"ten-carat-hill--farthest-hollow","url":"https://pokeapi.co/api/v2/location/724/"},{"name":"hauoli-cemetery","url":"https://pokeapi.co/api/v2/location/725/"},{"name":"melemele-meadow","url":"https://pokeapi.co/api/v2/location/726/"},{"name":"seaward-cave","url":"https://pokeapi.co/api/v2/location/727/"},{"name":"berry-fields","url":"https://pokeapi.co/api/v2/location/728/"},{"name":"verdant-cavern--trial-site","url":"https://pokeapi.co/api/v2/location/729/"},{"name":"verdant-cavern--totems-den","url":"https://pokeapi.co/api/v2/location/730/"},{"name":"alola-route-4","url":"https://pokeapi.co/api/v2/location/731/"},{"name":"alola-route-5","url":"https://pokeapi.co/api/v2/location/732/"},{"name":"alola-route-6","url":"https://pokeapi.co/api/v2/location/733/"},{"name":"alola-route-7","url":"https://pokeapi.co/api/v2/location/734/"},{"name":"alola-route-8","url":"https://pokeapi.co/api/v2/location/735/"},{"name":"alola-route-9","url":"https://pokeapi.co/api/v2/location/736/"},{"name":"hano-grand-resort","url":"https://pokeapi.co/api/v2/location/737/"},{"name":"hano-beach","url":"https://pokeapi.co/api/v2/location/738/"},{"name":"akala-meadow","url":"https://pokeapi.co/api/v2/location/739/"},{"name":"paniola-town","url":"https://pokeapi.co/api/v2/location/740/"},{"name":"heahea-city","url":"https://pokeapi.co/api/v2/location/741/"},{"name":"konikoni-city","url":"https://pokeapi.co/api/v2/location/742/"},{"name":"royal-avenue","url":"https://pokeapi.co/api/v2/location/743/"},{"name":"memorial-hill","url":"https://pokeapi.co/api/v2/location/744/"},{"name":"paniola-ranch","url":"https://pokeapi.co/api/v2/location/745/"},{"name":"wela-volcano-park","url":"https://pokeapi.co/api/v2/location/746/"},{"name":"wela-volcano-park--totems-den","url":"https://pokeapi.co/api/v2/location/747/"},{"name":"brooklet-hill","url":"https://pokeapi.co/api/v2/location/748/"},{"name":"brooklet-hill--totems-den","url":"https://pokeapi.co/api/v2/location/749/"},{"name":"lush-jungle","url":"https://pokeapi.co/api/v2/location/750/"},{"name":"ruins-of-life","url":"https://pokeapi.co/api/v2/location/751/"},{"name":"akala-outskirts","url":"https://pokeapi.co/api/v2/location/752/"},{"name":"digletts-tunnel","url":"https://pokeapi.co/api/v2/location/753/"},{"name":"battle-royal-dome","url":"https://pokeapi.co/api/v2/location/754/"},{"name":"alola-route-10","url":"https://pokeapi.co/api/v2/location/755/"},{"name":"alola-route-11","url":"https://pokeapi.co/api/v2/location/756/"},{"name":"secluded-shore","url":"https://pokeapi.co/api/v2/location/757/"},{"name":"alola-route-13","url":"https://pokeapi.co/api/v2/location/758/"},{"name":"tapu-village","url":"https://pokeapi.co/api/v2/location/759/"},{"name":"alola-route-15","url":"https://pokeapi.co/api/v2/location/760/"},{"name":"alola-route-16","url":"https://pokeapi.co/api/v2/location/761/"},{"name":"alola-route-17","url":"https://pokeapi.co/api/v2/location/762/"},{"name":"alola-route-12","url":"https://pokeapi.co/api/v2/location/763/"},{"name":"haina-desert","url":"https://pokeapi.co/api/v2/location/764/"},{"name":"alola-route-14","url":"https://pokeapi.co/api/v2/location/765/"},{"name":"ulaula-meadow","url":"https://pokeapi.co/api/v2/location/766/"},{"name":"po-town","url":"https://pokeapi.co/api/v2/location/767/"},{"name":"malie-city","url":"https://pokeapi.co/api/v2/location/768/"},{"name":"malie-garden","url":"https://pokeapi.co/api/v2/location/769/"},{"name":"mount-hokulani","url":"https://pokeapi.co/api/v2/location/770/"},{"name":"blush-mountain","url":"https://pokeapi.co/api/v2/location/771/"},{"name":"ruins-of-abundance","url":"https://pokeapi.co/api/v2/location/772/"},{"name":"lake-of-the-sunne","url":"https://pokeapi.co/api/v2/location/773/"},{"name":"lake-of-the-moone","url":"https://pokeapi.co/api/v2/location/774/"},{"name":"mount-lanakila","url":"https://pokeapi.co/api/v2/location/775/"},{"name":"shady-house","url":"https://pokeapi.co/api/v2/location/776/"},{"name":"thrifty-megamart--abandoned-site","url":"https://pokeapi.co/api/v2/location/777/"},{"name":"hokulani-observatory","url":"https://pokeapi.co/api/v2/location/778/"},{"name":"alola-pokemon-league","url":"https://pokeapi.co/api/v2/location/779/"},{"name":"poni-meadow","url":"https://pokeapi.co/api/v2/location/780/"},{"name":"poni-wilds","url":"https://pokeapi.co/api/v2/location/781/"},{"name":"ancient-poni-path","url":"https://pokeapi.co/api/v2/location/782/"},{"name":"poni-breaker-coast","url":"https://pokeapi.co/api/v2/location/783/"},{"name":"poni-grove","url":"https://pokeapi.co/api/v2/location/784/"},{"name":"poni-plains","url":"https://pokeapi.co/api/v2/location/785/"},{"name":"poni-coast","url":"https://pokeapi.co/api/v2/location/786/"},{"name":"poni-gauntlet","url":"https://pokeapi.co/api/v2/location/787/"},{"name":"seafolk-village","url":"https://pokeapi.co/api/v2/location/788/"},{"name":"vast-poni-canyon","url":"https://pokeapi.co/api/v2/location/789/"},{"name":"altar-of-the-sunne","url":"https://pokeapi.co/api/v2/location/790/"},{"name":"altar-of-the-moone","url":"https://pokeapi.co/api/v2/location/791/"},{"name":"ruins-of-hope","url":"https://pokeapi.co/api/v2/location/792/"},{"name":"resolution-cave","url":"https://pokeapi.co/api/v2/location/793/"},{"name":"exeggutor-island","url":"https://pokeapi.co/api/v2/location/794/"},{"name":"battle-tree","url":"https://pokeapi.co/api/v2/location/795/"},{"name":"aether-paradise","url":"https://pokeapi.co/api/v2/location/796/"},{"name":"ultra-space","url":"https://pokeapi.co/api/v2/location/797/"},{"name":"malie-city--outer-cape","url":"https://pokeapi.co/api/v2/location/798/"}],"main_generation":{"name":"generation-vii","url":"https://pokeapi.co/api/v2/generation/7/"},"name":"alola","names":[{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Alola"}],"pokedexes":[],"version_groups":[{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"},{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}]}"#;

#[test]
fn region_from_id() {
    test_from_id::<Region>(7, ALOLA_JSON);
}

#[test]
fn region_from_name() {
    test_from_name::<Region>("alola", ALOLA_JSON);
}

// https://pokeapi.co/api/v2/machine/1442/
static DIVE_JSON: &str = r#"{"id":1442,"item":{"name":"hm08","url":"https://pokeapi.co/api/v2/item/404/"},"move":{"name":"dive","url":"https://pokeapi.co/api/v2/move/291/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}}"#;

#[test]
fn machine_from_id() {
    test_from_id::<Machine>(1442, DIVE_JSON);
}

// https://pokeapi.co/api/v2/move/92/
static TOXIC_JSON: &str = r#"{"accuracy":90,"contest_combos":null,"contest_effect":{"url":"https://pokeapi.co/api/v2/contest-effect/33/"},"contest_type":{"name":"smart","url":"https://pokeapi.co/api/v2/contest-type/4/"},"damage_class":{"name":"status","url":"https://pokeapi.co/api/v2/move-damage-class/1/"},"effect_chance":null,"effect_changes":[],"effect_entries":[{"effect":"Badly poisons the target.  Never misses when used by a poison-type Pokémon.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"short_effect":"Badly poisons the target, inflicting more damage every turn."}],"flavor_text_entries":[{"flavor_text":"让对手陷入剧毒状态。\n随着回合的推进，\n中毒伤害会增加。","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"相手を　猛毒の　状態に　する。\nターンが　すすむほど\n毒の　ダメージが　増えていく。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"A move that leaves the target badly poisoned.\nIts poison damage worsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Iperavvelena il bersaglio con una potente tossina.\nIl danno peggiora a ogni turno.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Envenena gravemente al objetivo y causa un daño \nmayor en cada turno.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Vergiftet das Ziel mit einem potenten Toxin schwer.\nDie Vergiftung wird von Runde zu Runde stärker.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Empoisonne gravement l’ennemi.\nLes dégâts dus au poison augmentent à chaque\ntour.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"讓對手陷入劇毒狀態。\n中毒傷害會隨著\n回合的進行而增加。","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"상대를 맹독의 상태로 만든다.\n턴이 진행될수록\n독의 데미지가 증가한다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"あいてを　もうどくの\nじょうたいに　する。　ターンが　すすむほど\nどくの　ダメージが　ふえていく。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"让对手陷入剧毒状态。\n随着回合的推进，\n中毒伤害会增加。","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"相手を　猛毒の　状態に　する。\nターンが　すすむほど\n毒の　ダメージが　増えていく。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"A move that leaves the target badly poisoned.\nIts poison damage worsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Iperavvelena il bersaglio con una potente tossina.\nIl danno peggiora a ogni turno.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Envenena gravemente al objetivo y causa un daño \nmayor en cada turno.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Vergiftet das Ziel mit einem potenten Toxin schwer.\nDie Vergiftung wird von Runde zu Runde stärker.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Empoisonne gravement l’ennemi.\nLes dégâts dus au poison augmentent à chaque\ntour.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"讓對手陷入劇毒狀態。\n中毒傷害會隨著\n回合的進行而增加。","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"상대를 맹독의 상태로 만든다.\n턴이 진행될수록\n독의 데미지가 증가한다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"あいてを　もうどくの\nじょうたいに　する。　ターンが　すすむほど\nどくの　ダメージが　ふえていく。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"相手を　猛毒の　状態に　する。\nターンが　すすむほど\n毒の　ダメージが　増えていく。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"A move that leaves the target\nbadly poisoned.\nIts poison damage worsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"Iperavvelena il bersaglio con una potente tossina.\nIl danno peggiora a ogni turno.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"Envenena gravemente al objetivo y causa un daño \nmayor en cada turno.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"Vergiftet das Ziel mit einem potenten Toxin schwer.\nVergiftung wird von Runde zu Runde stärker.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"Empoisonne gravement l’ennemi.\nLes dégâts dus au poison augmentent à chaque\ntour.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"상대를 맹독의 상태로 만든다.\n턴이 진행될수록\n독의 데미지가 증가한다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"あいてを　もうどくの\nじょうたいに　する。ターンが　すすむほど\nどくの　ダメージが　ふえていく。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"flavor_text":"相手を　猛毒の　状態に　する。\nターンが　すすむほど\n毒の　ダメージが　増えていく。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"A move that leaves the target\nbadly poisoned.\nIts poison damage worsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"Iperavvelena il bersaglio con una potente tossina.\nIl danno peggiora a ogni turno.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"Envenena gravemente al objetivo y causa un daño\nmayor en cada turno.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"Vergiftet das Ziel mit einem potenten Toxin schwer.\nVergiftung wird von Runde zu Runde stärker.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"Empoisonne gravement l’ennemi.\nLes dégâts dus au poison augmentent à chaque\ntour.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"상대를 맹독의 상태로 만든다.\n턴이 진행될수록\n독의 데미지가 증가한다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"あいてを　もうどくの\nじょうたいに　する。ターンが　すすむほど\nどくの　ダメージが　ふえていく。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"flavor_text":"A move that leaves the target\nbadly poisoned.\nIts poison damage worsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"flavor_text":"A move that leaves the target\nbadly poisoned.\nIts poison damage worsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"flavor_text":"Empoisonne gravement l’ennemi. Les dégâts\ndus au poison augmentent à chaque tour.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"flavor_text":"A move that leaves\nthe target badly\npoisoned.\nIts poison damage\nworsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"flavor_text":"A move that leaves\nthe target badly\npoisoned.\nIts poison damage\nworsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"flavor_text":"A move that leaves\nthe target badly\npoisoned.\nIts poison damage\nworsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"flavor_text":"A move that badly\npoisons the foe.\nIts poison damage\nworsens every turn.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"flavor_text":"Poisons the foe with an\nintensifying toxin.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"flavor_text":"Poisons the foe with an\nintensifying toxin.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"flavor_text":"A poison move with\nincreasing damage.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"flavor_text":"A poison move with\nincreasing damage.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}}],"generation":{"name":"generation-i","url":"https://pokeapi.co/api/v2/generation/1/"},"id":92,"machines":[{"machine":{"url":"https://pokeapi.co/api/v2/machine/91/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/92/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/93/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/94/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/95/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/96/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/97/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/98/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/99/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/100/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/101/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/102/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/103/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/104/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/105/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/106/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/107/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"machine":{"url":"https://pokeapi.co/api/v2/machine/108/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}}],"meta":{"ailment":{"name":"poison","url":"https://pokeapi.co/api/v2/move-ailment/5/"},"ailment_chance":0,"category":{"name":"ailment","url":"https://pokeapi.co/api/v2/move-category/1/"},"crit_rate":0,"drain":0,"flinch_chance":0,"healing":0,"max_hits":null,"max_turns":15,"min_hits":null,"min_turns":15,"stat_chance":0},"name":"toxic","names":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"name":"剧毒"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"どくどく"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Toxic"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Tossina"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Tóxico"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Toxin"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Toxik"},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"name":"劇毒"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"맹독"},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"どくどく"}],"past_values":[{"accuracy":85,"effect_chance":null,"effect_entries":[],"power":null,"pp":null,"type":null,"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}],"power":null,"pp":10,"priority":0,"stat_changes":[],"super_contest_effect":{"url":"https://pokeapi.co/api/v2/super-contest-effect/19/"},"target":{"name":"selected-pokemon","url":"https://pokeapi.co/api/v2/move-target/10/"},"type":{"name":"poison","url":"https://pokeapi.co/api/v2/type/4/"}}"#;

#[test]
fn move_from_id() {
    test_from_id::<Move>(92, TOXIC_JSON);
}

#[test]
fn move_from_name() {
    test_from_name::<Move>("toxic", TOXIC_JSON);
}

// https://pokeapi.co/api/v2/move-ailment/-1/
static UNKNOWN_JSON: &str = r#"{"id":-1,"moves":[{"name":"tri-attack","url":"https://pokeapi.co/api/v2/move/161/"},{"name":"telekinesis","url":"https://pokeapi.co/api/v2/move/477/"},{"name":"smack-down","url":"https://pokeapi.co/api/v2/move/479/"},{"name":"thousand-arrows","url":"https://pokeapi.co/api/v2/move/614/"}],"name":"unknown","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"????"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"????"}]}"#;

#[test]
fn move_ailment_from_id() {
    test_from_id::<MoveAilment>(-1, UNKNOWN_JSON);
}

#[test]
fn move_ailment_from_name() {
    test_from_name::<MoveAilment>("unknown", UNKNOWN_JSON);
}

// https://pokeapi.co/api/v2/move-battle-style/3/
static SUPPORT_JSON: &str = r#"{"id":3,"name":"support","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Assistance"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Support"}]}"#;

#[test]
fn move_battle_style_from_id() {
    test_from_id::<MoveBattleStyle>(3, SUPPORT_JSON);
}

#[test]
fn move_battle_style_from_name() {
    test_from_name::<MoveBattleStyle>("support", SUPPORT_JSON);
}

// https://pokeapi.co/api/v2/move-category/9/
static OHKO_JSON: &str = r#"{"descriptions":[{"description":"One-hit KO","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"id":9,"moves":[{"name":"guillotine","url":"https://pokeapi.co/api/v2/move/12/"},{"name":"horn-drill","url":"https://pokeapi.co/api/v2/move/32/"},{"name":"fissure","url":"https://pokeapi.co/api/v2/move/90/"},{"name":"sheer-cold","url":"https://pokeapi.co/api/v2/move/329/"}],"name":"ohko"}"#;

#[test]
fn move_category_from_id() {
    test_from_id::<MoveCategory>(9, OHKO_JSON);
}

#[test]
fn move_category_from_name() {
    test_from_name::<MoveCategory>("ohko", OHKO_JSON);
}

// https://pokeapi.co/api/v2/move-damage-class/3/
static SPECIAL_JSON: &str = r#"{"descriptions":[{"description":"Special damage, controlled by Special Attack and Special Defense","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"description":"Daño Especial, controlado por el Ataque Esecieal y Defensa Especial","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"}},{"description":"Spezieller Schaden, beeinflusst von Spezialangriff und Spezialverteidigung","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"}},{"description":"Dégâts spéciaux, influencés par l'Attaque Spéciale et la Défense Spéciale","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"}},{"description":"とくしゅのダメージ。とくこうととくぼうを行使する","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"}}],"id":3,"moves":[{"name":"razor-wind","url":"https://pokeapi.co/api/v2/move/13/"},{"name":"gust","url":"https://pokeapi.co/api/v2/move/16/"},{"name":"sonic-boom","url":"https://pokeapi.co/api/v2/move/49/"},{"name":"acid","url":"https://pokeapi.co/api/v2/move/51/"},{"name":"ember","url":"https://pokeapi.co/api/v2/move/52/"},{"name":"flamethrower","url":"https://pokeapi.co/api/v2/move/53/"},{"name":"water-gun","url":"https://pokeapi.co/api/v2/move/55/"},{"name":"hydro-pump","url":"https://pokeapi.co/api/v2/move/56/"},{"name":"surf","url":"https://pokeapi.co/api/v2/move/57/"},{"name":"ice-beam","url":"https://pokeapi.co/api/v2/move/58/"},{"name":"blizzard","url":"https://pokeapi.co/api/v2/move/59/"},{"name":"psybeam","url":"https://pokeapi.co/api/v2/move/60/"},{"name":"bubble-beam","url":"https://pokeapi.co/api/v2/move/61/"},{"name":"aurora-beam","url":"https://pokeapi.co/api/v2/move/62/"},{"name":"hyper-beam","url":"https://pokeapi.co/api/v2/move/63/"},{"name":"absorb","url":"https://pokeapi.co/api/v2/move/71/"},{"name":"mega-drain","url":"https://pokeapi.co/api/v2/move/72/"},{"name":"solar-beam","url":"https://pokeapi.co/api/v2/move/76/"},{"name":"petal-dance","url":"https://pokeapi.co/api/v2/move/80/"},{"name":"dragon-rage","url":"https://pokeapi.co/api/v2/move/82/"},{"name":"fire-spin","url":"https://pokeapi.co/api/v2/move/83/"},{"name":"thunder-shock","url":"https://pokeapi.co/api/v2/move/84/"},{"name":"thunderbolt","url":"https://pokeapi.co/api/v2/move/85/"},{"name":"thunder","url":"https://pokeapi.co/api/v2/move/87/"},{"name":"confusion","url":"https://pokeapi.co/api/v2/move/93/"},{"name":"psychic","url":"https://pokeapi.co/api/v2/move/94/"},{"name":"night-shade","url":"https://pokeapi.co/api/v2/move/101/"},{"name":"smog","url":"https://pokeapi.co/api/v2/move/123/"},{"name":"sludge","url":"https://pokeapi.co/api/v2/move/124/"},{"name":"fire-blast","url":"https://pokeapi.co/api/v2/move/126/"},{"name":"swift","url":"https://pokeapi.co/api/v2/move/129/"},{"name":"dream-eater","url":"https://pokeapi.co/api/v2/move/138/"},{"name":"bubble","url":"https://pokeapi.co/api/v2/move/145/"},{"name":"psywave","url":"https://pokeapi.co/api/v2/move/149/"},{"name":"tri-attack","url":"https://pokeapi.co/api/v2/move/161/"},{"name":"snore","url":"https://pokeapi.co/api/v2/move/173/"},{"name":"aeroblast","url":"https://pokeapi.co/api/v2/move/177/"},{"name":"powder-snow","url":"https://pokeapi.co/api/v2/move/181/"},{"name":"sludge-bomb","url":"https://pokeapi.co/api/v2/move/188/"},{"name":"mud-slap","url":"https://pokeapi.co/api/v2/move/189/"},{"name":"octazooka","url":"https://pokeapi.co/api/v2/move/190/"},{"name":"zap-cannon","url":"https://pokeapi.co/api/v2/move/192/"},{"name":"icy-wind","url":"https://pokeapi.co/api/v2/move/196/"},{"name":"giga-drain","url":"https://pokeapi.co/api/v2/move/202/"},{"name":"dragon-breath","url":"https://pokeapi.co/api/v2/move/225/"},{"name":"hidden-power","url":"https://pokeapi.co/api/v2/move/237/"},{"name":"twister","url":"https://pokeapi.co/api/v2/move/239/"},{"name":"mirror-coat","url":"https://pokeapi.co/api/v2/move/243/"},{"name":"ancient-power","url":"https://pokeapi.co/api/v2/move/246/"},{"name":"shadow-ball","url":"https://pokeapi.co/api/v2/move/247/"},{"name":"future-sight","url":"https://pokeapi.co/api/v2/move/248/"},{"name":"whirlpool","url":"https://pokeapi.co/api/v2/move/250/"},{"name":"uproar","url":"https://pokeapi.co/api/v2/move/253/"},{"name":"spit-up","url":"https://pokeapi.co/api/v2/move/255/"},{"name":"heat-wave","url":"https://pokeapi.co/api/v2/move/257/"},{"name":"eruption","url":"https://pokeapi.co/api/v2/move/284/"},{"name":"luster-purge","url":"https://pokeapi.co/api/v2/move/295/"},{"name":"mist-ball","url":"https://pokeapi.co/api/v2/move/296/"},{"name":"hyper-voice","url":"https://pokeapi.co/api/v2/move/304/"},{"name":"blast-burn","url":"https://pokeapi.co/api/v2/move/307/"},{"name":"hydro-cannon","url":"https://pokeapi.co/api/v2/move/308/"},{"name":"weather-ball","url":"https://pokeapi.co/api/v2/move/311/"},{"name":"air-cutter","url":"https://pokeapi.co/api/v2/move/314/"},{"name":"overheat","url":"https://pokeapi.co/api/v2/move/315/"},{"name":"silver-wind","url":"https://pokeapi.co/api/v2/move/318/"},{"name":"water-spout","url":"https://pokeapi.co/api/v2/move/323/"},{"name":"signal-beam","url":"https://pokeapi.co/api/v2/move/324/"},{"name":"extrasensory","url":"https://pokeapi.co/api/v2/move/326/"},{"name":"sheer-cold","url":"https://pokeapi.co/api/v2/move/329/"},{"name":"muddy-water","url":"https://pokeapi.co/api/v2/move/330/"},{"name":"frenzy-plant","url":"https://pokeapi.co/api/v2/move/338/"},{"name":"mud-shot","url":"https://pokeapi.co/api/v2/move/341/"},{"name":"magical-leaf","url":"https://pokeapi.co/api/v2/move/345/"},{"name":"shock-wave","url":"https://pokeapi.co/api/v2/move/351/"},{"name":"water-pulse","url":"https://pokeapi.co/api/v2/move/352/"},{"name":"doom-desire","url":"https://pokeapi.co/api/v2/move/353/"},{"name":"psycho-boost","url":"https://pokeapi.co/api/v2/move/354/"},{"name":"brine","url":"https://pokeapi.co/api/v2/move/362/"},{"name":"trump-card","url":"https://pokeapi.co/api/v2/move/376/"},{"name":"wring-out","url":"https://pokeapi.co/api/v2/move/378/"},{"name":"aura-sphere","url":"https://pokeapi.co/api/v2/move/396/"},{"name":"dark-pulse","url":"https://pokeapi.co/api/v2/move/399/"},{"name":"air-slash","url":"https://pokeapi.co/api/v2/move/403/"},{"name":"bug-buzz","url":"https://pokeapi.co/api/v2/move/405/"},{"name":"dragon-pulse","url":"https://pokeapi.co/api/v2/move/406/"},{"name":"power-gem","url":"https://pokeapi.co/api/v2/move/408/"},{"name":"vacuum-wave","url":"https://pokeapi.co/api/v2/move/410/"},{"name":"focus-blast","url":"https://pokeapi.co/api/v2/move/411/"},{"name":"energy-ball","url":"https://pokeapi.co/api/v2/move/412/"},{"name":"earth-power","url":"https://pokeapi.co/api/v2/move/414/"},{"name":"mud-bomb","url":"https://pokeapi.co/api/v2/move/426/"},{"name":"mirror-shot","url":"https://pokeapi.co/api/v2/move/429/"},{"name":"flash-cannon","url":"https://pokeapi.co/api/v2/move/430/"},{"name":"draco-meteor","url":"https://pokeapi.co/api/v2/move/434/"},{"name":"discharge","url":"https://pokeapi.co/api/v2/move/435/"},{"name":"lava-plume","url":"https://pokeapi.co/api/v2/move/436/"},{"name":"leaf-storm","url":"https://pokeapi.co/api/v2/move/437/"},{"name":"grass-knot","url":"https://pokeapi.co/api/v2/move/447/"},{"name":"chatter","url":"https://pokeapi.co/api/v2/move/448/"},{"name":"judgment","url":"https://pokeapi.co/api/v2/move/449/"},{"name":"charge-beam","url":"https://pokeapi.co/api/v2/move/451/"},{"name":"roar-of-time","url":"https://pokeapi.co/api/v2/move/459/"},{"name":"spacial-rend","url":"https://pokeapi.co/api/v2/move/460/"},{"name":"magma-storm","url":"https://pokeapi.co/api/v2/move/463/"},{"name":"seed-flare","url":"https://pokeapi.co/api/v2/move/465/"},{"name":"ominous-wind","url":"https://pokeapi.co/api/v2/move/466/"},{"name":"psyshock","url":"https://pokeapi.co/api/v2/move/473/"},{"name":"venoshock","url":"https://pokeapi.co/api/v2/move/474/"},{"name":"flame-burst","url":"https://pokeapi.co/api/v2/move/481/"},{"name":"sludge-wave","url":"https://pokeapi.co/api/v2/move/482/"},{"name":"synchronoise","url":"https://pokeapi.co/api/v2/move/485/"},{"name":"electro-ball","url":"https://pokeapi.co/api/v2/move/486/"},{"name":"acid-spray","url":"https://pokeapi.co/api/v2/move/491/"},{"name":"round","url":"https://pokeapi.co/api/v2/move/496/"},{"name":"echoed-voice","url":"https://pokeapi.co/api/v2/move/497/"},{"name":"clear-smog","url":"https://pokeapi.co/api/v2/move/499/"},{"name":"stored-power","url":"https://pokeapi.co/api/v2/move/500/"},{"name":"scald","url":"https://pokeapi.co/api/v2/move/503/"},{"name":"hex","url":"https://pokeapi.co/api/v2/move/506/"},{"name":"incinerate","url":"https://pokeapi.co/api/v2/move/510/"},{"name":"final-gambit","url":"https://pokeapi.co/api/v2/move/515/"},{"name":"inferno","url":"https://pokeapi.co/api/v2/move/517/"},{"name":"water-pledge","url":"https://pokeapi.co/api/v2/move/518/"},{"name":"fire-pledge","url":"https://pokeapi.co/api/v2/move/519/"},{"name":"grass-pledge","url":"https://pokeapi.co/api/v2/move/520/"},{"name":"volt-switch","url":"https://pokeapi.co/api/v2/move/521/"},{"name":"struggle-bug","url":"https://pokeapi.co/api/v2/move/522/"},{"name":"frost-breath","url":"https://pokeapi.co/api/v2/move/524/"},{"name":"electroweb","url":"https://pokeapi.co/api/v2/move/527/"},{"name":"leaf-tornado","url":"https://pokeapi.co/api/v2/move/536/"},{"name":"night-daze","url":"https://pokeapi.co/api/v2/move/539/"},{"name":"psystrike","url":"https://pokeapi.co/api/v2/move/540/"},{"name":"hurricane","url":"https://pokeapi.co/api/v2/move/542/"},{"name":"searing-shot","url":"https://pokeapi.co/api/v2/move/545/"},{"name":"techno-blast","url":"https://pokeapi.co/api/v2/move/546/"},{"name":"relic-song","url":"https://pokeapi.co/api/v2/move/547/"},{"name":"secret-sword","url":"https://pokeapi.co/api/v2/move/548/"},{"name":"glaciate","url":"https://pokeapi.co/api/v2/move/549/"},{"name":"blue-flare","url":"https://pokeapi.co/api/v2/move/551/"},{"name":"fiery-dance","url":"https://pokeapi.co/api/v2/move/552/"},{"name":"ice-burn","url":"https://pokeapi.co/api/v2/move/554/"},{"name":"snarl","url":"https://pokeapi.co/api/v2/move/555/"},{"name":"fusion-flare","url":"https://pokeapi.co/api/v2/move/558/"},{"name":"belch","url":"https://pokeapi.co/api/v2/move/562/"},{"name":"parabolic-charge","url":"https://pokeapi.co/api/v2/move/570/"},{"name":"freeze-dry","url":"https://pokeapi.co/api/v2/move/573/"},{"name":"disarming-voice","url":"https://pokeapi.co/api/v2/move/574/"},{"name":"draining-kiss","url":"https://pokeapi.co/api/v2/move/577/"},{"name":"fairy-wind","url":"https://pokeapi.co/api/v2/move/584/"},{"name":"moonblast","url":"https://pokeapi.co/api/v2/move/585/"},{"name":"boomburst","url":"https://pokeapi.co/api/v2/move/586/"},{"name":"steam-eruption","url":"https://pokeapi.co/api/v2/move/592/"},{"name":"hyperspace-hole","url":"https://pokeapi.co/api/v2/move/593/"},{"name":"water-shuriken","url":"https://pokeapi.co/api/v2/move/594/"},{"name":"mystical-fire","url":"https://pokeapi.co/api/v2/move/595/"},{"name":"dazzling-gleam","url":"https://pokeapi.co/api/v2/move/605/"},{"name":"infestation","url":"https://pokeapi.co/api/v2/move/611/"},{"name":"oblivion-wing","url":"https://pokeapi.co/api/v2/move/613/"},{"name":"light-of-ruin","url":"https://pokeapi.co/api/v2/move/617/"},{"name":"origin-pulse","url":"https://pokeapi.co/api/v2/move/618/"},{"name":"breakneck-blitz--special","url":"https://pokeapi.co/api/v2/move/623/"},{"name":"all-out-pummeling--special","url":"https://pokeapi.co/api/v2/move/625/"},{"name":"supersonic-skystrike--special","url":"https://pokeapi.co/api/v2/move/627/"},{"name":"acid-downpour--special","url":"https://pokeapi.co/api/v2/move/629/"},{"name":"tectonic-rage--special","url":"https://pokeapi.co/api/v2/move/631/"},{"name":"continental-crush--special","url":"https://pokeapi.co/api/v2/move/633/"},{"name":"savage-spin-out--special","url":"https://pokeapi.co/api/v2/move/635/"},{"name":"never-ending-nightmare--special","url":"https://pokeapi.co/api/v2/move/637/"},{"name":"corkscrew-crash--special","url":"https://pokeapi.co/api/v2/move/639/"},{"name":"inferno-overdrive--special","url":"https://pokeapi.co/api/v2/move/641/"},{"name":"hydro-vortex--special","url":"https://pokeapi.co/api/v2/move/643/"},{"name":"bloom-doom--special","url":"https://pokeapi.co/api/v2/move/645/"},{"name":"gigavolt-havoc--special","url":"https://pokeapi.co/api/v2/move/647/"},{"name":"shattered-psyche--special","url":"https://pokeapi.co/api/v2/move/649/"},{"name":"subzero-slammer--special","url":"https://pokeapi.co/api/v2/move/651/"},{"name":"devastating-drake--special","url":"https://pokeapi.co/api/v2/move/653/"},{"name":"black-hole-eclipse--special","url":"https://pokeapi.co/api/v2/move/655/"},{"name":"twinkle-tackle--special","url":"https://pokeapi.co/api/v2/move/657/"},{"name":"sparkling-aria","url":"https://pokeapi.co/api/v2/move/664/"},{"name":"pollen-puff","url":"https://pokeapi.co/api/v2/move/676/"},{"name":"burn-up","url":"https://pokeapi.co/api/v2/move/682/"},{"name":"revelation-dance","url":"https://pokeapi.co/api/v2/move/686/"},{"name":"core-enforcer","url":"https://pokeapi.co/api/v2/move/687/"},{"name":"clanging-scales","url":"https://pokeapi.co/api/v2/move/691/"},{"name":"oceanic-operetta","url":"https://pokeapi.co/api/v2/move/697/"},{"name":"guardian-of-alola","url":"https://pokeapi.co/api/v2/move/698/"},{"name":"stoked-sparksurfer","url":"https://pokeapi.co/api/v2/move/700/"},{"name":"genesis-supernova","url":"https://pokeapi.co/api/v2/move/703/"},{"name":"shell-trap","url":"https://pokeapi.co/api/v2/move/704/"},{"name":"fleur-cannon","url":"https://pokeapi.co/api/v2/move/705/"},{"name":"prismatic-laser","url":"https://pokeapi.co/api/v2/move/711/"},{"name":"moongeist-beam","url":"https://pokeapi.co/api/v2/move/714/"},{"name":"natures-madness","url":"https://pokeapi.co/api/v2/move/717/"},{"name":"10-000-000-volt-thunderbolt","url":"https://pokeapi.co/api/v2/move/719/"},{"name":"mind-blown","url":"https://pokeapi.co/api/v2/move/720/"},{"name":"photon-geyser","url":"https://pokeapi.co/api/v2/move/722/"},{"name":"light-that-burns-the-sky","url":"https://pokeapi.co/api/v2/move/723/"},{"name":"menacing-moonraze-maelstrom","url":"https://pokeapi.co/api/v2/move/725/"},{"name":"clangorous-soulblaze","url":"https://pokeapi.co/api/v2/move/728/"},{"name":"shadow-bolt","url":"https://pokeapi.co/api/v2/move/10004/"},{"name":"shadow-chill","url":"https://pokeapi.co/api/v2/move/10006/"},{"name":"shadow-fire","url":"https://pokeapi.co/api/v2/move/10008/"},{"name":"shadow-rave","url":"https://pokeapi.co/api/v2/move/10009/"},{"name":"shadow-storm","url":"https://pokeapi.co/api/v2/move/10010/"},{"name":"shadow-wave","url":"https://pokeapi.co/api/v2/move/10011/"},{"name":"shadow-half","url":"https://pokeapi.co/api/v2/move/10013/"}],"name":"special","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"とくしゅ"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"spéciale"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"spezial"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"especial"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"special"}]}"#;

#[test]
fn move_damage_class_from_id() {
    test_from_id::<MoveDamageClass>(3, SPECIAL_JSON);
}

#[test]
fn move_damage_class_from_name() {
    test_from_name::<MoveDamageClass>("special", SPECIAL_JSON);
}

// https://pokeapi.co/api/v2/move-learn-method/2/
static EGG_JSON: &str = r#"{"descriptions":[{"description":"Appears on a newly-hatched Pokémon, if the father had the same move.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"description":"Aparece en un Pokémon recién nacido, si el padre tuvo el mismo movimiento.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"}},{"description":"Erscheint bei einem frisch geschlüpften Pokémon, wenn der Vater die selbe Fähigkeit hatte.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"}}],"id":2,"name":"egg","names":[{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Ei"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Huevo"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Egg"}],"version_groups":[{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"},{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"},{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"},{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"},{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"},{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"},{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"},{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"},{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"},{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"},{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"},{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"},{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"},{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}]}"#;

#[test]
fn move_learn_method_from_id() {
    test_from_id::<MoveLearnMethod>(2, EGG_JSON);
}

#[test]
fn move_learn_method_from_name() {
    test_from_name::<MoveLearnMethod>("egg", EGG_JSON);
}

// https://pokeapi.co/api/v2/move-target/7/
static USER_JSON: &str = r#"{"descriptions":[{"description":"The user.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"description":"Der Anwender.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"}}],"id":7,"moves":[{"name":"swords-dance","url":"https://pokeapi.co/api/v2/move/14/"},{"name":"growth","url":"https://pokeapi.co/api/v2/move/74/"},{"name":"meditate","url":"https://pokeapi.co/api/v2/move/96/"},{"name":"agility","url":"https://pokeapi.co/api/v2/move/97/"},{"name":"teleport","url":"https://pokeapi.co/api/v2/move/100/"},{"name":"double-team","url":"https://pokeapi.co/api/v2/move/104/"},{"name":"recover","url":"https://pokeapi.co/api/v2/move/105/"},{"name":"harden","url":"https://pokeapi.co/api/v2/move/106/"},{"name":"minimize","url":"https://pokeapi.co/api/v2/move/107/"},{"name":"withdraw","url":"https://pokeapi.co/api/v2/move/110/"},{"name":"defense-curl","url":"https://pokeapi.co/api/v2/move/111/"},{"name":"barrier","url":"https://pokeapi.co/api/v2/move/112/"},{"name":"focus-energy","url":"https://pokeapi.co/api/v2/move/116/"},{"name":"bide","url":"https://pokeapi.co/api/v2/move/117/"},{"name":"metronome","url":"https://pokeapi.co/api/v2/move/118/"},{"name":"amnesia","url":"https://pokeapi.co/api/v2/move/133/"},{"name":"soft-boiled","url":"https://pokeapi.co/api/v2/move/135/"},{"name":"splash","url":"https://pokeapi.co/api/v2/move/150/"},{"name":"acid-armor","url":"https://pokeapi.co/api/v2/move/151/"},{"name":"rest","url":"https://pokeapi.co/api/v2/move/156/"},{"name":"sharpen","url":"https://pokeapi.co/api/v2/move/159/"},{"name":"conversion","url":"https://pokeapi.co/api/v2/move/160/"},{"name":"substitute","url":"https://pokeapi.co/api/v2/move/164/"},{"name":"protect","url":"https://pokeapi.co/api/v2/move/182/"},{"name":"belly-drum","url":"https://pokeapi.co/api/v2/move/187/"},{"name":"destiny-bond","url":"https://pokeapi.co/api/v2/move/194/"},{"name":"detect","url":"https://pokeapi.co/api/v2/move/197/"},{"name":"endure","url":"https://pokeapi.co/api/v2/move/203/"},{"name":"milk-drink","url":"https://pokeapi.co/api/v2/move/208/"},{"name":"sleep-talk","url":"https://pokeapi.co/api/v2/move/214/"},{"name":"baton-pass","url":"https://pokeapi.co/api/v2/move/226/"},{"name":"morning-sun","url":"https://pokeapi.co/api/v2/move/234/"},{"name":"synthesis","url":"https://pokeapi.co/api/v2/move/235/"},{"name":"moonlight","url":"https://pokeapi.co/api/v2/move/236/"},{"name":"stockpile","url":"https://pokeapi.co/api/v2/move/254/"},{"name":"swallow","url":"https://pokeapi.co/api/v2/move/256/"},{"name":"follow-me","url":"https://pokeapi.co/api/v2/move/266/"},{"name":"charge","url":"https://pokeapi.co/api/v2/move/268/"},{"name":"wish","url":"https://pokeapi.co/api/v2/move/273/"},{"name":"assist","url":"https://pokeapi.co/api/v2/move/274/"},{"name":"ingrain","url":"https://pokeapi.co/api/v2/move/275/"},{"name":"magic-coat","url":"https://pokeapi.co/api/v2/move/277/"},{"name":"recycle","url":"https://pokeapi.co/api/v2/move/278/"},{"name":"imprison","url":"https://pokeapi.co/api/v2/move/286/"},{"name":"refresh","url":"https://pokeapi.co/api/v2/move/287/"},{"name":"grudge","url":"https://pokeapi.co/api/v2/move/288/"},{"name":"snatch","url":"https://pokeapi.co/api/v2/move/289/"},{"name":"camouflage","url":"https://pokeapi.co/api/v2/move/293/"},{"name":"tail-glow","url":"https://pokeapi.co/api/v2/move/294/"},{"name":"slack-off","url":"https://pokeapi.co/api/v2/move/303/"},{"name":"cosmic-power","url":"https://pokeapi.co/api/v2/move/322/"},{"name":"iron-defense","url":"https://pokeapi.co/api/v2/move/334/"},{"name":"howl","url":"https://pokeapi.co/api/v2/move/336/"},{"name":"bulk-up","url":"https://pokeapi.co/api/v2/move/339/"},{"name":"calm-mind","url":"https://pokeapi.co/api/v2/move/347/"},{"name":"dragon-dance","url":"https://pokeapi.co/api/v2/move/349/"},{"name":"roost","url":"https://pokeapi.co/api/v2/move/355/"},{"name":"healing-wish","url":"https://pokeapi.co/api/v2/move/361/"},{"name":"power-trick","url":"https://pokeapi.co/api/v2/move/379/"},{"name":"copycat","url":"https://pokeapi.co/api/v2/move/383/"},{"name":"aqua-ring","url":"https://pokeapi.co/api/v2/move/392/"},{"name":"magnet-rise","url":"https://pokeapi.co/api/v2/move/393/"},{"name":"rock-polish","url":"https://pokeapi.co/api/v2/move/397/"},{"name":"nasty-plot","url":"https://pokeapi.co/api/v2/move/417/"},{"name":"defend-order","url":"https://pokeapi.co/api/v2/move/455/"},{"name":"heal-order","url":"https://pokeapi.co/api/v2/move/456/"},{"name":"lunar-dance","url":"https://pokeapi.co/api/v2/move/461/"},{"name":"hone-claws","url":"https://pokeapi.co/api/v2/move/468/"},{"name":"autotomize","url":"https://pokeapi.co/api/v2/move/475/"},{"name":"rage-powder","url":"https://pokeapi.co/api/v2/move/476/"},{"name":"quiver-dance","url":"https://pokeapi.co/api/v2/move/483/"},{"name":"coil","url":"https://pokeapi.co/api/v2/move/489/"},{"name":"ally-switch","url":"https://pokeapi.co/api/v2/move/502/"},{"name":"shell-smash","url":"https://pokeapi.co/api/v2/move/504/"},{"name":"shift-gear","url":"https://pokeapi.co/api/v2/move/508/"},{"name":"work-up","url":"https://pokeapi.co/api/v2/move/526/"},{"name":"cotton-guard","url":"https://pokeapi.co/api/v2/move/538/"},{"name":"kings-shield","url":"https://pokeapi.co/api/v2/move/588/"},{"name":"spiky-shield","url":"https://pokeapi.co/api/v2/move/596/"},{"name":"geomancy","url":"https://pokeapi.co/api/v2/move/601/"},{"name":"celebrate","url":"https://pokeapi.co/api/v2/move/606/"},{"name":"shore-up","url":"https://pokeapi.co/api/v2/move/659/"},{"name":"baneful-bunker","url":"https://pokeapi.co/api/v2/move/661/"},{"name":"laser-focus","url":"https://pokeapi.co/api/v2/move/673/"},{"name":"extreme-evoboost","url":"https://pokeapi.co/api/v2/move/702/"}],"name":"user","names":[{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Anwender"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"User"}]}"#;

#[test]
fn move_target_from_id() {
    test_from_id::<MoveTarget>(7, USER_JSON);
}

#[test]
fn move_target_from_name() {
    test_from_name::<MoveTarget>("user", USER_JSON);
}

// https://pokeapi.co/api/v2/ability/dazzling
static DAZZLING_JSON: &str = r#"{"effect_changes":[],"effect_entries":[{"effect":"When an opposing Pokémon attempts to use a move that targets this Pokémon or an ally, and that move has priority, it will fail.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"short_effect":"Opposing Pokémon cannot use priority attacks."}],"flavor_text_entries":[{"flavor_text":"让对手吓一跳，\n使其无法对我方使出先制招式。","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"相手を　びっくり　させて\nこちらに　むかって　先制技を\n出せない　ようにする。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Surprises the opposing Pokémon, making it unable\nto attack using priority moves.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Il Pokémon sbalordisce il nemico e non gli permette\ndi attaccarlo con mosse che hanno priorità alta.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Desconcierta al rival y le impide utilizar movimientos\ncon prioridad contra el usuario.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"Überrascht den Gegner und hindert ihn so daran,\nAttacken mit hoher Erstschlagquote einzusetzen.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"L’adversaire est abasourdi par le Pokémon et\nne peut pas le viser avec une capacité prioritaire.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"讓對手嚇一跳，\n使其無法對我方使出先制招式。","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"상대를 놀라게 해서\n이쪽을 향한 선제 기술을\n사용할 수 없게 한다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"あいてを　びっくり　させて\nこちらに　むかって　せんせいわざを\nだせない　ようにする。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"flavor_text":"让对手吓一跳，\n使其无法对我方使出先制招式。","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"相手を　びっくり　させて\nこちらに　むかって　先制技を\n出せない　ようにする。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Surprises the opposing Pokémon, making it unable\nto attack using priority moves.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Il Pokémon sbalordisce il nemico e non gli permette\ndi attaccarlo con mosse che hanno priorità alta.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Desconcierta al rival y le impide utilizar movimientos\ncon prioridad contra el usuario.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"Überrascht den Gegner und hindert ihn so daran,\nAttacken mit hoher Erstschlagquote einzusetzen.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"L’adversaire est abasourdi par le Pokémon et\nne peut pas le viser avec une capacité prioritaire.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"讓對手嚇一跳，\n使其無法對我方使出先制招式。","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"상대를 놀라게 해서\n이쪽을 향한 선제 기술을\n사용할 수 없게 한다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"flavor_text":"あいてを　びっくり　させて\nこちらに　むかって　せんせいわざを\nだせない　ようにする。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}}],"generation":{"name":"generation-vii","url":"https://pokeapi.co/api/v2/generation/7/"},"id":219,"is_main_series":true,"name":"dazzling","names":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"name":"鲜艳之躯"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"ビビッドボディ"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Dazzling"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Corposgargiante"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Cuerpo Vívido"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Buntkörper"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Corps Coloré"},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"name":"鮮艷之軀"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"비비드바디"},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"ビビッドボディ"}],"pokemon":[{"is_hidden":false,"pokemon":{"name":"bruxish","url":"https://pokeapi.co/api/v2/pokemon/779/"},"slot":1}]}"#;

#[test]
fn ability_from_id() {
    test_from_id::<Ability>(219, DAZZLING_JSON);
}

#[test]
fn ability_from_name() {
    test_from_name::<Ability>("dazzling", DAZZLING_JSON);
}

// https://pokeapi.co/api/v2/characteristic/13/
static CHARACTERISTIC_13_JSON: &str = r#"{"descriptions":[{"description":"Dort beaucoup","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"}},{"description":"Nods off a lot","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}}],"gene_modulo":2,"highest_stat":{"name":"hp","url":"https://pokeapi.co/api/v2/stat/1/"},"id":13,"possible_values":[2,7,12,17,22,27]}"#;

#[test]
fn characteristic_from_id() {
    test_from_id::<Characteristic>(13, CHARACTERISTIC_13_JSON);
}

// https://pokeapi.co/api/v2/egg-group/11/
static INDETERMINATE_JSON: &str = r#"{"id":11,"name":"indeterminate","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"ふていけい"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Amorphe"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Amorph"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Amorfo"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Amorfo"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Amorphous"}],"pokemon_species":[{"name":"grimer","url":"https://pokeapi.co/api/v2/pokemon-species/88/"},{"name":"muk","url":"https://pokeapi.co/api/v2/pokemon-species/89/"},{"name":"gastly","url":"https://pokeapi.co/api/v2/pokemon-species/92/"},{"name":"haunter","url":"https://pokeapi.co/api/v2/pokemon-species/93/"},{"name":"gengar","url":"https://pokeapi.co/api/v2/pokemon-species/94/"},{"name":"koffing","url":"https://pokeapi.co/api/v2/pokemon-species/109/"},{"name":"weezing","url":"https://pokeapi.co/api/v2/pokemon-species/110/"},{"name":"misdreavus","url":"https://pokeapi.co/api/v2/pokemon-species/200/"},{"name":"wobbuffet","url":"https://pokeapi.co/api/v2/pokemon-species/202/"},{"name":"slugma","url":"https://pokeapi.co/api/v2/pokemon-species/218/"},{"name":"magcargo","url":"https://pokeapi.co/api/v2/pokemon-species/219/"},{"name":"ralts","url":"https://pokeapi.co/api/v2/pokemon-species/280/"},{"name":"kirlia","url":"https://pokeapi.co/api/v2/pokemon-species/281/"},{"name":"gardevoir","url":"https://pokeapi.co/api/v2/pokemon-species/282/"},{"name":"gulpin","url":"https://pokeapi.co/api/v2/pokemon-species/316/"},{"name":"swalot","url":"https://pokeapi.co/api/v2/pokemon-species/317/"},{"name":"castform","url":"https://pokeapi.co/api/v2/pokemon-species/351/"},{"name":"shuppet","url":"https://pokeapi.co/api/v2/pokemon-species/353/"},{"name":"banette","url":"https://pokeapi.co/api/v2/pokemon-species/354/"},{"name":"duskull","url":"https://pokeapi.co/api/v2/pokemon-species/355/"},{"name":"dusclops","url":"https://pokeapi.co/api/v2/pokemon-species/356/"},{"name":"chimecho","url":"https://pokeapi.co/api/v2/pokemon-species/358/"},{"name":"shellos","url":"https://pokeapi.co/api/v2/pokemon-species/422/"},{"name":"gastrodon","url":"https://pokeapi.co/api/v2/pokemon-species/423/"},{"name":"drifloon","url":"https://pokeapi.co/api/v2/pokemon-species/425/"},{"name":"drifblim","url":"https://pokeapi.co/api/v2/pokemon-species/426/"},{"name":"mismagius","url":"https://pokeapi.co/api/v2/pokemon-species/429/"},{"name":"spiritomb","url":"https://pokeapi.co/api/v2/pokemon-species/442/"},{"name":"gallade","url":"https://pokeapi.co/api/v2/pokemon-species/475/"},{"name":"dusknoir","url":"https://pokeapi.co/api/v2/pokemon-species/477/"},{"name":"rotom","url":"https://pokeapi.co/api/v2/pokemon-species/479/"},{"name":"yamask","url":"https://pokeapi.co/api/v2/pokemon-species/562/"},{"name":"cofagrigus","url":"https://pokeapi.co/api/v2/pokemon-species/563/"},{"name":"solosis","url":"https://pokeapi.co/api/v2/pokemon-species/577/"},{"name":"duosion","url":"https://pokeapi.co/api/v2/pokemon-species/578/"},{"name":"reuniclus","url":"https://pokeapi.co/api/v2/pokemon-species/579/"},{"name":"frillish","url":"https://pokeapi.co/api/v2/pokemon-species/592/"},{"name":"jellicent","url":"https://pokeapi.co/api/v2/pokemon-species/593/"},{"name":"tynamo","url":"https://pokeapi.co/api/v2/pokemon-species/602/"},{"name":"eelektrik","url":"https://pokeapi.co/api/v2/pokemon-species/603/"},{"name":"eelektross","url":"https://pokeapi.co/api/v2/pokemon-species/604/"},{"name":"litwick","url":"https://pokeapi.co/api/v2/pokemon-species/607/"},{"name":"lampent","url":"https://pokeapi.co/api/v2/pokemon-species/608/"},{"name":"chandelure","url":"https://pokeapi.co/api/v2/pokemon-species/609/"},{"name":"stunfisk","url":"https://pokeapi.co/api/v2/pokemon-species/618/"},{"name":"phantump","url":"https://pokeapi.co/api/v2/pokemon-species/708/"},{"name":"trevenant","url":"https://pokeapi.co/api/v2/pokemon-species/709/"},{"name":"pumpkaboo","url":"https://pokeapi.co/api/v2/pokemon-species/710/"},{"name":"gourgeist","url":"https://pokeapi.co/api/v2/pokemon-species/711/"},{"name":"sandygast","url":"https://pokeapi.co/api/v2/pokemon-species/769/"},{"name":"palossand","url":"https://pokeapi.co/api/v2/pokemon-species/770/"},{"name":"mimikyu","url":"https://pokeapi.co/api/v2/pokemon-species/778/"}]}"#;

#[test]
fn egg_group_from_id() {
    test_from_id::<EggGroup>(11, INDETERMINATE_JSON);
}

#[test]
fn egg_group_from_name() {
    test_from_name::<EggGroup>("indeterminate", INDETERMINATE_JSON);
}

// https://pokeapi.co/api/v2/gender/3/
static GENDERLESS_JSON: &str = r#"{"id":3,"name":"genderless","pokemon_species_details":[{"pokemon_species":{"name":"magnemite","url":"https://pokeapi.co/api/v2/pokemon-species/81/"},"rate":-1},{"pokemon_species":{"name":"voltorb","url":"https://pokeapi.co/api/v2/pokemon-species/100/"},"rate":-1},{"pokemon_species":{"name":"staryu","url":"https://pokeapi.co/api/v2/pokemon-species/120/"},"rate":-1},{"pokemon_species":{"name":"ditto","url":"https://pokeapi.co/api/v2/pokemon-species/132/"},"rate":-1},{"pokemon_species":{"name":"porygon","url":"https://pokeapi.co/api/v2/pokemon-species/137/"},"rate":-1},{"pokemon_species":{"name":"articuno","url":"https://pokeapi.co/api/v2/pokemon-species/144/"},"rate":-1},{"pokemon_species":{"name":"zapdos","url":"https://pokeapi.co/api/v2/pokemon-species/145/"},"rate":-1},{"pokemon_species":{"name":"moltres","url":"https://pokeapi.co/api/v2/pokemon-species/146/"},"rate":-1},{"pokemon_species":{"name":"mewtwo","url":"https://pokeapi.co/api/v2/pokemon-species/150/"},"rate":-1},{"pokemon_species":{"name":"mew","url":"https://pokeapi.co/api/v2/pokemon-species/151/"},"rate":-1},{"pokemon_species":{"name":"magneton","url":"https://pokeapi.co/api/v2/pokemon-species/82/"},"rate":-1},{"pokemon_species":{"name":"electrode","url":"https://pokeapi.co/api/v2/pokemon-species/101/"},"rate":-1},{"pokemon_species":{"name":"starmie","url":"https://pokeapi.co/api/v2/pokemon-species/121/"},"rate":-1},{"pokemon_species":{"name":"unown","url":"https://pokeapi.co/api/v2/pokemon-species/201/"},"rate":-1},{"pokemon_species":{"name":"porygon2","url":"https://pokeapi.co/api/v2/pokemon-species/233/"},"rate":-1},{"pokemon_species":{"name":"raikou","url":"https://pokeapi.co/api/v2/pokemon-species/243/"},"rate":-1},{"pokemon_species":{"name":"entei","url":"https://pokeapi.co/api/v2/pokemon-species/244/"},"rate":-1},{"pokemon_species":{"name":"suicune","url":"https://pokeapi.co/api/v2/pokemon-species/245/"},"rate":-1},{"pokemon_species":{"name":"lugia","url":"https://pokeapi.co/api/v2/pokemon-species/249/"},"rate":-1},{"pokemon_species":{"name":"ho-oh","url":"https://pokeapi.co/api/v2/pokemon-species/250/"},"rate":-1},{"pokemon_species":{"name":"celebi","url":"https://pokeapi.co/api/v2/pokemon-species/251/"},"rate":-1},{"pokemon_species":{"name":"shedinja","url":"https://pokeapi.co/api/v2/pokemon-species/292/"},"rate":-1},{"pokemon_species":{"name":"lunatone","url":"https://pokeapi.co/api/v2/pokemon-species/337/"},"rate":-1},{"pokemon_species":{"name":"solrock","url":"https://pokeapi.co/api/v2/pokemon-species/338/"},"rate":-1},{"pokemon_species":{"name":"baltoy","url":"https://pokeapi.co/api/v2/pokemon-species/343/"},"rate":-1},{"pokemon_species":{"name":"beldum","url":"https://pokeapi.co/api/v2/pokemon-species/374/"},"rate":-1},{"pokemon_species":{"name":"regirock","url":"https://pokeapi.co/api/v2/pokemon-species/377/"},"rate":-1},{"pokemon_species":{"name":"regice","url":"https://pokeapi.co/api/v2/pokemon-species/378/"},"rate":-1},{"pokemon_species":{"name":"registeel","url":"https://pokeapi.co/api/v2/pokemon-species/379/"},"rate":-1},{"pokemon_species":{"name":"kyogre","url":"https://pokeapi.co/api/v2/pokemon-species/382/"},"rate":-1},{"pokemon_species":{"name":"groudon","url":"https://pokeapi.co/api/v2/pokemon-species/383/"},"rate":-1},{"pokemon_species":{"name":"rayquaza","url":"https://pokeapi.co/api/v2/pokemon-species/384/"},"rate":-1},{"pokemon_species":{"name":"jirachi","url":"https://pokeapi.co/api/v2/pokemon-species/385/"},"rate":-1},{"pokemon_species":{"name":"deoxys","url":"https://pokeapi.co/api/v2/pokemon-species/386/"},"rate":-1},{"pokemon_species":{"name":"claydol","url":"https://pokeapi.co/api/v2/pokemon-species/344/"},"rate":-1},{"pokemon_species":{"name":"metang","url":"https://pokeapi.co/api/v2/pokemon-species/375/"},"rate":-1},{"pokemon_species":{"name":"metagross","url":"https://pokeapi.co/api/v2/pokemon-species/376/"},"rate":-1},{"pokemon_species":{"name":"bronzor","url":"https://pokeapi.co/api/v2/pokemon-species/436/"},"rate":-1},{"pokemon_species":{"name":"bronzong","url":"https://pokeapi.co/api/v2/pokemon-species/437/"},"rate":-1},{"pokemon_species":{"name":"magnezone","url":"https://pokeapi.co/api/v2/pokemon-species/462/"},"rate":-1},{"pokemon_species":{"name":"porygon-z","url":"https://pokeapi.co/api/v2/pokemon-species/474/"},"rate":-1},{"pokemon_species":{"name":"rotom","url":"https://pokeapi.co/api/v2/pokemon-species/479/"},"rate":-1},{"pokemon_species":{"name":"uxie","url":"https://pokeapi.co/api/v2/pokemon-species/480/"},"rate":-1},{"pokemon_species":{"name":"mesprit","url":"https://pokeapi.co/api/v2/pokemon-species/481/"},"rate":-1},{"pokemon_species":{"name":"azelf","url":"https://pokeapi.co/api/v2/pokemon-species/482/"},"rate":-1},{"pokemon_species":{"name":"dialga","url":"https://pokeapi.co/api/v2/pokemon-species/483/"},"rate":-1},{"pokemon_species":{"name":"palkia","url":"https://pokeapi.co/api/v2/pokemon-species/484/"},"rate":-1},{"pokemon_species":{"name":"regigigas","url":"https://pokeapi.co/api/v2/pokemon-species/486/"},"rate":-1},{"pokemon_species":{"name":"giratina","url":"https://pokeapi.co/api/v2/pokemon-species/487/"},"rate":-1},{"pokemon_species":{"name":"phione","url":"https://pokeapi.co/api/v2/pokemon-species/489/"},"rate":-1},{"pokemon_species":{"name":"manaphy","url":"https://pokeapi.co/api/v2/pokemon-species/490/"},"rate":-1},{"pokemon_species":{"name":"darkrai","url":"https://pokeapi.co/api/v2/pokemon-species/491/"},"rate":-1},{"pokemon_species":{"name":"shaymin","url":"https://pokeapi.co/api/v2/pokemon-species/492/"},"rate":-1},{"pokemon_species":{"name":"arceus","url":"https://pokeapi.co/api/v2/pokemon-species/493/"},"rate":-1},{"pokemon_species":{"name":"victini","url":"https://pokeapi.co/api/v2/pokemon-species/494/"},"rate":-1},{"pokemon_species":{"name":"klink","url":"https://pokeapi.co/api/v2/pokemon-species/599/"},"rate":-1},{"pokemon_species":{"name":"cryogonal","url":"https://pokeapi.co/api/v2/pokemon-species/615/"},"rate":-1},{"pokemon_species":{"name":"golett","url":"https://pokeapi.co/api/v2/pokemon-species/622/"},"rate":-1},{"pokemon_species":{"name":"klang","url":"https://pokeapi.co/api/v2/pokemon-species/600/"},"rate":-1},{"pokemon_species":{"name":"klinklang","url":"https://pokeapi.co/api/v2/pokemon-species/601/"},"rate":-1},{"pokemon_species":{"name":"golurk","url":"https://pokeapi.co/api/v2/pokemon-species/623/"},"rate":-1},{"pokemon_species":{"name":"cobalion","url":"https://pokeapi.co/api/v2/pokemon-species/638/"},"rate":-1},{"pokemon_species":{"name":"terrakion","url":"https://pokeapi.co/api/v2/pokemon-species/639/"},"rate":-1},{"pokemon_species":{"name":"virizion","url":"https://pokeapi.co/api/v2/pokemon-species/640/"},"rate":-1},{"pokemon_species":{"name":"reshiram","url":"https://pokeapi.co/api/v2/pokemon-species/643/"},"rate":-1},{"pokemon_species":{"name":"zekrom","url":"https://pokeapi.co/api/v2/pokemon-species/644/"},"rate":-1},{"pokemon_species":{"name":"kyurem","url":"https://pokeapi.co/api/v2/pokemon-species/646/"},"rate":-1},{"pokemon_species":{"name":"keldeo","url":"https://pokeapi.co/api/v2/pokemon-species/647/"},"rate":-1},{"pokemon_species":{"name":"meloetta","url":"https://pokeapi.co/api/v2/pokemon-species/648/"},"rate":-1},{"pokemon_species":{"name":"genesect","url":"https://pokeapi.co/api/v2/pokemon-species/649/"},"rate":-1},{"pokemon_species":{"name":"carbink","url":"https://pokeapi.co/api/v2/pokemon-species/703/"},"rate":-1},{"pokemon_species":{"name":"xerneas","url":"https://pokeapi.co/api/v2/pokemon-species/716/"},"rate":-1},{"pokemon_species":{"name":"yveltal","url":"https://pokeapi.co/api/v2/pokemon-species/717/"},"rate":-1},{"pokemon_species":{"name":"zygarde","url":"https://pokeapi.co/api/v2/pokemon-species/718/"},"rate":-1},{"pokemon_species":{"name":"diancie","url":"https://pokeapi.co/api/v2/pokemon-species/719/"},"rate":-1},{"pokemon_species":{"name":"hoopa","url":"https://pokeapi.co/api/v2/pokemon-species/720/"},"rate":-1},{"pokemon_species":{"name":"volcanion","url":"https://pokeapi.co/api/v2/pokemon-species/721/"},"rate":-1},{"pokemon_species":{"name":"type-null","url":"https://pokeapi.co/api/v2/pokemon-species/772/"},"rate":-1},{"pokemon_species":{"name":"minior","url":"https://pokeapi.co/api/v2/pokemon-species/774/"},"rate":-1},{"pokemon_species":{"name":"dhelmise","url":"https://pokeapi.co/api/v2/pokemon-species/781/"},"rate":-1},{"pokemon_species":{"name":"tapu-koko","url":"https://pokeapi.co/api/v2/pokemon-species/785/"},"rate":-1},{"pokemon_species":{"name":"tapu-lele","url":"https://pokeapi.co/api/v2/pokemon-species/786/"},"rate":-1},{"pokemon_species":{"name":"tapu-bulu","url":"https://pokeapi.co/api/v2/pokemon-species/787/"},"rate":-1},{"pokemon_species":{"name":"tapu-fini","url":"https://pokeapi.co/api/v2/pokemon-species/788/"},"rate":-1},{"pokemon_species":{"name":"cosmog","url":"https://pokeapi.co/api/v2/pokemon-species/789/"},"rate":-1},{"pokemon_species":{"name":"nihilego","url":"https://pokeapi.co/api/v2/pokemon-species/793/"},"rate":-1},{"pokemon_species":{"name":"buzzwole","url":"https://pokeapi.co/api/v2/pokemon-species/794/"},"rate":-1},{"pokemon_species":{"name":"pheromosa","url":"https://pokeapi.co/api/v2/pokemon-species/795/"},"rate":-1},{"pokemon_species":{"name":"xurkitree","url":"https://pokeapi.co/api/v2/pokemon-species/796/"},"rate":-1},{"pokemon_species":{"name":"celesteela","url":"https://pokeapi.co/api/v2/pokemon-species/797/"},"rate":-1},{"pokemon_species":{"name":"kartana","url":"https://pokeapi.co/api/v2/pokemon-species/798/"},"rate":-1},{"pokemon_species":{"name":"silvally","url":"https://pokeapi.co/api/v2/pokemon-species/773/"},"rate":-1},{"pokemon_species":{"name":"solgaleo","url":"https://pokeapi.co/api/v2/pokemon-species/791/"},"rate":-1},{"pokemon_species":{"name":"lunala","url":"https://pokeapi.co/api/v2/pokemon-species/792/"},"rate":-1},{"pokemon_species":{"name":"guzzlord","url":"https://pokeapi.co/api/v2/pokemon-species/799/"},"rate":-1},{"pokemon_species":{"name":"necrozma","url":"https://pokeapi.co/api/v2/pokemon-species/800/"},"rate":-1},{"pokemon_species":{"name":"magearna","url":"https://pokeapi.co/api/v2/pokemon-species/801/"},"rate":-1},{"pokemon_species":{"name":"marshadow","url":"https://pokeapi.co/api/v2/pokemon-species/802/"},"rate":-1},{"pokemon_species":{"name":"poipole","url":"https://pokeapi.co/api/v2/pokemon-species/803/"},"rate":-1},{"pokemon_species":{"name":"stakataka","url":"https://pokeapi.co/api/v2/pokemon-species/805/"},"rate":-1},{"pokemon_species":{"name":"blacephalon","url":"https://pokeapi.co/api/v2/pokemon-species/806/"},"rate":-1},{"pokemon_species":{"name":"zeraora","url":"https://pokeapi.co/api/v2/pokemon-species/807/"},"rate":-1},{"pokemon_species":{"name":"cosmoem","url":"https://pokeapi.co/api/v2/pokemon-species/790/"},"rate":-1},{"pokemon_species":{"name":"naganadel","url":"https://pokeapi.co/api/v2/pokemon-species/804/"},"rate":-1}],"required_for_evolution":[]}"#;

#[test]
fn gender_from_id() {
    test_from_id::<Gender>(3, GENDERLESS_JSON);
}

#[test]
fn gender_from_name() {
    test_from_name::<Gender>("genderless", GENDERLESS_JSON);
}

// https://pokeapi.co/api/v2/growth-rate/4/
static MEDIUM_SLOW_JSON: &str = r#"{"descriptions":[{"description":"medium slow","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"description":"mittel langsam","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"}},{"description":"parabolique","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"}}],"formula":"\\frac{6x^3}{5} - 15x^2 + 100x - 140","id":4,"levels":[{"experience":1059860,"level":100},{"experience":1027103,"level":99},{"experience":995030,"level":98},{"experience":963632,"level":97},{"experience":932903,"level":96},{"experience":902835,"level":95},{"experience":873420,"level":94},{"experience":844653,"level":93},{"experience":816525,"level":92},{"experience":789030,"level":91},{"experience":762160,"level":90},{"experience":735907,"level":89},{"experience":710266,"level":88},{"experience":685228,"level":87},{"experience":660787,"level":86},{"experience":636935,"level":85},{"experience":613664,"level":84},{"experience":590969,"level":83},{"experience":568841,"level":82},{"experience":547274,"level":81},{"experience":526260,"level":80},{"experience":505791,"level":79},{"experience":485862,"level":78},{"experience":466464,"level":77},{"experience":447591,"level":76},{"experience":429235,"level":75},{"experience":411388,"level":74},{"experience":394045,"level":73},{"experience":377197,"level":72},{"experience":360838,"level":71},{"experience":344960,"level":70},{"experience":329555,"level":69},{"experience":314618,"level":68},{"experience":300140,"level":67},{"experience":286115,"level":66},{"experience":272535,"level":65},{"experience":259392,"level":64},{"experience":246681,"level":63},{"experience":234393,"level":62},{"experience":222522,"level":61},{"experience":211060,"level":60},{"experience":199999,"level":59},{"experience":189334,"level":58},{"experience":179056,"level":57},{"experience":169159,"level":56},{"experience":159635,"level":55},{"experience":150476,"level":54},{"experience":141677,"level":53},{"experience":133229,"level":52},{"experience":125126,"level":51},{"experience":117360,"level":50},{"experience":109923,"level":49},{"experience":102810,"level":48},{"experience":96012,"level":47},{"experience":89523,"level":46},{"experience":83335,"level":45},{"experience":77440,"level":44},{"experience":71833,"level":43},{"experience":66505,"level":42},{"experience":61450,"level":41},{"experience":56660,"level":40},{"experience":52127,"level":39},{"experience":47846,"level":38},{"experience":43808,"level":37},{"experience":40007,"level":36},{"experience":36435,"level":35},{"experience":33084,"level":34},{"experience":29949,"level":33},{"experience":27021,"level":32},{"experience":24294,"level":31},{"experience":21760,"level":30},{"experience":19411,"level":29},{"experience":17242,"level":28},{"experience":15244,"level":27},{"experience":13411,"level":26},{"experience":11735,"level":25},{"experience":10208,"level":24},{"experience":8825,"level":23},{"experience":7577,"level":22},{"experience":6458,"level":21},{"experience":5460,"level":20},{"experience":4575,"level":19},{"experience":3798,"level":18},{"experience":3120,"level":17},{"experience":2535,"level":16},{"experience":2035,"level":15},{"experience":1612,"level":14},{"experience":1261,"level":13},{"experience":973,"level":12},{"experience":742,"level":11},{"experience":560,"level":10},{"experience":419,"level":9},{"experience":314,"level":8},{"experience":236,"level":7},{"experience":179,"level":6},{"experience":135,"level":5},{"experience":96,"level":4},{"experience":57,"level":3},{"experience":9,"level":2},{"experience":0,"level":1}],"name":"medium-slow","pokemon_species":[{"name":"bulbasaur","url":"https://pokeapi.co/api/v2/pokemon-species/1/"},{"name":"charmander","url":"https://pokeapi.co/api/v2/pokemon-species/4/"},{"name":"squirtle","url":"https://pokeapi.co/api/v2/pokemon-species/7/"},{"name":"pidgey","url":"https://pokeapi.co/api/v2/pokemon-species/16/"},{"name":"nidoran-f","url":"https://pokeapi.co/api/v2/pokemon-species/29/"},{"name":"nidoran-m","url":"https://pokeapi.co/api/v2/pokemon-species/32/"},{"name":"oddish","url":"https://pokeapi.co/api/v2/pokemon-species/43/"},{"name":"poliwag","url":"https://pokeapi.co/api/v2/pokemon-species/60/"},{"name":"abra","url":"https://pokeapi.co/api/v2/pokemon-species/63/"},{"name":"machop","url":"https://pokeapi.co/api/v2/pokemon-species/66/"},{"name":"bellsprout","url":"https://pokeapi.co/api/v2/pokemon-species/69/"},{"name":"geodude","url":"https://pokeapi.co/api/v2/pokemon-species/74/"},{"name":"venusaur","url":"https://pokeapi.co/api/v2/pokemon-species/3/"},{"name":"charmeleon","url":"https://pokeapi.co/api/v2/pokemon-species/5/"},{"name":"charizard","url":"https://pokeapi.co/api/v2/pokemon-species/6/"},{"name":"wartortle","url":"https://pokeapi.co/api/v2/pokemon-species/8/"},{"name":"blastoise","url":"https://pokeapi.co/api/v2/pokemon-species/9/"},{"name":"pidgeotto","url":"https://pokeapi.co/api/v2/pokemon-species/17/"},{"name":"pidgeot","url":"https://pokeapi.co/api/v2/pokemon-species/18/"},{"name":"nidorina","url":"https://pokeapi.co/api/v2/pokemon-species/30/"},{"name":"nidoqueen","url":"https://pokeapi.co/api/v2/pokemon-species/31/"},{"name":"nidorino","url":"https://pokeapi.co/api/v2/pokemon-species/33/"},{"name":"nidoking","url":"https://pokeapi.co/api/v2/pokemon-species/34/"},{"name":"gloom","url":"https://pokeapi.co/api/v2/pokemon-species/44/"},{"name":"vileplume","url":"https://pokeapi.co/api/v2/pokemon-species/45/"},{"name":"poliwhirl","url":"https://pokeapi.co/api/v2/pokemon-species/61/"},{"name":"poliwrath","url":"https://pokeapi.co/api/v2/pokemon-species/62/"},{"name":"kadabra","url":"https://pokeapi.co/api/v2/pokemon-species/64/"},{"name":"alakazam","url":"https://pokeapi.co/api/v2/pokemon-species/65/"},{"name":"machoke","url":"https://pokeapi.co/api/v2/pokemon-species/67/"},{"name":"machamp","url":"https://pokeapi.co/api/v2/pokemon-species/68/"},{"name":"weepinbell","url":"https://pokeapi.co/api/v2/pokemon-species/70/"},{"name":"victreebel","url":"https://pokeapi.co/api/v2/pokemon-species/71/"},{"name":"graveler","url":"https://pokeapi.co/api/v2/pokemon-species/75/"},{"name":"golem","url":"https://pokeapi.co/api/v2/pokemon-species/76/"},{"name":"gastly","url":"https://pokeapi.co/api/v2/pokemon-species/92/"},{"name":"mew","url":"https://pokeapi.co/api/v2/pokemon-species/151/"},{"name":"chikorita","url":"https://pokeapi.co/api/v2/pokemon-species/152/"},{"name":"cyndaquil","url":"https://pokeapi.co/api/v2/pokemon-species/155/"},{"name":"haunter","url":"https://pokeapi.co/api/v2/pokemon-species/93/"},{"name":"gengar","url":"https://pokeapi.co/api/v2/pokemon-species/94/"},{"name":"bayleef","url":"https://pokeapi.co/api/v2/pokemon-species/153/"},{"name":"meganium","url":"https://pokeapi.co/api/v2/pokemon-species/154/"},{"name":"quilava","url":"https://pokeapi.co/api/v2/pokemon-species/156/"},{"name":"typhlosion","url":"https://pokeapi.co/api/v2/pokemon-species/157/"},{"name":"totodile","url":"https://pokeapi.co/api/v2/pokemon-species/158/"},{"name":"mareep","url":"https://pokeapi.co/api/v2/pokemon-species/179/"},{"name":"hoppip","url":"https://pokeapi.co/api/v2/pokemon-species/187/"},{"name":"sunkern","url":"https://pokeapi.co/api/v2/pokemon-species/191/"},{"name":"murkrow","url":"https://pokeapi.co/api/v2/pokemon-species/198/"},{"name":"gligar","url":"https://pokeapi.co/api/v2/pokemon-species/207/"},{"name":"shuckle","url":"https://pokeapi.co/api/v2/pokemon-species/213/"},{"name":"sneasel","url":"https://pokeapi.co/api/v2/pokemon-species/215/"},{"name":"feraligatr","url":"https://pokeapi.co/api/v2/pokemon-species/160/"},{"name":"flaaffy","url":"https://pokeapi.co/api/v2/pokemon-species/180/"},{"name":"ampharos","url":"https://pokeapi.co/api/v2/pokemon-species/181/"},{"name":"bellossom","url":"https://pokeapi.co/api/v2/pokemon-species/182/"},{"name":"politoed","url":"https://pokeapi.co/api/v2/pokemon-species/186/"},{"name":"skiploom","url":"https://pokeapi.co/api/v2/pokemon-species/188/"},{"name":"jumpluff","url":"https://pokeapi.co/api/v2/pokemon-species/189/"},{"name":"sunflora","url":"https://pokeapi.co/api/v2/pokemon-species/192/"},{"name":"celebi","url":"https://pokeapi.co/api/v2/pokemon-species/251/"},{"name":"treecko","url":"https://pokeapi.co/api/v2/pokemon-species/252/"},{"name":"torchic","url":"https://pokeapi.co/api/v2/pokemon-species/255/"},{"name":"mudkip","url":"https://pokeapi.co/api/v2/pokemon-species/258/"},{"name":"lotad","url":"https://pokeapi.co/api/v2/pokemon-species/270/"},{"name":"seedot","url":"https://pokeapi.co/api/v2/pokemon-species/273/"},{"name":"taillow","url":"https://pokeapi.co/api/v2/pokemon-species/276/"},{"name":"whismur","url":"https://pokeapi.co/api/v2/pokemon-species/293/"},{"name":"sableye","url":"https://pokeapi.co/api/v2/pokemon-species/302/"},{"name":"grovyle","url":"https://pokeapi.co/api/v2/pokemon-species/253/"},{"name":"sceptile","url":"https://pokeapi.co/api/v2/pokemon-species/254/"},{"name":"combusken","url":"https://pokeapi.co/api/v2/pokemon-species/256/"},{"name":"blaziken","url":"https://pokeapi.co/api/v2/pokemon-species/257/"},{"name":"marshtomp","url":"https://pokeapi.co/api/v2/pokemon-species/259/"},{"name":"swampert","url":"https://pokeapi.co/api/v2/pokemon-species/260/"},{"name":"lombre","url":"https://pokeapi.co/api/v2/pokemon-species/271/"},{"name":"ludicolo","url":"https://pokeapi.co/api/v2/pokemon-species/272/"},{"name":"nuzleaf","url":"https://pokeapi.co/api/v2/pokemon-species/274/"},{"name":"shiftry","url":"https://pokeapi.co/api/v2/pokemon-species/275/"},{"name":"swellow","url":"https://pokeapi.co/api/v2/pokemon-species/277/"},{"name":"loudred","url":"https://pokeapi.co/api/v2/pokemon-species/294/"},{"name":"exploud","url":"https://pokeapi.co/api/v2/pokemon-species/295/"},{"name":"trapinch","url":"https://pokeapi.co/api/v2/pokemon-species/328/"},{"name":"cacnea","url":"https://pokeapi.co/api/v2/pokemon-species/331/"},{"name":"kecleon","url":"https://pokeapi.co/api/v2/pokemon-species/352/"},{"name":"absol","url":"https://pokeapi.co/api/v2/pokemon-species/359/"},{"name":"spheal","url":"https://pokeapi.co/api/v2/pokemon-species/363/"},{"name":"turtwig","url":"https://pokeapi.co/api/v2/pokemon-species/387/"},{"name":"chimchar","url":"https://pokeapi.co/api/v2/pokemon-species/390/"},{"name":"piplup","url":"https://pokeapi.co/api/v2/pokemon-species/393/"},{"name":"vibrava","url":"https://pokeapi.co/api/v2/pokemon-species/329/"},{"name":"flygon","url":"https://pokeapi.co/api/v2/pokemon-species/330/"},{"name":"cacturne","url":"https://pokeapi.co/api/v2/pokemon-species/332/"},{"name":"sealeo","url":"https://pokeapi.co/api/v2/pokemon-species/364/"},{"name":"walrein","url":"https://pokeapi.co/api/v2/pokemon-species/365/"},{"name":"grotle","url":"https://pokeapi.co/api/v2/pokemon-species/388/"},{"name":"torterra","url":"https://pokeapi.co/api/v2/pokemon-species/389/"},{"name":"monferno","url":"https://pokeapi.co/api/v2/pokemon-species/391/"},{"name":"infernape","url":"https://pokeapi.co/api/v2/pokemon-species/392/"},{"name":"starly","url":"https://pokeapi.co/api/v2/pokemon-species/396/"},{"name":"kricketot","url":"https://pokeapi.co/api/v2/pokemon-species/401/"},{"name":"shinx","url":"https://pokeapi.co/api/v2/pokemon-species/403/"},{"name":"budew","url":"https://pokeapi.co/api/v2/pokemon-species/406/"},{"name":"combee","url":"https://pokeapi.co/api/v2/pokemon-species/415/"},{"name":"chatot","url":"https://pokeapi.co/api/v2/pokemon-species/441/"},{"name":"riolu","url":"https://pokeapi.co/api/v2/pokemon-species/447/"},{"name":"empoleon","url":"https://pokeapi.co/api/v2/pokemon-species/395/"},{"name":"staravia","url":"https://pokeapi.co/api/v2/pokemon-species/397/"},{"name":"staraptor","url":"https://pokeapi.co/api/v2/pokemon-species/398/"},{"name":"kricketune","url":"https://pokeapi.co/api/v2/pokemon-species/402/"},{"name":"luxio","url":"https://pokeapi.co/api/v2/pokemon-species/404/"},{"name":"luxray","url":"https://pokeapi.co/api/v2/pokemon-species/405/"},{"name":"vespiquen","url":"https://pokeapi.co/api/v2/pokemon-species/416/"},{"name":"honchkrow","url":"https://pokeapi.co/api/v2/pokemon-species/430/"},{"name":"lucario","url":"https://pokeapi.co/api/v2/pokemon-species/448/"},{"name":"weavile","url":"https://pokeapi.co/api/v2/pokemon-species/461/"},{"name":"gliscor","url":"https://pokeapi.co/api/v2/pokemon-species/472/"},{"name":"shaymin","url":"https://pokeapi.co/api/v2/pokemon-species/492/"},{"name":"snivy","url":"https://pokeapi.co/api/v2/pokemon-species/495/"},{"name":"tepig","url":"https://pokeapi.co/api/v2/pokemon-species/498/"},{"name":"oshawott","url":"https://pokeapi.co/api/v2/pokemon-species/501/"},{"name":"lillipup","url":"https://pokeapi.co/api/v2/pokemon-species/506/"},{"name":"pidove","url":"https://pokeapi.co/api/v2/pokemon-species/519/"},{"name":"roggenrola","url":"https://pokeapi.co/api/v2/pokemon-species/524/"},{"name":"timburr","url":"https://pokeapi.co/api/v2/pokemon-species/532/"},{"name":"tympole","url":"https://pokeapi.co/api/v2/pokemon-species/535/"},{"name":"sewaddle","url":"https://pokeapi.co/api/v2/pokemon-species/540/"},{"name":"venipede","url":"https://pokeapi.co/api/v2/pokemon-species/543/"},{"name":"sandile","url":"https://pokeapi.co/api/v2/pokemon-species/551/"},{"name":"darumaka","url":"https://pokeapi.co/api/v2/pokemon-species/554/"},{"name":"servine","url":"https://pokeapi.co/api/v2/pokemon-species/496/"},{"name":"serperior","url":"https://pokeapi.co/api/v2/pokemon-species/497/"},{"name":"pignite","url":"https://pokeapi.co/api/v2/pokemon-species/499/"},{"name":"emboar","url":"https://pokeapi.co/api/v2/pokemon-species/500/"},{"name":"dewott","url":"https://pokeapi.co/api/v2/pokemon-species/502/"},{"name":"herdier","url":"https://pokeapi.co/api/v2/pokemon-species/507/"},{"name":"stoutland","url":"https://pokeapi.co/api/v2/pokemon-species/508/"},{"name":"tranquill","url":"https://pokeapi.co/api/v2/pokemon-species/520/"},{"name":"unfezant","url":"https://pokeapi.co/api/v2/pokemon-species/521/"},{"name":"boldore","url":"https://pokeapi.co/api/v2/pokemon-species/525/"},{"name":"gigalith","url":"https://pokeapi.co/api/v2/pokemon-species/526/"},{"name":"gurdurr","url":"https://pokeapi.co/api/v2/pokemon-species/533/"},{"name":"conkeldurr","url":"https://pokeapi.co/api/v2/pokemon-species/534/"},{"name":"palpitoad","url":"https://pokeapi.co/api/v2/pokemon-species/536/"},{"name":"seismitoad","url":"https://pokeapi.co/api/v2/pokemon-species/537/"},{"name":"swadloon","url":"https://pokeapi.co/api/v2/pokemon-species/541/"},{"name":"leavanny","url":"https://pokeapi.co/api/v2/pokemon-species/542/"},{"name":"scolipede","url":"https://pokeapi.co/api/v2/pokemon-species/545/"},{"name":"krokorok","url":"https://pokeapi.co/api/v2/pokemon-species/552/"},{"name":"krookodile","url":"https://pokeapi.co/api/v2/pokemon-species/553/"},{"name":"darmanitan","url":"https://pokeapi.co/api/v2/pokemon-species/555/"},{"name":"zorua","url":"https://pokeapi.co/api/v2/pokemon-species/570/"},{"name":"gothita","url":"https://pokeapi.co/api/v2/pokemon-species/574/"},{"name":"solosis","url":"https://pokeapi.co/api/v2/pokemon-species/577/"},{"name":"klink","url":"https://pokeapi.co/api/v2/pokemon-species/599/"},{"name":"litwick","url":"https://pokeapi.co/api/v2/pokemon-species/607/"},{"name":"mienfoo","url":"https://pokeapi.co/api/v2/pokemon-species/619/"},{"name":"zoroark","url":"https://pokeapi.co/api/v2/pokemon-species/571/"},{"name":"gothitelle","url":"https://pokeapi.co/api/v2/pokemon-species/576/"},{"name":"duosion","url":"https://pokeapi.co/api/v2/pokemon-species/578/"},{"name":"reuniclus","url":"https://pokeapi.co/api/v2/pokemon-species/579/"},{"name":"klang","url":"https://pokeapi.co/api/v2/pokemon-species/600/"},{"name":"klinklang","url":"https://pokeapi.co/api/v2/pokemon-species/601/"},{"name":"lampent","url":"https://pokeapi.co/api/v2/pokemon-species/608/"},{"name":"chandelure","url":"https://pokeapi.co/api/v2/pokemon-species/609/"},{"name":"mienshao","url":"https://pokeapi.co/api/v2/pokemon-species/620/"},{"name":"chespin","url":"https://pokeapi.co/api/v2/pokemon-species/650/"},{"name":"fennekin","url":"https://pokeapi.co/api/v2/pokemon-species/653/"},{"name":"froakie","url":"https://pokeapi.co/api/v2/pokemon-species/656/"},{"name":"fletchling","url":"https://pokeapi.co/api/v2/pokemon-species/661/"},{"name":"litleo","url":"https://pokeapi.co/api/v2/pokemon-species/667/"},{"name":"quilladin","url":"https://pokeapi.co/api/v2/pokemon-species/651/"},{"name":"chesnaught","url":"https://pokeapi.co/api/v2/pokemon-species/652/"},{"name":"braixen","url":"https://pokeapi.co/api/v2/pokemon-species/654/"},{"name":"delphox","url":"https://pokeapi.co/api/v2/pokemon-species/655/"},{"name":"frogadier","url":"https://pokeapi.co/api/v2/pokemon-species/657/"},{"name":"greninja","url":"https://pokeapi.co/api/v2/pokemon-species/658/"},{"name":"talonflame","url":"https://pokeapi.co/api/v2/pokemon-species/663/"},{"name":"pyroar","url":"https://pokeapi.co/api/v2/pokemon-species/668/"},{"name":"rowlet","url":"https://pokeapi.co/api/v2/pokemon-species/722/"},{"name":"litten","url":"https://pokeapi.co/api/v2/pokemon-species/725/"},{"name":"popplio","url":"https://pokeapi.co/api/v2/pokemon-species/728/"},{"name":"bounsweet","url":"https://pokeapi.co/api/v2/pokemon-species/761/"},{"name":"minior","url":"https://pokeapi.co/api/v2/pokemon-species/774/"},{"name":"decidueye","url":"https://pokeapi.co/api/v2/pokemon-species/724/"},{"name":"torracat","url":"https://pokeapi.co/api/v2/pokemon-species/726/"},{"name":"incineroar","url":"https://pokeapi.co/api/v2/pokemon-species/727/"},{"name":"brionne","url":"https://pokeapi.co/api/v2/pokemon-species/729/"},{"name":"primarina","url":"https://pokeapi.co/api/v2/pokemon-species/730/"},{"name":"steenee","url":"https://pokeapi.co/api/v2/pokemon-species/762/"},{"name":"tsareena","url":"https://pokeapi.co/api/v2/pokemon-species/763/"},{"name":"ivysaur","url":"https://pokeapi.co/api/v2/pokemon-species/2/"},{"name":"croconaw","url":"https://pokeapi.co/api/v2/pokemon-species/159/"},{"name":"roselia","url":"https://pokeapi.co/api/v2/pokemon-species/315/"},{"name":"prinplup","url":"https://pokeapi.co/api/v2/pokemon-species/394/"},{"name":"roserade","url":"https://pokeapi.co/api/v2/pokemon-species/407/"},{"name":"samurott","url":"https://pokeapi.co/api/v2/pokemon-species/503/"},{"name":"whirlipede","url":"https://pokeapi.co/api/v2/pokemon-species/544/"},{"name":"gothorita","url":"https://pokeapi.co/api/v2/pokemon-species/575/"},{"name":"fletchinder","url":"https://pokeapi.co/api/v2/pokemon-species/662/"},{"name":"dartrix","url":"https://pokeapi.co/api/v2/pokemon-species/723/"}]}"#;

#[test]
fn growth_rate_from_id() {
    test_from_id::<GrowthRate>(4, MEDIUM_SLOW_JSON);
}

#[test]
fn growth_rate_from_name() {
    test_from_name::<GrowthRate>("medium-slow", MEDIUM_SLOW_JSON);
}

// https://pokeapi.co/api/v2/nature/16/
static JOLLY_JSON: &str = r#"{"decreased_stat":{"name":"special-attack","url":"https://pokeapi.co/api/v2/stat/4/"},"hates_flavor":{"name":"dry","url":"https://pokeapi.co/api/v2/berry-flavor/2/"},"id":16,"increased_stat":{"name":"speed","url":"https://pokeapi.co/api/v2/stat/6/"},"likes_flavor":{"name":"sweet","url":"https://pokeapi.co/api/v2/berry-flavor/3/"},"move_battle_style_preferences":[{"high_hp_preference":35,"low_hp_preference":35,"move_battle_style":{"name":"attack","url":"https://pokeapi.co/api/v2/move-battle-style/1/"}},{"high_hp_preference":5,"low_hp_preference":60,"move_battle_style":{"name":"defense","url":"https://pokeapi.co/api/v2/move-battle-style/2/"}},{"high_hp_preference":60,"low_hp_preference":5,"move_battle_style":{"name":"support","url":"https://pokeapi.co/api/v2/move-battle-style/3/"}}],"name":"jolly","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"ようき"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"명랑"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Jovial"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Froh"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Alegre"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Allegra"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Jolly"}],"pokeathlon_stat_changes":[{"max_change":2,"pokeathlon_stat":{"name":"speed","url":"https://pokeapi.co/api/v2/pokeathlon-stat/1/"}},{"max_change":-2,"pokeathlon_stat":{"name":"jump","url":"https://pokeapi.co/api/v2/pokeathlon-stat/5/"}}]}"#;

#[test]
fn nature_from_id() {
    test_from_id::<Nature>(16, JOLLY_JSON);
}

#[test]
fn nature_from_name() {
    test_from_name::<Nature>("jolly", JOLLY_JSON);
}

// https://pokeapi.co/api/v2/pokeathlon-stat/2/
static POWER_JSON: &str = r#"{"affecting_natures":{"decrease":[{"max_change":-2,"nature":{"name":"timid","url":"https://pokeapi.co/api/v2/nature/5/"}},{"max_change":-2,"nature":{"name":"rash","url":"https://pokeapi.co/api/v2/nature/15/"}},{"max_change":-2,"nature":{"name":"naughty","url":"https://pokeapi.co/api/v2/nature/17/"}},{"max_change":-2,"nature":{"name":"naive","url":"https://pokeapi.co/api/v2/nature/20/"}},{"max_change":-1,"nature":{"name":"serious","url":"https://pokeapi.co/api/v2/nature/25/"}}],"increase":[{"max_change":1,"nature":{"name":"hardy","url":"https://pokeapi.co/api/v2/nature/1/"}},{"max_change":2,"nature":{"name":"calm","url":"https://pokeapi.co/api/v2/nature/4/"}},{"max_change":2,"nature":{"name":"careful","url":"https://pokeapi.co/api/v2/nature/14/"}},{"max_change":2,"nature":{"name":"brave","url":"https://pokeapi.co/api/v2/nature/21/"}},{"max_change":2,"nature":{"name":"sassy","url":"https://pokeapi.co/api/v2/nature/24/"}}]},"id":2,"name":"power","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Force"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Power"}]}"#;

#[test]
fn pokeathlon_stat_from_id() {
    test_from_id::<PokeathlonStat>(2, POWER_JSON);
}

#[test]
fn pokeathlon_stat_from_name() {
    test_from_name::<PokeathlonStat>("power", POWER_JSON);
}

// https://pokeapi.co/api/v2/pokemon/25
static PIKACHU_JSON: &str = r#"{"abilities":[{"ability":{"name":"lightning-rod","url":"https://pokeapi.co/api/v2/ability/31/"},"is_hidden":true,"slot":3},{"ability":{"name":"static","url":"https://pokeapi.co/api/v2/ability/9/"},"is_hidden":false,"slot":1}],"base_experience":112,"forms":[{"name":"pikachu","url":"https://pokeapi.co/api/v2/pokemon-form/25/"}],"game_indices":[{"game_index":25,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"game_index":25,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"game_index":25,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"game_index":25,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"game_index":25,"version":{"name":"soulsilver","url":"https://pokeapi.co/api/v2/version/16/"}},{"game_index":25,"version":{"name":"heartgold","url":"https://pokeapi.co/api/v2/version/15/"}},{"game_index":25,"version":{"name":"platinum","url":"https://pokeapi.co/api/v2/version/14/"}},{"game_index":25,"version":{"name":"pearl","url":"https://pokeapi.co/api/v2/version/13/"}},{"game_index":25,"version":{"name":"diamond","url":"https://pokeapi.co/api/v2/version/12/"}},{"game_index":25,"version":{"name":"leafgreen","url":"https://pokeapi.co/api/v2/version/11/"}},{"game_index":25,"version":{"name":"firered","url":"https://pokeapi.co/api/v2/version/10/"}},{"game_index":25,"version":{"name":"emerald","url":"https://pokeapi.co/api/v2/version/9/"}},{"game_index":25,"version":{"name":"sapphire","url":"https://pokeapi.co/api/v2/version/8/"}},{"game_index":25,"version":{"name":"ruby","url":"https://pokeapi.co/api/v2/version/7/"}},{"game_index":25,"version":{"name":"crystal","url":"https://pokeapi.co/api/v2/version/6/"}},{"game_index":25,"version":{"name":"silver","url":"https://pokeapi.co/api/v2/version/5/"}},{"game_index":25,"version":{"name":"gold","url":"https://pokeapi.co/api/v2/version/4/"}},{"game_index":84,"version":{"name":"yellow","url":"https://pokeapi.co/api/v2/version/3/"}},{"game_index":84,"version":{"name":"blue","url":"https://pokeapi.co/api/v2/version/2/"}},{"game_index":84,"version":{"name":"red","url":"https://pokeapi.co/api/v2/version/1/"}}],"height":4,"held_items":[{"item":{"name":"oran-berry","url":"https://pokeapi.co/api/v2/item/132/"},"version_details":[{"rarity":50,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"rarity":50,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"rarity":50,"version":{"name":"soulsilver","url":"https://pokeapi.co/api/v2/version/16/"}},{"rarity":50,"version":{"name":"heartgold","url":"https://pokeapi.co/api/v2/version/15/"}},{"rarity":50,"version":{"name":"platinum","url":"https://pokeapi.co/api/v2/version/14/"}},{"rarity":50,"version":{"name":"pearl","url":"https://pokeapi.co/api/v2/version/13/"}},{"rarity":50,"version":{"name":"diamond","url":"https://pokeapi.co/api/v2/version/12/"}},{"rarity":50,"version":{"name":"emerald","url":"https://pokeapi.co/api/v2/version/9/"}},{"rarity":50,"version":{"name":"sapphire","url":"https://pokeapi.co/api/v2/version/8/"}},{"rarity":50,"version":{"name":"ruby","url":"https://pokeapi.co/api/v2/version/7/"}}]},{"item":{"name":"light-ball","url":"https://pokeapi.co/api/v2/item/213/"},"version_details":[{"rarity":5,"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"rarity":5,"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"rarity":5,"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"rarity":5,"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"rarity":5,"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"rarity":5,"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"rarity":1,"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"rarity":1,"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"rarity":5,"version":{"name":"soulsilver","url":"https://pokeapi.co/api/v2/version/16/"}},{"rarity":5,"version":{"name":"heartgold","url":"https://pokeapi.co/api/v2/version/15/"}},{"rarity":5,"version":{"name":"platinum","url":"https://pokeapi.co/api/v2/version/14/"}},{"rarity":5,"version":{"name":"pearl","url":"https://pokeapi.co/api/v2/version/13/"}},{"rarity":5,"version":{"name":"diamond","url":"https://pokeapi.co/api/v2/version/12/"}},{"rarity":5,"version":{"name":"emerald","url":"https://pokeapi.co/api/v2/version/9/"}},{"rarity":5,"version":{"name":"sapphire","url":"https://pokeapi.co/api/v2/version/8/"}},{"rarity":5,"version":{"name":"ruby","url":"https://pokeapi.co/api/v2/version/7/"}}]}],"id":25,"is_default":true,"location_area_encounters":"https://pokeapi.co/api/v2/pokemon/25/encounters","moves":[{"move":{"name":"mega-punch","url":"https://pokeapi.co/api/v2/move/5/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"pay-day","url":"https://pokeapi.co/api/v2/move/6/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"thunder-punch","url":"https://pokeapi.co/api/v2/move/9/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"slam","url":"https://pokeapi.co/api/v2/move/21/"},"version_group_details":[{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":20,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}}]},{"move":{"name":"mega-kick","url":"https://pokeapi.co/api/v2/move/25/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"headbutt","url":"https://pokeapi.co/api/v2/move/29/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}}]},{"move":{"name":"body-slam","url":"https://pokeapi.co/api/v2/move/34/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"take-down","url":"https://pokeapi.co/api/v2/move/36/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"double-edge","url":"https://pokeapi.co/api/v2/move/38/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"tail-whip","url":"https://pokeapi.co/api/v2/move/39/"},"version_group_details":[{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":6,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}}]},{"move":{"name":"growl","url":"https://pokeapi.co/api/v2/move/45/"},"version_group_details":[{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":5,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"surf","url":"https://pokeapi.co/api/v2/move/57/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"stadium-surfing-pikachu","url":"https://pokeapi.co/api/v2/move-learn-method/5/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"stadium-surfing-pikachu","url":"https://pokeapi.co/api/v2/move-learn-method/5/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"submission","url":"https://pokeapi.co/api/v2/move/66/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"counter","url":"https://pokeapi.co/api/v2/move/68/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}}]},{"move":{"name":"seismic-toss","url":"https://pokeapi.co/api/v2/move/69/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"strength","url":"https://pokeapi.co/api/v2/move/70/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"thunder-shock","url":"https://pokeapi.co/api/v2/move/84/"},"version_group_details":[{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":1,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"thunderbolt","url":"https://pokeapi.co/api/v2/move/85/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"thunder-wave","url":"https://pokeapi.co/api/v2/move/86/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":8,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}},{"level_learned_at":9,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"thunder","url":"https://pokeapi.co/api/v2/move/87/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":58,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":58,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":58,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":41,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}},{"level_learned_at":43,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"dig","url":"https://pokeapi.co/api/v2/move/91/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"toxic","url":"https://pokeapi.co/api/v2/move/92/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"agility","url":"https://pokeapi.co/api/v2/move/97/"},"version_group_details":[{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":33,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"quick-attack","url":"https://pokeapi.co/api/v2/move/98/"},"version_group_details":[{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":10,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":11,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":16,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"rage","url":"https://pokeapi.co/api/v2/move/99/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"mimic","url":"https://pokeapi.co/api/v2/move/102/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"double-team","url":"https://pokeapi.co/api/v2/move/104/"},"version_group_details":[{"level_learned_at":23,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":23,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":23,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":15,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"defense-curl","url":"https://pokeapi.co/api/v2/move/111/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}}]},{"move":{"name":"light-screen","url":"https://pokeapi.co/api/v2/move/113/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":53,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":53,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":53,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":45,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"reflect","url":"https://pokeapi.co/api/v2/move/115/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"bide","url":"https://pokeapi.co/api/v2/move/117/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"swift","url":"https://pokeapi.co/api/v2/move/129/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"skull-bash","url":"https://pokeapi.co/api/v2/move/130/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"flash","url":"https://pokeapi.co/api/v2/move/148/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"rest","url":"https://pokeapi.co/api/v2/move/156/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"substitute","url":"https://pokeapi.co/api/v2/move/164/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"yellow","url":"https://pokeapi.co/api/v2/version-group/2/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"red-blue","url":"https://pokeapi.co/api/v2/version-group/1/"}}]},{"move":{"name":"snore","url":"https://pokeapi.co/api/v2/move/173/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"curse","url":"https://pokeapi.co/api/v2/move/174/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}}]},{"move":{"name":"protect","url":"https://pokeapi.co/api/v2/move/182/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"mud-slap","url":"https://pokeapi.co/api/v2/move/189/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"zap-cannon","url":"https://pokeapi.co/api/v2/move/192/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}}]},{"move":{"name":"detect","url":"https://pokeapi.co/api/v2/move/197/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}}]},{"move":{"name":"endure","url":"https://pokeapi.co/api/v2/move/203/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"rollout","url":"https://pokeapi.co/api/v2/move/205/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"swagger","url":"https://pokeapi.co/api/v2/move/207/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"spark","url":"https://pokeapi.co/api/v2/move/209/"},"version_group_details":[{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":26,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}}]},{"move":{"name":"attract","url":"https://pokeapi.co/api/v2/move/213/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"sleep-talk","url":"https://pokeapi.co/api/v2/move/214/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"return","url":"https://pokeapi.co/api/v2/move/216/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"frustration","url":"https://pokeapi.co/api/v2/move/218/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"dynamic-punch","url":"https://pokeapi.co/api/v2/move/223/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}}]},{"move":{"name":"iron-tail","url":"https://pokeapi.co/api/v2/move/231/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"hidden-power","url":"https://pokeapi.co/api/v2/move/237/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"rain-dance","url":"https://pokeapi.co/api/v2/move/240/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"crystal","url":"https://pokeapi.co/api/v2/version-group/4/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"gold-silver","url":"https://pokeapi.co/api/v2/version-group/3/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"rock-smash","url":"https://pokeapi.co/api/v2/move/249/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"facade","url":"https://pokeapi.co/api/v2/move/263/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"focus-punch","url":"https://pokeapi.co/api/v2/move/264/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"helping-hand","url":"https://pokeapi.co/api/v2/move/270/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"brick-break","url":"https://pokeapi.co/api/v2/move/280/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}}]},{"move":{"name":"knock-off","url":"https://pokeapi.co/api/v2/move/282/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"secret-power","url":"https://pokeapi.co/api/v2/move/290/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}}]},{"move":{"name":"signal-beam","url":"https://pokeapi.co/api/v2/move/324/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"covet","url":"https://pokeapi.co/api/v2/move/343/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}}]},{"move":{"name":"shock-wave","url":"https://pokeapi.co/api/v2/move/351/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"xd","url":"https://pokeapi.co/api/v2/version-group/13/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"emerald","url":"https://pokeapi.co/api/v2/version-group/6/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ruby-sapphire","url":"https://pokeapi.co/api/v2/version-group/5/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"colosseum","url":"https://pokeapi.co/api/v2/version-group/12/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"firered-leafgreen","url":"https://pokeapi.co/api/v2/version-group/7/"}}]},{"move":{"name":"natural-gift","url":"https://pokeapi.co/api/v2/move/363/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"feint","url":"https://pokeapi.co/api/v2/move/364/"},"version_group_details":[{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":21,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"fling","url":"https://pokeapi.co/api/v2/move/374/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"magnet-rise","url":"https://pokeapi.co/api/v2/move/393/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}}]},{"move":{"name":"discharge","url":"https://pokeapi.co/api/v2/move/435/"},"version_group_details":[{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":34,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":42,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":37,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"captivate","url":"https://pokeapi.co/api/v2/move/445/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"grass-knot","url":"https://pokeapi.co/api/v2/move/447/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"charge-beam","url":"https://pokeapi.co/api/v2/move/451/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"heartgold-soulsilver","url":"https://pokeapi.co/api/v2/version-group/10/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"platinum","url":"https://pokeapi.co/api/v2/version-group/9/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"diamond-pearl","url":"https://pokeapi.co/api/v2/version-group/8/"}}]},{"move":{"name":"electro-ball","url":"https://pokeapi.co/api/v2/move/486/"},"version_group_details":[{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":13,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":18,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}]},{"move":{"name":"round","url":"https://pokeapi.co/api/v2/move/496/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}]},{"move":{"name":"echoed-voice","url":"https://pokeapi.co/api/v2/move/497/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}]},{"move":{"name":"volt-switch","url":"https://pokeapi.co/api/v2/move/521/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}]},{"move":{"name":"electroweb","url":"https://pokeapi.co/api/v2/move/527/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"tutor","url":"https://pokeapi.co/api/v2/move-learn-method/3/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}}]},{"move":{"name":"wild-charge","url":"https://pokeapi.co/api/v2/move/528/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-2-white-2","url":"https://pokeapi.co/api/v2/version-group/14/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":50,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"black-white","url":"https://pokeapi.co/api/v2/version-group/11/"}}]},{"move":{"name":"play-nice","url":"https://pokeapi.co/api/v2/move/589/"},"version_group_details":[{"level_learned_at":7,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":7,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":7,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}},{"level_learned_at":7,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}}]},{"move":{"name":"confide","url":"https://pokeapi.co/api/v2/move/590/"},"version_group_details":[{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":0,"move_learn_method":{"name":"machine","url":"https://pokeapi.co/api/v2/move-learn-method/4/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}}]},{"move":{"name":"nuzzle","url":"https://pokeapi.co/api/v2/move/609/"},"version_group_details":[{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"ultra-sun-ultra-moon","url":"https://pokeapi.co/api/v2/version-group/18/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"sun-moon","url":"https://pokeapi.co/api/v2/version-group/17/"}},{"level_learned_at":29,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}},{"level_learned_at":23,"move_learn_method":{"name":"level-up","url":"https://pokeapi.co/api/v2/move-learn-method/1/"},"version_group":{"name":"x-y","url":"https://pokeapi.co/api/v2/version-group/15/"}}]}],"name":"pikachu","order":35,"species":{"name":"pikachu","url":"https://pokeapi.co/api/v2/pokemon-species/25/"},"sprites":{"back_default":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/back/25.png","back_female":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/back/female/25.png","back_shiny":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/back/shiny/25.png","back_shiny_female":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/back/shiny/female/25.png","front_default":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/25.png","front_female":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/female/25.png","front_shiny":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/shiny/25.png","front_shiny_female":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/shiny/female/25.png"},"stats":[{"base_stat":90,"effort":2,"stat":{"name":"speed","url":"https://pokeapi.co/api/v2/stat/6/"}},{"base_stat":50,"effort":0,"stat":{"name":"special-defense","url":"https://pokeapi.co/api/v2/stat/5/"}},{"base_stat":50,"effort":0,"stat":{"name":"special-attack","url":"https://pokeapi.co/api/v2/stat/4/"}},{"base_stat":40,"effort":0,"stat":{"name":"defense","url":"https://pokeapi.co/api/v2/stat/3/"}},{"base_stat":55,"effort":0,"stat":{"name":"attack","url":"https://pokeapi.co/api/v2/stat/2/"}},{"base_stat":35,"effort":0,"stat":{"name":"hp","url":"https://pokeapi.co/api/v2/stat/1/"}}],"types":[{"slot":1,"type":{"name":"electric","url":"https://pokeapi.co/api/v2/type/13/"}}],"weight":60}"#;
static PIKACHU_ENCOUNTERS_JSON: &str = r#"[{"location_area":{"name":"trophy-garden-area","url":"https://pokeapi.co/api/v2/location-area/118/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":18,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":18},{"chance":4,"condition_values":[],"max_level":18,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":18},{"chance":1,"condition_values":[],"max_level":18,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":18}],"max_chance":10,"version":{"name":"diamond","url":"https://pokeapi.co/api/v2/version/12/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":18,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":18},{"chance":4,"condition_values":[],"max_level":18,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":18},{"chance":1,"condition_values":[],"max_level":18,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":18}],"max_chance":10,"version":{"name":"pearl","url":"https://pokeapi.co/api/v2/version/13/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":22,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":22},{"chance":4,"condition_values":[],"max_level":24,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":24},{"chance":1,"condition_values":[],"max_level":24,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":24}],"max_chance":10,"version":{"name":"platinum","url":"https://pokeapi.co/api/v2/version/14/"}}]},{"location_area":{"name":"viridian-forest-area","url":"https://pokeapi.co/api/v2/location-area/321/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":3,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":3},{"chance":1,"condition_values":[],"max_level":5,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":5}],"max_chance":5,"version":{"name":"red","url":"https://pokeapi.co/api/v2/version/1/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":3,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":3},{"chance":1,"condition_values":[],"max_level":5,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":5}],"max_chance":5,"version":{"name":"blue","url":"https://pokeapi.co/api/v2/version/2/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":3,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":3},{"chance":1,"condition_values":[],"max_level":5,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":5}],"max_chance":5,"version":{"name":"firered","url":"https://pokeapi.co/api/v2/version/10/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":3,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":3},{"chance":1,"condition_values":[],"max_level":5,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":5}],"max_chance":5,"version":{"name":"leafgreen","url":"https://pokeapi.co/api/v2/version/11/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4},{"chance":1,"condition_values":[{"name":"time-morning","url":"https://pokeapi.co/api/v2/encounter-condition-value/3/"}],"max_level":7,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":7},{"chance":1,"condition_values":[{"name":"time-day","url":"https://pokeapi.co/api/v2/encounter-condition-value/4/"}],"max_level":7,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":7},{"chance":1,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4}],"max_chance":7,"version":{"name":"heartgold","url":"https://pokeapi.co/api/v2/version/15/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4},{"chance":1,"condition_values":[{"name":"time-morning","url":"https://pokeapi.co/api/v2/encounter-condition-value/3/"}],"max_level":7,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":7},{"chance":1,"condition_values":[{"name":"time-day","url":"https://pokeapi.co/api/v2/encounter-condition-value/4/"}],"max_level":7,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":7},{"chance":1,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4}],"max_chance":7,"version":{"name":"soulsilver","url":"https://pokeapi.co/api/v2/version/16/"}}]},{"location_area":{"name":"power-plant-area","url":"https://pokeapi.co/api/v2/location-area/330/"},"version_details":[{"encounter_details":[{"chance":15,"condition_values":[],"max_level":20,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":20},{"chance":10,"condition_values":[],"max_level":24,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":24}],"max_chance":25,"version":{"name":"red","url":"https://pokeapi.co/api/v2/version/1/"}},{"encounter_details":[{"chance":15,"condition_values":[],"max_level":20,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":20},{"chance":10,"condition_values":[],"max_level":24,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":24}],"max_chance":25,"version":{"name":"blue","url":"https://pokeapi.co/api/v2/version/2/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":22,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":22},{"chance":10,"condition_values":[],"max_level":24,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":24},{"chance":4,"condition_values":[],"max_level":26,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":26},{"chance":1,"condition_values":[],"max_level":26,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":26}],"max_chance":25,"version":{"name":"firered","url":"https://pokeapi.co/api/v2/version/10/"}},{"encounter_details":[{"chance":10,"condition_values":[],"max_level":22,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":22},{"chance":10,"condition_values":[],"max_level":24,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":24},{"chance":4,"condition_values":[],"max_level":26,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":26},{"chance":1,"condition_values":[],"max_level":26,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":26}],"max_chance":25,"version":{"name":"leafgreen","url":"https://pokeapi.co/api/v2/version/11/"}}]},{"location_area":{"name":"hoenn-safari-zone-sw","url":"https://pokeapi.co/api/v2/location-area/431/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":25,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":27,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":27}],"max_chance":5,"version":{"name":"ruby","url":"https://pokeapi.co/api/v2/version/7/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":25,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":27,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":27}],"max_chance":5,"version":{"name":"sapphire","url":"https://pokeapi.co/api/v2/version/8/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":25,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":27,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":27}],"max_chance":5,"version":{"name":"emerald","url":"https://pokeapi.co/api/v2/version/9/"}}]},{"location_area":{"name":"hoenn-safari-zone-se","url":"https://pokeapi.co/api/v2/location-area/432/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":25,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":27,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":27}],"max_chance":5,"version":{"name":"ruby","url":"https://pokeapi.co/api/v2/version/7/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":25,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":27,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":27}],"max_chance":5,"version":{"name":"sapphire","url":"https://pokeapi.co/api/v2/version/8/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":25,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":25},{"chance":1,"condition_values":[],"max_level":27,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":27}],"max_chance":5,"version":{"name":"emerald","url":"https://pokeapi.co/api/v2/version/9/"}}]},{"location_area":{"name":"kalos-route-3-area","url":"https://pokeapi.co/api/v2/location-area/714/"},"version_details":[{"encounter_details":[{"chance":4,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4},{"chance":1,"condition_values":[],"max_level":5,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":5}],"max_chance":5,"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"encounter_details":[{"chance":4,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4},{"chance":1,"condition_values":[],"max_level":5,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":5}],"max_chance":5,"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}}]},{"location_area":{"name":"santalune-forest-area","url":"https://pokeapi.co/api/v2/location-area/734/"},"version_details":[{"encounter_details":[{"chance":5,"condition_values":[],"max_level":3,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":3},{"chance":1,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4}],"max_chance":6,"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"encounter_details":[{"chance":5,"condition_values":[],"max_level":3,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":3},{"chance":1,"condition_values":[],"max_level":4,"method":{"name":"walk","url":"https://pokeapi.co/api/v2/encounter-method/1/"},"min_level":4}],"max_chance":6,"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}}]}]"#;

#[test]
fn pokemon_from_id() {
    test_from_id::<Pokemon>(25, PIKACHU_JSON);
}

#[test]
fn pokemon_from_name() {
    test_from_name::<Pokemon>("pikachu", PIKACHU_JSON);
}

#[test]
fn pokemon_encounters() {
    let name = "pikachu";
    let id = 25;
    let _m1 = common::make_mock(&format!("/api/v2/pokemon/{}/", name), PIKACHU_JSON);
    let _m2 = common::make_mock(
        &format!("/api/v2/pokemon/{}/encounters", id),
        PIKACHU_ENCOUNTERS_JSON,
    );

    let pika = Pokemon::from_name(&name).unwrap();
    pika.get_encounters().unwrap();
}

// https://pokeapi.co/api/v2/pokemon-color/6/
static PINK_JSON: &str = r#"{"id":6,"name":"pink","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"ピンク"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Rose"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Rosa"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Rosa"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Pink"}],"pokemon_species":[{"name":"clefairy","url":"https://pokeapi.co/api/v2/pokemon-species/35/"},{"name":"clefable","url":"https://pokeapi.co/api/v2/pokemon-species/36/"},{"name":"jigglypuff","url":"https://pokeapi.co/api/v2/pokemon-species/39/"},{"name":"wigglytuff","url":"https://pokeapi.co/api/v2/pokemon-species/40/"},{"name":"slowpoke","url":"https://pokeapi.co/api/v2/pokemon-species/79/"},{"name":"exeggcute","url":"https://pokeapi.co/api/v2/pokemon-species/102/"},{"name":"lickitung","url":"https://pokeapi.co/api/v2/pokemon-species/108/"},{"name":"porygon","url":"https://pokeapi.co/api/v2/pokemon-species/137/"},{"name":"mew","url":"https://pokeapi.co/api/v2/pokemon-species/151/"},{"name":"chansey","url":"https://pokeapi.co/api/v2/pokemon-species/113/"},{"name":"mr-mime","url":"https://pokeapi.co/api/v2/pokemon-species/122/"},{"name":"cleffa","url":"https://pokeapi.co/api/v2/pokemon-species/173/"},{"name":"igglybuff","url":"https://pokeapi.co/api/v2/pokemon-species/174/"},{"name":"hoppip","url":"https://pokeapi.co/api/v2/pokemon-species/187/"},{"name":"snubbull","url":"https://pokeapi.co/api/v2/pokemon-species/209/"},{"name":"corsola","url":"https://pokeapi.co/api/v2/pokemon-species/222/"},{"name":"flaaffy","url":"https://pokeapi.co/api/v2/pokemon-species/180/"},{"name":"slowking","url":"https://pokeapi.co/api/v2/pokemon-species/199/"},{"name":"smoochum","url":"https://pokeapi.co/api/v2/pokemon-species/238/"},{"name":"miltank","url":"https://pokeapi.co/api/v2/pokemon-species/241/"},{"name":"whismur","url":"https://pokeapi.co/api/v2/pokemon-species/293/"},{"name":"skitty","url":"https://pokeapi.co/api/v2/pokemon-species/300/"},{"name":"blissey","url":"https://pokeapi.co/api/v2/pokemon-species/242/"},{"name":"luvdisc","url":"https://pokeapi.co/api/v2/pokemon-species/370/"},{"name":"milotic","url":"https://pokeapi.co/api/v2/pokemon-species/350/"},{"name":"gorebyss","url":"https://pokeapi.co/api/v2/pokemon-species/368/"},{"name":"cherubi","url":"https://pokeapi.co/api/v2/pokemon-species/420/"},{"name":"mime-jr","url":"https://pokeapi.co/api/v2/pokemon-species/439/"},{"name":"happiny","url":"https://pokeapi.co/api/v2/pokemon-species/440/"},{"name":"lickilicky","url":"https://pokeapi.co/api/v2/pokemon-species/463/"},{"name":"mesprit","url":"https://pokeapi.co/api/v2/pokemon-species/481/"},{"name":"munna","url":"https://pokeapi.co/api/v2/pokemon-species/517/"},{"name":"audino","url":"https://pokeapi.co/api/v2/pokemon-species/531/"},{"name":"musharna","url":"https://pokeapi.co/api/v2/pokemon-species/518/"},{"name":"deerling","url":"https://pokeapi.co/api/v2/pokemon-species/585/"},{"name":"alomomola","url":"https://pokeapi.co/api/v2/pokemon-species/594/"},{"name":"spritzee","url":"https://pokeapi.co/api/v2/pokemon-species/682/"},{"name":"aromatisse","url":"https://pokeapi.co/api/v2/pokemon-species/683/"},{"name":"sylveon","url":"https://pokeapi.co/api/v2/pokemon-species/700/"},{"name":"diancie","url":"https://pokeapi.co/api/v2/pokemon-species/719/"},{"name":"fomantis","url":"https://pokeapi.co/api/v2/pokemon-species/753/"},{"name":"stufful","url":"https://pokeapi.co/api/v2/pokemon-species/759/"},{"name":"bruxish","url":"https://pokeapi.co/api/v2/pokemon-species/779/"},{"name":"tapu-lele","url":"https://pokeapi.co/api/v2/pokemon-species/786/"},{"name":"bewear","url":"https://pokeapi.co/api/v2/pokemon-species/760/"},{"name":"slowbro","url":"https://pokeapi.co/api/v2/pokemon-species/80/"},{"name":"lurantis","url":"https://pokeapi.co/api/v2/pokemon-species/754/"}]}"#;

#[test]
fn pokemon_color_from_id() {
    test_from_id::<PokemonColor>(6, PINK_JSON);
}

#[test]
fn pokemon_color_from_name() {
    test_from_name::<PokemonColor>("pink", PINK_JSON);
}

// https://pokeapi.co/api/v2/pokemon-form/10168/
static SABLEYE_MEGA_JSON: &str = r#"{"form_name":"mega","form_names":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"name":"超级勾魂眼"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"メガヤミラミ"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Mega Sableye"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"MegaSableye"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Mega-Sableye"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Mega-Zobiris"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Méga-Ténéfix"},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"name":"超級勾魂眼"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"메가깜까미"},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"メガヤミラミ"}],"form_order":2,"id":10168,"is_battle_only":true,"is_default":true,"is_mega":true,"name":"sableye-mega","names":[{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Mega Sableye"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Méga-Ténéfix"}],"order":410,"pokemon":{"name":"sableye-mega","url":"https://pokeapi.co/api/v2/pokemon/10066/"},"sprites":{"back_default":null,"back_shiny":null,"front_default":"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/10066.png","front_shiny":null},"version_group":{"name":"omega-ruby-alpha-sapphire","url":"https://pokeapi.co/api/v2/version-group/16/"}}"#;

#[test]
fn pokemon_form_from_id() {
    test_from_id::<PokemonForm>(10168, SABLEYE_MEGA_JSON);
}

#[test]
fn pokemon_form_from_name() {
    test_from_name::<PokemonForm>("sableye-mega", SABLEYE_MEGA_JSON);
}

// https://pokeapi.co/api/v2/pokemon-habitat/5/
static RARE_JSON: &str = r#"{"id":5,"name":"rare","names":[{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"rares"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"rare"}],"pokemon_species":[{"name":"articuno","url":"https://pokeapi.co/api/v2/pokemon-species/144/"},{"name":"zapdos","url":"https://pokeapi.co/api/v2/pokemon-species/145/"},{"name":"moltres","url":"https://pokeapi.co/api/v2/pokemon-species/146/"},{"name":"mewtwo","url":"https://pokeapi.co/api/v2/pokemon-species/150/"},{"name":"mew","url":"https://pokeapi.co/api/v2/pokemon-species/151/"},{"name":"unown","url":"https://pokeapi.co/api/v2/pokemon-species/201/"},{"name":"lugia","url":"https://pokeapi.co/api/v2/pokemon-species/249/"},{"name":"ho-oh","url":"https://pokeapi.co/api/v2/pokemon-species/250/"},{"name":"rayquaza","url":"https://pokeapi.co/api/v2/pokemon-species/384/"},{"name":"deoxys","url":"https://pokeapi.co/api/v2/pokemon-species/386/"}]}"#;

#[test]
fn pokemon_habitat_from_id() {
    test_from_id::<PokemonHabitat>(9, RARE_JSON);
}

#[test]
fn pokemon_habitat_from_name() {
    test_from_name::<PokemonHabitat>("rare", RARE_JSON);
}

// https://pokeapi.co/api/v2/pokemon-shape/6/
static UPRIGHT_JSON: &str = r#"{"awesome_names":[{"awesome_name":"Sciurine","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"awesome_name":"Sciurine","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"}}],"id":6,"name":"upright","names":[{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Upright"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Droit"}],"pokemon_species":[{"name":"charmander","url":"https://pokeapi.co/api/v2/pokemon-species/4/"},{"name":"squirtle","url":"https://pokeapi.co/api/v2/pokemon-species/7/"},{"name":"sandshrew","url":"https://pokeapi.co/api/v2/pokemon-species/27/"},{"name":"psyduck","url":"https://pokeapi.co/api/v2/pokemon-species/54/"},{"name":"mankey","url":"https://pokeapi.co/api/v2/pokemon-species/56/"},{"name":"abra","url":"https://pokeapi.co/api/v2/pokemon-species/63/"},{"name":"machop","url":"https://pokeapi.co/api/v2/pokemon-species/66/"},{"name":"charmeleon","url":"https://pokeapi.co/api/v2/pokemon-species/5/"},{"name":"charizard","url":"https://pokeapi.co/api/v2/pokemon-species/6/"},{"name":"wartortle","url":"https://pokeapi.co/api/v2/pokemon-species/8/"},{"name":"blastoise","url":"https://pokeapi.co/api/v2/pokemon-species/9/"},{"name":"raichu","url":"https://pokeapi.co/api/v2/pokemon-species/26/"},{"name":"sandslash","url":"https://pokeapi.co/api/v2/pokemon-species/28/"},{"name":"nidoqueen","url":"https://pokeapi.co/api/v2/pokemon-species/31/"},{"name":"nidoking","url":"https://pokeapi.co/api/v2/pokemon-species/34/"},{"name":"clefairy","url":"https://pokeapi.co/api/v2/pokemon-species/35/"},{"name":"clefable","url":"https://pokeapi.co/api/v2/pokemon-species/36/"},{"name":"golduck","url":"https://pokeapi.co/api/v2/pokemon-species/55/"},{"name":"primeape","url":"https://pokeapi.co/api/v2/pokemon-species/57/"},{"name":"kadabra","url":"https://pokeapi.co/api/v2/pokemon-species/64/"},{"name":"cubone","url":"https://pokeapi.co/api/v2/pokemon-species/104/"},{"name":"lickitung","url":"https://pokeapi.co/api/v2/pokemon-species/108/"},{"name":"kangaskhan","url":"https://pokeapi.co/api/v2/pokemon-species/115/"},{"name":"mewtwo","url":"https://pokeapi.co/api/v2/pokemon-species/150/"},{"name":"mew","url":"https://pokeapi.co/api/v2/pokemon-species/151/"},{"name":"gengar","url":"https://pokeapi.co/api/v2/pokemon-species/94/"},{"name":"marowak","url":"https://pokeapi.co/api/v2/pokemon-species/105/"},{"name":"rhydon","url":"https://pokeapi.co/api/v2/pokemon-species/112/"},{"name":"chansey","url":"https://pokeapi.co/api/v2/pokemon-species/113/"},{"name":"electabuzz","url":"https://pokeapi.co/api/v2/pokemon-species/125/"},{"name":"magmar","url":"https://pokeapi.co/api/v2/pokemon-species/126/"},{"name":"kabutops","url":"https://pokeapi.co/api/v2/pokemon-species/141/"},{"name":"dragonite","url":"https://pokeapi.co/api/v2/pokemon-species/149/"},{"name":"totodile","url":"https://pokeapi.co/api/v2/pokemon-species/158/"},{"name":"cleffa","url":"https://pokeapi.co/api/v2/pokemon-species/173/"},{"name":"hoppip","url":"https://pokeapi.co/api/v2/pokemon-species/187/"},{"name":"aipom","url":"https://pokeapi.co/api/v2/pokemon-species/190/"},{"name":"sneasel","url":"https://pokeapi.co/api/v2/pokemon-species/215/"},{"name":"teddiursa","url":"https://pokeapi.co/api/v2/pokemon-species/216/"},{"name":"smeargle","url":"https://pokeapi.co/api/v2/pokemon-species/235/"},{"name":"feraligatr","url":"https://pokeapi.co/api/v2/pokemon-species/160/"},{"name":"flaaffy","url":"https://pokeapi.co/api/v2/pokemon-species/180/"},{"name":"ampharos","url":"https://pokeapi.co/api/v2/pokemon-species/181/"},{"name":"marill","url":"https://pokeapi.co/api/v2/pokemon-species/183/"},{"name":"azumarill","url":"https://pokeapi.co/api/v2/pokemon-species/184/"},{"name":"skiploom","url":"https://pokeapi.co/api/v2/pokemon-species/188/"},{"name":"jumpluff","url":"https://pokeapi.co/api/v2/pokemon-species/189/"},{"name":"quagsire","url":"https://pokeapi.co/api/v2/pokemon-species/195/"},{"name":"slowking","url":"https://pokeapi.co/api/v2/pokemon-species/199/"},{"name":"granbull","url":"https://pokeapi.co/api/v2/pokemon-species/210/"},{"name":"ursaring","url":"https://pokeapi.co/api/v2/pokemon-species/217/"},{"name":"magby","url":"https://pokeapi.co/api/v2/pokemon-species/240/"},{"name":"miltank","url":"https://pokeapi.co/api/v2/pokemon-species/241/"},{"name":"larvitar","url":"https://pokeapi.co/api/v2/pokemon-species/246/"},{"name":"treecko","url":"https://pokeapi.co/api/v2/pokemon-species/252/"},{"name":"whismur","url":"https://pokeapi.co/api/v2/pokemon-species/293/"},{"name":"plusle","url":"https://pokeapi.co/api/v2/pokemon-species/311/"},{"name":"minun","url":"https://pokeapi.co/api/v2/pokemon-species/312/"},{"name":"volbeat","url":"https://pokeapi.co/api/v2/pokemon-species/313/"},{"name":"tyranitar","url":"https://pokeapi.co/api/v2/pokemon-species/248/"},{"name":"grovyle","url":"https://pokeapi.co/api/v2/pokemon-species/253/"},{"name":"sceptile","url":"https://pokeapi.co/api/v2/pokemon-species/254/"},{"name":"combusken","url":"https://pokeapi.co/api/v2/pokemon-species/256/"},{"name":"blaziken","url":"https://pokeapi.co/api/v2/pokemon-species/257/"},{"name":"marshtomp","url":"https://pokeapi.co/api/v2/pokemon-species/259/"},{"name":"swampert","url":"https://pokeapi.co/api/v2/pokemon-species/260/"},{"name":"breloom","url":"https://pokeapi.co/api/v2/pokemon-species/286/"},{"name":"vigoroth","url":"https://pokeapi.co/api/v2/pokemon-species/288/"},{"name":"loudred","url":"https://pokeapi.co/api/v2/pokemon-species/294/"},{"name":"exploud","url":"https://pokeapi.co/api/v2/pokemon-species/295/"},{"name":"aggron","url":"https://pokeapi.co/api/v2/pokemon-species/306/"},{"name":"spinda","url":"https://pokeapi.co/api/v2/pokemon-species/327/"},{"name":"zangoose","url":"https://pokeapi.co/api/v2/pokemon-species/335/"},{"name":"kecleon","url":"https://pokeapi.co/api/v2/pokemon-species/352/"},{"name":"wynaut","url":"https://pokeapi.co/api/v2/pokemon-species/360/"},{"name":"groudon","url":"https://pokeapi.co/api/v2/pokemon-species/383/"},{"name":"chimchar","url":"https://pokeapi.co/api/v2/pokemon-species/390/"},{"name":"grumpig","url":"https://pokeapi.co/api/v2/pokemon-species/326/"},{"name":"armaldo","url":"https://pokeapi.co/api/v2/pokemon-species/348/"},{"name":"banette","url":"https://pokeapi.co/api/v2/pokemon-species/354/"},{"name":"monferno","url":"https://pokeapi.co/api/v2/pokemon-species/391/"},{"name":"infernape","url":"https://pokeapi.co/api/v2/pokemon-species/392/"},{"name":"cranidos","url":"https://pokeapi.co/api/v2/pokemon-species/408/"},{"name":"buneary","url":"https://pokeapi.co/api/v2/pokemon-species/427/"},{"name":"gible","url":"https://pokeapi.co/api/v2/pokemon-species/443/"},{"name":"riolu","url":"https://pokeapi.co/api/v2/pokemon-species/447/"},{"name":"snover","url":"https://pokeapi.co/api/v2/pokemon-species/459/"},{"name":"empoleon","url":"https://pokeapi.co/api/v2/pokemon-species/395/"},{"name":"bibarel","url":"https://pokeapi.co/api/v2/pokemon-species/400/"},{"name":"rampardos","url":"https://pokeapi.co/api/v2/pokemon-species/409/"},{"name":"ambipom","url":"https://pokeapi.co/api/v2/pokemon-species/424/"},{"name":"lopunny","url":"https://pokeapi.co/api/v2/pokemon-species/428/"},{"name":"gabite","url":"https://pokeapi.co/api/v2/pokemon-species/444/"},{"name":"garchomp","url":"https://pokeapi.co/api/v2/pokemon-species/445/"},{"name":"lucario","url":"https://pokeapi.co/api/v2/pokemon-species/448/"},{"name":"weavile","url":"https://pokeapi.co/api/v2/pokemon-species/461/"},{"name":"rhyperior","url":"https://pokeapi.co/api/v2/pokemon-species/464/"},{"name":"electivire","url":"https://pokeapi.co/api/v2/pokemon-species/466/"},{"name":"magmortar","url":"https://pokeapi.co/api/v2/pokemon-species/467/"},{"name":"uxie","url":"https://pokeapi.co/api/v2/pokemon-species/480/"},{"name":"mesprit","url":"https://pokeapi.co/api/v2/pokemon-species/481/"},{"name":"azelf","url":"https://pokeapi.co/api/v2/pokemon-species/482/"},{"name":"palkia","url":"https://pokeapi.co/api/v2/pokemon-species/484/"},{"name":"snivy","url":"https://pokeapi.co/api/v2/pokemon-species/495/"},{"name":"oshawott","url":"https://pokeapi.co/api/v2/pokemon-species/501/"},{"name":"pansage","url":"https://pokeapi.co/api/v2/pokemon-species/511/"},{"name":"pansear","url":"https://pokeapi.co/api/v2/pokemon-species/513/"},{"name":"panpour","url":"https://pokeapi.co/api/v2/pokemon-species/515/"},{"name":"drilbur","url":"https://pokeapi.co/api/v2/pokemon-species/529/"},{"name":"audino","url":"https://pokeapi.co/api/v2/pokemon-species/531/"},{"name":"servine","url":"https://pokeapi.co/api/v2/pokemon-species/496/"},{"name":"pignite","url":"https://pokeapi.co/api/v2/pokemon-species/499/"},{"name":"emboar","url":"https://pokeapi.co/api/v2/pokemon-species/500/"},{"name":"dewott","url":"https://pokeapi.co/api/v2/pokemon-species/502/"},{"name":"watchog","url":"https://pokeapi.co/api/v2/pokemon-species/505/"},{"name":"simisage","url":"https://pokeapi.co/api/v2/pokemon-species/512/"},{"name":"simisear","url":"https://pokeapi.co/api/v2/pokemon-species/514/"},{"name":"simipour","url":"https://pokeapi.co/api/v2/pokemon-species/516/"},{"name":"palpitoad","url":"https://pokeapi.co/api/v2/pokemon-species/536/"},{"name":"krookodile","url":"https://pokeapi.co/api/v2/pokemon-species/553/"},{"name":"scraggy","url":"https://pokeapi.co/api/v2/pokemon-species/559/"},{"name":"elgyem","url":"https://pokeapi.co/api/v2/pokemon-species/605/"},{"name":"axew","url":"https://pokeapi.co/api/v2/pokemon-species/610/"},{"name":"cubchoo","url":"https://pokeapi.co/api/v2/pokemon-species/613/"},{"name":"mienfoo","url":"https://pokeapi.co/api/v2/pokemon-species/619/"},{"name":"druddigon","url":"https://pokeapi.co/api/v2/pokemon-species/621/"},{"name":"heatmor","url":"https://pokeapi.co/api/v2/pokemon-species/631/"},{"name":"scrafty","url":"https://pokeapi.co/api/v2/pokemon-species/560/"},{"name":"carracosta","url":"https://pokeapi.co/api/v2/pokemon-species/565/"},{"name":"zoroark","url":"https://pokeapi.co/api/v2/pokemon-species/571/"},{"name":"fraxure","url":"https://pokeapi.co/api/v2/pokemon-species/611/"},{"name":"haxorus","url":"https://pokeapi.co/api/v2/pokemon-species/612/"},{"name":"mienshao","url":"https://pokeapi.co/api/v2/pokemon-species/620/"},{"name":"hydreigon","url":"https://pokeapi.co/api/v2/pokemon-species/635/"},{"name":"zekrom","url":"https://pokeapi.co/api/v2/pokemon-species/644/"},{"name":"kyurem","url":"https://pokeapi.co/api/v2/pokemon-species/646/"},{"name":"chespin","url":"https://pokeapi.co/api/v2/pokemon-species/650/"},{"name":"bunnelby","url":"https://pokeapi.co/api/v2/pokemon-species/659/"},{"name":"pancham","url":"https://pokeapi.co/api/v2/pokemon-species/674/"},{"name":"espurr","url":"https://pokeapi.co/api/v2/pokemon-species/677/"},{"name":"helioptile","url":"https://pokeapi.co/api/v2/pokemon-species/694/"},{"name":"tyrunt","url":"https://pokeapi.co/api/v2/pokemon-species/696/"},{"name":"dedenne","url":"https://pokeapi.co/api/v2/pokemon-species/702/"},{"name":"quilladin","url":"https://pokeapi.co/api/v2/pokemon-species/651/"},{"name":"chesnaught","url":"https://pokeapi.co/api/v2/pokemon-species/652/"},{"name":"braixen","url":"https://pokeapi.co/api/v2/pokemon-species/654/"},{"name":"delphox","url":"https://pokeapi.co/api/v2/pokemon-species/655/"},{"name":"diggersby","url":"https://pokeapi.co/api/v2/pokemon-species/660/"},{"name":"meowstic","url":"https://pokeapi.co/api/v2/pokemon-species/678/"},{"name":"heliolisk","url":"https://pokeapi.co/api/v2/pokemon-species/695/"},{"name":"tyrantrum","url":"https://pokeapi.co/api/v2/pokemon-species/697/"},{"name":"goodra","url":"https://pokeapi.co/api/v2/pokemon-species/706/"},{"name":"fomantis","url":"https://pokeapi.co/api/v2/pokemon-species/753/"},{"name":"passimian","url":"https://pokeapi.co/api/v2/pokemon-species/766/"},{"name":"turtonator","url":"https://pokeapi.co/api/v2/pokemon-species/776/"},{"name":"togedemaru","url":"https://pokeapi.co/api/v2/pokemon-species/777/"},{"name":"xurkitree","url":"https://pokeapi.co/api/v2/pokemon-species/796/"},{"name":"incineroar","url":"https://pokeapi.co/api/v2/pokemon-species/727/"},{"name":"bewear","url":"https://pokeapi.co/api/v2/pokemon-species/760/"},{"name":"hakamo-o","url":"https://pokeapi.co/api/v2/pokemon-species/783/"},{"name":"kommo-o","url":"https://pokeapi.co/api/v2/pokemon-species/784/"},{"name":"guzzlord","url":"https://pokeapi.co/api/v2/pokemon-species/799/"},{"name":"poipole","url":"https://pokeapi.co/api/v2/pokemon-species/803/"},{"name":"slowbro","url":"https://pokeapi.co/api/v2/pokemon-species/80/"},{"name":"croconaw","url":"https://pokeapi.co/api/v2/pokemon-species/159/"},{"name":"hitmontop","url":"https://pokeapi.co/api/v2/pokemon-species/237/"},{"name":"prinplup","url":"https://pokeapi.co/api/v2/pokemon-species/394/"},{"name":"abomasnow","url":"https://pokeapi.co/api/v2/pokemon-species/460/"}]}"#;

#[test]
fn pokemon_shape_from_id() {
    test_from_id::<PokemonShape>(6, UPRIGHT_JSON);
}

#[test]
fn pokemon_shape_from_name() {
    test_from_name::<PokemonShape>("upright", UPRIGHT_JSON);
}

// https://pokeapi.co/api/v2/pokemon-species/133
static EEVEE_JSON: &str = r#"{"base_happiness":70,"capture_rate":45,"color":{"name":"brown","url":"https://pokeapi.co/api/v2/pokemon-color/3/"},"egg_groups":[{"name":"ground","url":"https://pokeapi.co/api/v2/egg-group/5/"}],"evolution_chain":{"url":"https://pokeapi.co/api/v2/evolution-chain/67/"},"evolves_from_species":null,"flavor_text_entries":[{"flavor_text":"至今未弄清为何只有伊布\n携带着非常不稳定的遗传基因。","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"イーブイだけが　とても　不安定な\n遺伝子を　持っている　理由は\n未だ　解明　されて　いない。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"The question of why only Eevee has such\nunstable genes has still not been solved.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"Non si conosce ancora il motivo per cui Eevee\nsia l’unico Pokémon a possedere un corredo\ngenetico così instabile.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"Todavía no se ha esclarecido el motivo por el\nque es el único Pokémon que presenta un\ncódigo genético inestable.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"Evoli ist das einzige Pokémon mit einem höchst\ninstabilen Erbmaterial. Die Ursache dafür ist\nnach wie vor unklar.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"À ce jour, nul ne sait pourquoi Évoli est le seul\nPokémon dont l’ADN est instable.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"為何只有伊布有著\n極其不穩定的基因？\n理由至今仍然不明。","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"이브이만 매우 불안정한\n유전자를 갖고 있는 이유는\n아직 해명되지 않았다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"イーブイだけが　とても　ふあんていな\nいでんしを　もっている　りゆうは\nいまだ　かいめい　されて　いない。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version":{"name":"ultra-sun","url":"https://pokeapi.co/api/v2/version/29/"}},{"flavor_text":"根据目前的调查，\n拥有进化为多达８种\n宝可梦的可能性。","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"今　現在の　調査では\nなんと　８種類もの　ポケモンへ\n進化する　可能性を　持つ。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"Current studies show it can evolve into an\nincredible eight different species of Pokémon.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"Secondo le ultime ricerche, può evolversi in ben\notto specie diverse.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"Según las últimas investigaciones, Eevee es\ncapaz de evolucionar a ocho especies de\nPokémon distintas.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"Nach derzeitigem Forschungsstand kann es sich\nzu acht verschiedenen Pokémon entwickeln.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"D’après les dernières recherches, il semble que\nce Pokémon soit capable d’évoluer en huit\nespèces différentes.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"根據目前的調查，\n擁有進化為多達８種\n寶可夢的可能性。","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"지금까지 조사에 의하면\n무려 8종류의 포켓몬으로\n진화할 가능성을 갖고 있다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"いま　げんざいの　ちょうさでは\nなんと　８しゅるいもの　ポケモンへ\nしんかする　かのうせいを　もつ。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version":{"name":"moon","url":"https://pokeapi.co/api/v2/version/28/"}},{"flavor_text":"暮らしている　環境で　突然変異　する\n不安定な　遺伝子を　持つ　ポケモン。\n石の　放射線が　進化を　引き起こす。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"Eevee has an unstable genetic makeup that suddenly mutates\ndue to the environment in which it lives. Radiation from various\nstones causes this Pokémon to evolve.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"Eevee è dotato di un instabile corredo genetico che muta\nin base all’habitat naturale in cui è integrato. Le radiazioni\ndi diverse pietre scatenano l’evoluzione di questo Pokémon.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"La configuración genética de Eevee le permite mutar y\nadaptarse enseguida a cualquier medio en el que viva.\nLa evolución de este Pokémon suele ser posible gracias a\nlas radiaciones emitidas por varias piedras.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"Evolis genetisches Erbmaterial ist so instabil, dass die\nUmgebung plötzliche Mutationen auslösen kann. Unter\nanderem führt die Wirkung verschiedener Steine zur\nEntwicklung dieses Pokémon.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"Évoli a une structure génétique instable qui se transforme\nen fonction de l’environnement dans lequel il vit.\nCe Pokémon peut évoluer grâce aux radiations de\ndiverses pierres.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"사는 환경에 따라 돌연변이하는\n불안정한 유전자를 지닌 포켓몬이다.\n돌의 방사선이 진화를 일으킨다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"くらしている　かんきょうで　とつぜんへんい　する\nふあんていな　いでんしを　もつ　ポケモン。\nいしの　ほうしゃせんが　しんかを　ひきおこす。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version":{"name":"alpha-sapphire","url":"https://pokeapi.co/api/v2/version/26/"}},{"flavor_text":"暮らしている　環境で　突然変異　する\n不安定な　遺伝子を　持つ　ポケモン。\n石の　放射線が　進化を　引き起こす。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"Eevee has an unstable genetic makeup that suddenly mutates\ndue to the environment in which it lives. Radiation from various\nstones causes this Pokémon to evolve.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"Eevee è dotato di un instabile corredo genetico che muta\nin base all’habitat naturale in cui è integrato. Le radiazioni\ndi diverse pietre scatenano l’evoluzione di questo Pokémon.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"La configuración genética de Eevee le permite mutar y\nadaptarse enseguida a cualquier medio en el que viva.\nLa evolución de este Pokémon suele ser posible gracias a\nlas radiaciones emitidas por varias piedras.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"Evolis genetisches Erbmaterial ist so instabil, dass die\nUmgebung plötzliche Mutationen auslösen kann. Unter\nanderem führt die Wirkung verschiedener Steine zur\nEntwicklung dieses Pokémon.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"Évoli a une structure génétique instable qui se transforme\nen fonction de l’environnement dans lequel il vit.\nCe Pokémon peut évoluer grâce aux radiations de\ndiverses pierres.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"사는 환경에 따라 돌연변이하는\n불안정한 유전자를 지닌 포켓몬이다.\n돌의 방사선이 진화를 일으킨다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"くらしている　かんきょうで　とつぜんへんい　する\nふあんていな　いでんしを　もつ　ポケモン。\nいしの　ほうしゃせんが　しんかを　ひきおこす。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version":{"name":"omega-ruby","url":"https://pokeapi.co/api/v2/version/25/"}},{"flavor_text":"不安定な　遺伝子の　おかげで\nさまざまな　進化の　可能性を\n秘めている　特殊な　ポケモン。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"Thanks to its unstable genetic makeup, this\nspecial Pokémon conceals many different\npossible evolutions.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"È un Pokémon molto particolare. Dotato di un corredo\ngenetico instabile, serba in sé le più varie possibilità\nevolutive.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"Su irregular estructura genética encierra el secreto de\nsu capacidad para adoptar evoluciones de lo más\ndiversas.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"Aufgrund einer genetischen Anomalie kann seine\nEntwicklung viele verschiedene Formen annehmen.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"L’ADN de ce Pokémon très particulier recèle de\nnombreuses possibilités d’évolution.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"불안정한 유전자 덕분에\n여러 가지 진화 가능성을\n간직하고 있는 특수한 포켓몬.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"ふあんていな　いでんしの　おかげで\nさまざまな　しんかの　かのうせいを\nひめている　とくしゅな　ポケモン。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version":{"name":"y","url":"https://pokeapi.co/api/v2/version/24/"}},{"flavor_text":"進化のとき　姿と　能力が\n変わることで　きびしい　環境に\n対応する　珍しい　ポケモン。","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"A rare Pokémon that adapts to harsh environments\nby taking on different evolutionary forms.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"Un Pokémon raro che si adatta agli ambienti più\ndifficili assumendo diverse forme nell’evoluzione.","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"Un extraño Pokémon que se adapta a los entornos\nmás hostiles gracias a sus diferentes evoluciones.","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"Ein seltenes Pokémon, das sich seiner Umgebung\nanpasst, indem es sich in unterschiedliche Formen\nentwickelt.","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"Un Pokémon rare qui s’adapte aux environnements\nhostiles en variant ses formes évolutives.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"진화할 때 모습과 능력이\n바뀜으로써 혹독한 환경에\n적응하는 희귀한 포켓몬이다.","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"しんかのとき　すがたと　のうりょくが\nかわることで　きびしい　かんきょうに\nたいおうする　めずらしい　ポケモン。","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"version":{"name":"x","url":"https://pokeapi.co/api/v2/version/23/"}},{"flavor_text":"Thanks to its unstable genetic makeup,\nthis special Pokémon conceals many\ndifferent possible evolutions.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"white-2","url":"https://pokeapi.co/api/v2/version/22/"}},{"flavor_text":"Thanks to its unstable genetic makeup,\nthis special Pokémon conceals many\ndifferent possible evolutions.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"black-2","url":"https://pokeapi.co/api/v2/version/21/"}},{"flavor_text":"Because its genetic makeup is\nirregular, it quickly changes its\nform due to a variety of causes.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"flavor_text":"Son ADN particulier lui permet de\ns’adapter très rapidement à son\nenvironnement.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"white","url":"https://pokeapi.co/api/v2/version/18/"}},{"flavor_text":"Because its genetic makeup is\nirregular, it quickly changes its\nform due to a variety of causes.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"flavor_text":"Son ADN particulier lui permet de\ns’adapter très rapidement à son\nenvironnement.","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"version":{"name":"black","url":"https://pokeapi.co/api/v2/version/17/"}},{"flavor_text":"Its irregularly configured DNA is\naffected by its surroundings. It\nevolves if its environment changes.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"soulsilver","url":"https://pokeapi.co/api/v2/version/16/"}},{"flavor_text":"It has the ability to alter the\ncomposition of its body to suit its\nsurrounding environment.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"heartgold","url":"https://pokeapi.co/api/v2/version/15/"}},{"flavor_text":"Because its genetic makeup is\nirregular, it quickly changes its\nform due to a variety of causes.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"platinum","url":"https://pokeapi.co/api/v2/version/14/"}},{"flavor_text":"A rare Pokémon that adapts to\nharsh environments by taking on\ndifferent evolutionary forms.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"pearl","url":"https://pokeapi.co/api/v2/version/13/"}},{"flavor_text":"A rare Pokémon that adapts to\nharsh environments by taking on\ndifferent evolutionary forms.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"diamond","url":"https://pokeapi.co/api/v2/version/12/"}},{"flavor_text":"Its genetic code is irregular. It may\nmutate if it is exposed to radiation from\nelement STONES.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"leafgreen","url":"https://pokeapi.co/api/v2/version/11/"}},{"flavor_text":"An extremely rare POKéMON that may\nevolve in a number of different ways\ndepending on stimuli.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"firered","url":"https://pokeapi.co/api/v2/version/10/"}},{"flavor_text":"An EEVEE has an unstable genetic makeup\nthat suddenly mutates due to its\nenvironment. Radiation from various\nSTONES causes this POKéMON to evolve.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"emerald","url":"https://pokeapi.co/api/v2/version/9/"}},{"flavor_text":"EEVEE has an unstable genetic makeup\nthat suddenly mutates due to the\nenvironment in which it lives.\fRadiation from various STONES causes\nthis POKéMON to evolve.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"sapphire","url":"https://pokeapi.co/api/v2/version/8/"}},{"flavor_text":"EEVEE has an unstable genetic makeup\nthat suddenly mutates due to the\nenvironment in which it lives.\fRadiation from various STONES causes\nthis POKéMON to evolve.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"ruby","url":"https://pokeapi.co/api/v2/version/7/"}},{"flavor_text":"Its ability to\nevolve into many\nforms allows it to\fadapt smoothly\nand perfectly to\nany environment.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"crystal","url":"https://pokeapi.co/api/v2/version/6/"}},{"flavor_text":"Its irregularly\nconfigured DNA is\naffected by its\fsurroundings. It\nevolves if its en­\nvironment changes.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"silver","url":"https://pokeapi.co/api/v2/version/5/"}},{"flavor_text":"It has the ability\nto alter the com­\nposition of its\fbody to suit its\nsurrounding envi­\nronment.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"gold","url":"https://pokeapi.co/api/v2/version/4/"}},{"flavor_text":"Its genetic code\nis unstable, so it\ncould evolve in\fa variety of ways.\nThere are only a\nfew alive.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"yellow","url":"https://pokeapi.co/api/v2/version/3/"}},{"flavor_text":"Its genetic code\nis irregular.\nIt may mutate if\fit is exposed to\nradiation from\nelement STONEs.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"blue","url":"https://pokeapi.co/api/v2/version/2/"}},{"flavor_text":"Its genetic code\nis irregular.\nIt may mutate if\fit is exposed to\nradiation from\nelement STONEs.","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"version":{"name":"red","url":"https://pokeapi.co/api/v2/version/1/"}}],"form_descriptions":[],"forms_switchable":false,"gender_rate":1,"genera":[{"genus":"进化宝可梦","language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"}},{"genus":"しんかポケモン","language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"}},{"genus":"Evolution Pokémon","language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"}},{"genus":"Pokémon Evoluzione","language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"}},{"genus":"Pokémon Evolución","language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"}},{"genus":"Evolution","language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"}},{"genus":"Pokémon Évolutif","language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"}},{"genus":"進化寶可夢","language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"}},{"genus":"진화포켓몬","language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"}},{"genus":"しんかポケモン","language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"}}],"generation":{"name":"generation-i","url":"https://pokeapi.co/api/v2/generation/1/"},"growth_rate":{"name":"medium","url":"https://pokeapi.co/api/v2/growth-rate/2/"},"habitat":{"name":"urban","url":"https://pokeapi.co/api/v2/pokemon-habitat/8/"},"has_gender_differences":false,"hatch_counter":35,"id":133,"is_baby":false,"name":"eevee","names":[{"language":{"name":"zh-Hans","url":"https://pokeapi.co/api/v2/language/12/"},"name":"伊布"},{"language":{"name":"ja","url":"https://pokeapi.co/api/v2/language/11/"},"name":"イーブイ"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Eevee"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Eevee"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Eevee"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Evoli"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Évoli"},{"language":{"name":"zh-Hant","url":"https://pokeapi.co/api/v2/language/4/"},"name":"伊布"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"이브이"},{"language":{"name":"roomaji","url":"https://pokeapi.co/api/v2/language/2/"},"name":"Eievui"},{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"イーブイ"}],"order":157,"pal_park_encounters":[{"area":{"name":"field","url":"https://pokeapi.co/api/v2/pal-park-area/2/"},"base_score":90,"rate":3}],"pokedex_numbers":[{"entry_number":77,"pokedex":{"name":"kalos-coastal","url":"https://pokeapi.co/api/v2/pokedex/13/"}},{"entry_number":1,"pokedex":{"name":"conquest-gallery","url":"https://pokeapi.co/api/v2/pokedex/11/"}},{"entry_number":91,"pokedex":{"name":"updated-unova","url":"https://pokeapi.co/api/v2/pokedex/9/"}},{"entry_number":184,"pokedex":{"name":"updated-johto","url":"https://pokeapi.co/api/v2/pokedex/7/"}},{"entry_number":163,"pokedex":{"name":"extended-sinnoh","url":"https://pokeapi.co/api/v2/pokedex/6/"}},{"entry_number":180,"pokedex":{"name":"original-johto","url":"https://pokeapi.co/api/v2/pokedex/3/"}},{"entry_number":133,"pokedex":{"name":"kanto","url":"https://pokeapi.co/api/v2/pokedex/2/"}},{"entry_number":133,"pokedex":{"name":"national","url":"https://pokeapi.co/api/v2/pokedex/1/"}}],"shape":{"name":"quadruped","url":"https://pokeapi.co/api/v2/pokemon-shape/8/"},"varieties":[{"is_default":true,"pokemon":{"name":"eevee","url":"https://pokeapi.co/api/v2/pokemon/133/"}}]}"#;

#[test]
fn pokemon_species_from_id() {
    test_from_id::<PokemonSpecies>(133, EEVEE_JSON);
}

#[test]
fn pokemon_species_from_name() {
    test_from_name::<PokemonSpecies>("eevee", EEVEE_JSON);
}

// https://pokeapi.co/api/v2/stat/8/
static EVASION_JSON: &str = r#"{"affecting_moves":{"decrease":[{"change":-2,"move":{"name":"sweet-scent","url":"https://pokeapi.co/api/v2/move/230/"}}],"increase":[{"change":1,"move":{"name":"double-team","url":"https://pokeapi.co/api/v2/move/104/"}},{"change":2,"move":{"name":"minimize","url":"https://pokeapi.co/api/v2/move/107/"}}]},"affecting_natures":{"decrease":[],"increase":[]},"characteristics":[],"game_index":0,"id":8,"is_battle_only":true,"move_damage_class":null,"name":"evasion","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"かいひ"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Esquive"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Fluchtwert"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Evasión"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"elusione"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"evasion"}]}"#;

#[test]
fn stat_from_id() {
    test_from_id::<Stat>(7, EVASION_JSON);
}

#[test]
fn stat_from_name() {
    test_from_name::<Stat>("evasion", EVASION_JSON);
}

static NORMAL_JSON: &str = r#"{"damage_relations":{"double_damage_from":[{"name":"fighting","url":"https://pokeapi.co/api/v2/type/2/"}],"double_damage_to":[],"half_damage_from":[],"half_damage_to":[{"name":"rock","url":"https://pokeapi.co/api/v2/type/6/"},{"name":"steel","url":"https://pokeapi.co/api/v2/type/9/"}],"no_damage_from":[{"name":"ghost","url":"https://pokeapi.co/api/v2/type/8/"}],"no_damage_to":[{"name":"ghost","url":"https://pokeapi.co/api/v2/type/8/"}]},"game_indices":[{"game_index":0,"generation":{"name":"generation-i","url":"https://pokeapi.co/api/v2/generation/1/"}},{"game_index":0,"generation":{"name":"generation-ii","url":"https://pokeapi.co/api/v2/generation/2/"}},{"game_index":0,"generation":{"name":"generation-iii","url":"https://pokeapi.co/api/v2/generation/3/"}},{"game_index":0,"generation":{"name":"generation-iv","url":"https://pokeapi.co/api/v2/generation/4/"}},{"game_index":0,"generation":{"name":"generation-v","url":"https://pokeapi.co/api/v2/generation/5/"}},{"game_index":0,"generation":{"name":"generation-vi","url":"https://pokeapi.co/api/v2/generation/6/"}}],"generation":{"name":"generation-i","url":"https://pokeapi.co/api/v2/generation/1/"},"id":1,"move_damage_class":{"name":"physical","url":"https://pokeapi.co/api/v2/move-damage-class/2/"},"moves":[{"name":"pound","url":"https://pokeapi.co/api/v2/move/1/"},{"name":"double-slap","url":"https://pokeapi.co/api/v2/move/3/"},{"name":"comet-punch","url":"https://pokeapi.co/api/v2/move/4/"},{"name":"mega-punch","url":"https://pokeapi.co/api/v2/move/5/"},{"name":"pay-day","url":"https://pokeapi.co/api/v2/move/6/"},{"name":"scratch","url":"https://pokeapi.co/api/v2/move/10/"},{"name":"vice-grip","url":"https://pokeapi.co/api/v2/move/11/"},{"name":"guillotine","url":"https://pokeapi.co/api/v2/move/12/"},{"name":"razor-wind","url":"https://pokeapi.co/api/v2/move/13/"},{"name":"swords-dance","url":"https://pokeapi.co/api/v2/move/14/"},{"name":"cut","url":"https://pokeapi.co/api/v2/move/15/"},{"name":"whirlwind","url":"https://pokeapi.co/api/v2/move/18/"},{"name":"bind","url":"https://pokeapi.co/api/v2/move/20/"},{"name":"slam","url":"https://pokeapi.co/api/v2/move/21/"},{"name":"stomp","url":"https://pokeapi.co/api/v2/move/23/"},{"name":"mega-kick","url":"https://pokeapi.co/api/v2/move/25/"},{"name":"headbutt","url":"https://pokeapi.co/api/v2/move/29/"},{"name":"horn-attack","url":"https://pokeapi.co/api/v2/move/30/"},{"name":"fury-attack","url":"https://pokeapi.co/api/v2/move/31/"},{"name":"horn-drill","url":"https://pokeapi.co/api/v2/move/32/"},{"name":"tackle","url":"https://pokeapi.co/api/v2/move/33/"},{"name":"body-slam","url":"https://pokeapi.co/api/v2/move/34/"},{"name":"wrap","url":"https://pokeapi.co/api/v2/move/35/"},{"name":"take-down","url":"https://pokeapi.co/api/v2/move/36/"},{"name":"thrash","url":"https://pokeapi.co/api/v2/move/37/"},{"name":"double-edge","url":"https://pokeapi.co/api/v2/move/38/"},{"name":"tail-whip","url":"https://pokeapi.co/api/v2/move/39/"},{"name":"leer","url":"https://pokeapi.co/api/v2/move/43/"},{"name":"growl","url":"https://pokeapi.co/api/v2/move/45/"},{"name":"roar","url":"https://pokeapi.co/api/v2/move/46/"},{"name":"sing","url":"https://pokeapi.co/api/v2/move/47/"},{"name":"supersonic","url":"https://pokeapi.co/api/v2/move/48/"},{"name":"sonic-boom","url":"https://pokeapi.co/api/v2/move/49/"},{"name":"disable","url":"https://pokeapi.co/api/v2/move/50/"},{"name":"hyper-beam","url":"https://pokeapi.co/api/v2/move/63/"},{"name":"strength","url":"https://pokeapi.co/api/v2/move/70/"},{"name":"growth","url":"https://pokeapi.co/api/v2/move/74/"},{"name":"quick-attack","url":"https://pokeapi.co/api/v2/move/98/"},{"name":"rage","url":"https://pokeapi.co/api/v2/move/99/"},{"name":"mimic","url":"https://pokeapi.co/api/v2/move/102/"},{"name":"screech","url":"https://pokeapi.co/api/v2/move/103/"},{"name":"double-team","url":"https://pokeapi.co/api/v2/move/104/"},{"name":"recover","url":"https://pokeapi.co/api/v2/move/105/"},{"name":"harden","url":"https://pokeapi.co/api/v2/move/106/"},{"name":"minimize","url":"https://pokeapi.co/api/v2/move/107/"},{"name":"smokescreen","url":"https://pokeapi.co/api/v2/move/108/"},{"name":"defense-curl","url":"https://pokeapi.co/api/v2/move/111/"},{"name":"focus-energy","url":"https://pokeapi.co/api/v2/move/116/"},{"name":"bide","url":"https://pokeapi.co/api/v2/move/117/"},{"name":"metronome","url":"https://pokeapi.co/api/v2/move/118/"},{"name":"self-destruct","url":"https://pokeapi.co/api/v2/move/120/"},{"name":"egg-bomb","url":"https://pokeapi.co/api/v2/move/121/"},{"name":"swift","url":"https://pokeapi.co/api/v2/move/129/"},{"name":"skull-bash","url":"https://pokeapi.co/api/v2/move/130/"},{"name":"spike-cannon","url":"https://pokeapi.co/api/v2/move/131/"},{"name":"constrict","url":"https://pokeapi.co/api/v2/move/132/"},{"name":"soft-boiled","url":"https://pokeapi.co/api/v2/move/135/"},{"name":"glare","url":"https://pokeapi.co/api/v2/move/137/"},{"name":"barrage","url":"https://pokeapi.co/api/v2/move/140/"},{"name":"lovely-kiss","url":"https://pokeapi.co/api/v2/move/142/"},{"name":"transform","url":"https://pokeapi.co/api/v2/move/144/"},{"name":"dizzy-punch","url":"https://pokeapi.co/api/v2/move/146/"},{"name":"flash","url":"https://pokeapi.co/api/v2/move/148/"},{"name":"splash","url":"https://pokeapi.co/api/v2/move/150/"},{"name":"explosion","url":"https://pokeapi.co/api/v2/move/153/"},{"name":"fury-swipes","url":"https://pokeapi.co/api/v2/move/154/"},{"name":"hyper-fang","url":"https://pokeapi.co/api/v2/move/158/"},{"name":"sharpen","url":"https://pokeapi.co/api/v2/move/159/"},{"name":"conversion","url":"https://pokeapi.co/api/v2/move/160/"},{"name":"tri-attack","url":"https://pokeapi.co/api/v2/move/161/"},{"name":"super-fang","url":"https://pokeapi.co/api/v2/move/162/"},{"name":"slash","url":"https://pokeapi.co/api/v2/move/163/"},{"name":"substitute","url":"https://pokeapi.co/api/v2/move/164/"},{"name":"struggle","url":"https://pokeapi.co/api/v2/move/165/"},{"name":"sketch","url":"https://pokeapi.co/api/v2/move/166/"},{"name":"mind-reader","url":"https://pokeapi.co/api/v2/move/170/"},{"name":"snore","url":"https://pokeapi.co/api/v2/move/173/"},{"name":"flail","url":"https://pokeapi.co/api/v2/move/175/"},{"name":"conversion-2","url":"https://pokeapi.co/api/v2/move/176/"},{"name":"protect","url":"https://pokeapi.co/api/v2/move/182/"},{"name":"scary-face","url":"https://pokeapi.co/api/v2/move/184/"},{"name":"belly-drum","url":"https://pokeapi.co/api/v2/move/187/"},{"name":"foresight","url":"https://pokeapi.co/api/v2/move/193/"},{"name":"perish-song","url":"https://pokeapi.co/api/v2/move/195/"},{"name":"lock-on","url":"https://pokeapi.co/api/v2/move/199/"},{"name":"endure","url":"https://pokeapi.co/api/v2/move/203/"},{"name":"false-swipe","url":"https://pokeapi.co/api/v2/move/206/"},{"name":"swagger","url":"https://pokeapi.co/api/v2/move/207/"},{"name":"milk-drink","url":"https://pokeapi.co/api/v2/move/208/"},{"name":"mean-look","url":"https://pokeapi.co/api/v2/move/212/"},{"name":"attract","url":"https://pokeapi.co/api/v2/move/213/"},{"name":"sleep-talk","url":"https://pokeapi.co/api/v2/move/214/"},{"name":"heal-bell","url":"https://pokeapi.co/api/v2/move/215/"},{"name":"return","url":"https://pokeapi.co/api/v2/move/216/"},{"name":"present","url":"https://pokeapi.co/api/v2/move/217/"},{"name":"frustration","url":"https://pokeapi.co/api/v2/move/218/"},{"name":"safeguard","url":"https://pokeapi.co/api/v2/move/219/"},{"name":"pain-split","url":"https://pokeapi.co/api/v2/move/220/"},{"name":"baton-pass","url":"https://pokeapi.co/api/v2/move/226/"},{"name":"encore","url":"https://pokeapi.co/api/v2/move/227/"},{"name":"rapid-spin","url":"https://pokeapi.co/api/v2/move/229/"},{"name":"sweet-scent","url":"https://pokeapi.co/api/v2/move/230/"},{"name":"morning-sun","url":"https://pokeapi.co/api/v2/move/234/"},{"name":"hidden-power","url":"https://pokeapi.co/api/v2/move/237/"},{"name":"psych-up","url":"https://pokeapi.co/api/v2/move/244/"},{"name":"extreme-speed","url":"https://pokeapi.co/api/v2/move/245/"},{"name":"fake-out","url":"https://pokeapi.co/api/v2/move/252/"},{"name":"uproar","url":"https://pokeapi.co/api/v2/move/253/"},{"name":"stockpile","url":"https://pokeapi.co/api/v2/move/254/"},{"name":"spit-up","url":"https://pokeapi.co/api/v2/move/255/"},{"name":"swallow","url":"https://pokeapi.co/api/v2/move/256/"},{"name":"facade","url":"https://pokeapi.co/api/v2/move/263/"},{"name":"smelling-salts","url":"https://pokeapi.co/api/v2/move/265/"},{"name":"follow-me","url":"https://pokeapi.co/api/v2/move/266/"},{"name":"nature-power","url":"https://pokeapi.co/api/v2/move/267/"},{"name":"helping-hand","url":"https://pokeapi.co/api/v2/move/270/"},{"name":"wish","url":"https://pokeapi.co/api/v2/move/273/"},{"name":"assist","url":"https://pokeapi.co/api/v2/move/274/"},{"name":"recycle","url":"https://pokeapi.co/api/v2/move/278/"},{"name":"yawn","url":"https://pokeapi.co/api/v2/move/281/"},{"name":"endeavor","url":"https://pokeapi.co/api/v2/move/283/"},{"name":"refresh","url":"https://pokeapi.co/api/v2/move/287/"},{"name":"secret-power","url":"https://pokeapi.co/api/v2/move/290/"},{"name":"camouflage","url":"https://pokeapi.co/api/v2/move/293/"},{"name":"teeter-dance","url":"https://pokeapi.co/api/v2/move/298/"},{"name":"slack-off","url":"https://pokeapi.co/api/v2/move/303/"},{"name":"hyper-voice","url":"https://pokeapi.co/api/v2/move/304/"},{"name":"crush-claw","url":"https://pokeapi.co/api/v2/move/306/"},{"name":"weather-ball","url":"https://pokeapi.co/api/v2/move/311/"},{"name":"odor-sleuth","url":"https://pokeapi.co/api/v2/move/316/"},{"name":"tickle","url":"https://pokeapi.co/api/v2/move/321/"},{"name":"block","url":"https://pokeapi.co/api/v2/move/335/"},{"name":"howl","url":"https://pokeapi.co/api/v2/move/336/"},{"name":"covet","url":"https://pokeapi.co/api/v2/move/343/"},{"name":"natural-gift","url":"https://pokeapi.co/api/v2/move/363/"},{"name":"feint","url":"https://pokeapi.co/api/v2/move/364/"},{"name":"acupressure","url":"https://pokeapi.co/api/v2/move/367/"},{"name":"trump-card","url":"https://pokeapi.co/api/v2/move/376/"},{"name":"wring-out","url":"https://pokeapi.co/api/v2/move/378/"},{"name":"lucky-chant","url":"https://pokeapi.co/api/v2/move/381/"},{"name":"me-first","url":"https://pokeapi.co/api/v2/move/382/"},{"name":"copycat","url":"https://pokeapi.co/api/v2/move/383/"},{"name":"last-resort","url":"https://pokeapi.co/api/v2/move/387/"},{"name":"giga-impact","url":"https://pokeapi.co/api/v2/move/416/"},{"name":"rock-climb","url":"https://pokeapi.co/api/v2/move/431/"},{"name":"captivate","url":"https://pokeapi.co/api/v2/move/445/"},{"name":"judgment","url":"https://pokeapi.co/api/v2/move/449/"},{"name":"double-hit","url":"https://pokeapi.co/api/v2/move/458/"},{"name":"crush-grip","url":"https://pokeapi.co/api/v2/move/462/"},{"name":"simple-beam","url":"https://pokeapi.co/api/v2/move/493/"},{"name":"entrainment","url":"https://pokeapi.co/api/v2/move/494/"},{"name":"after-you","url":"https://pokeapi.co/api/v2/move/495/"},{"name":"round","url":"https://pokeapi.co/api/v2/move/496/"},{"name":"echoed-voice","url":"https://pokeapi.co/api/v2/move/497/"},{"name":"chip-away","url":"https://pokeapi.co/api/v2/move/498/"},{"name":"shell-smash","url":"https://pokeapi.co/api/v2/move/504/"},{"name":"reflect-type","url":"https://pokeapi.co/api/v2/move/513/"},{"name":"retaliate","url":"https://pokeapi.co/api/v2/move/514/"},{"name":"bestow","url":"https://pokeapi.co/api/v2/move/516/"},{"name":"work-up","url":"https://pokeapi.co/api/v2/move/526/"},{"name":"tail-slap","url":"https://pokeapi.co/api/v2/move/541/"},{"name":"head-charge","url":"https://pokeapi.co/api/v2/move/543/"},{"name":"techno-blast","url":"https://pokeapi.co/api/v2/move/546/"},{"name":"relic-song","url":"https://pokeapi.co/api/v2/move/547/"},{"name":"noble-roar","url":"https://pokeapi.co/api/v2/move/568/"},{"name":"boomburst","url":"https://pokeapi.co/api/v2/move/586/"},{"name":"play-nice","url":"https://pokeapi.co/api/v2/move/589/"},{"name":"confide","url":"https://pokeapi.co/api/v2/move/590/"},{"name":"happy-hour","url":"https://pokeapi.co/api/v2/move/603/"},{"name":"celebrate","url":"https://pokeapi.co/api/v2/move/606/"},{"name":"hold-hands","url":"https://pokeapi.co/api/v2/move/607/"},{"name":"hold-back","url":"https://pokeapi.co/api/v2/move/610/"},{"name":"breakneck-blitz--physical","url":"https://pokeapi.co/api/v2/move/622/"},{"name":"breakneck-blitz--special","url":"https://pokeapi.co/api/v2/move/623/"},{"name":"spotlight","url":"https://pokeapi.co/api/v2/move/671/"},{"name":"laser-focus","url":"https://pokeapi.co/api/v2/move/673/"},{"name":"revelation-dance","url":"https://pokeapi.co/api/v2/move/686/"},{"name":"pulverizing-pancake","url":"https://pokeapi.co/api/v2/move/701/"},{"name":"extreme-evoboost","url":"https://pokeapi.co/api/v2/move/702/"},{"name":"tearful-look","url":"https://pokeapi.co/api/v2/move/715/"},{"name":"multi-attack","url":"https://pokeapi.co/api/v2/move/718/"}],"name":"normal","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"ノーマル"},{"language":{"name":"ko","url":"https://pokeapi.co/api/v2/language/3/"},"name":"노말"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Normal"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Normal"},{"language":{"name":"es","url":"https://pokeapi.co/api/v2/language/7/"},"name":"Normal"},{"language":{"name":"it","url":"https://pokeapi.co/api/v2/language/8/"},"name":"Normale"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"Normal"}],"pokemon":[{"pokemon":{"name":"pidgey","url":"https://pokeapi.co/api/v2/pokemon/16/"},"slot":1},{"pokemon":{"name":"pidgeotto","url":"https://pokeapi.co/api/v2/pokemon/17/"},"slot":1},{"pokemon":{"name":"pidgeot","url":"https://pokeapi.co/api/v2/pokemon/18/"},"slot":1},{"pokemon":{"name":"rattata","url":"https://pokeapi.co/api/v2/pokemon/19/"},"slot":1},{"pokemon":{"name":"raticate","url":"https://pokeapi.co/api/v2/pokemon/20/"},"slot":1},{"pokemon":{"name":"spearow","url":"https://pokeapi.co/api/v2/pokemon/21/"},"slot":1},{"pokemon":{"name":"fearow","url":"https://pokeapi.co/api/v2/pokemon/22/"},"slot":1},{"pokemon":{"name":"jigglypuff","url":"https://pokeapi.co/api/v2/pokemon/39/"},"slot":1},{"pokemon":{"name":"wigglytuff","url":"https://pokeapi.co/api/v2/pokemon/40/"},"slot":1},{"pokemon":{"name":"meowth","url":"https://pokeapi.co/api/v2/pokemon/52/"},"slot":1},{"pokemon":{"name":"persian","url":"https://pokeapi.co/api/v2/pokemon/53/"},"slot":1},{"pokemon":{"name":"farfetchd","url":"https://pokeapi.co/api/v2/pokemon/83/"},"slot":1},{"pokemon":{"name":"doduo","url":"https://pokeapi.co/api/v2/pokemon/84/"},"slot":1},{"pokemon":{"name":"dodrio","url":"https://pokeapi.co/api/v2/pokemon/85/"},"slot":1},{"pokemon":{"name":"lickitung","url":"https://pokeapi.co/api/v2/pokemon/108/"},"slot":1},{"pokemon":{"name":"chansey","url":"https://pokeapi.co/api/v2/pokemon/113/"},"slot":1},{"pokemon":{"name":"kangaskhan","url":"https://pokeapi.co/api/v2/pokemon/115/"},"slot":1},{"pokemon":{"name":"tauros","url":"https://pokeapi.co/api/v2/pokemon/128/"},"slot":1},{"pokemon":{"name":"ditto","url":"https://pokeapi.co/api/v2/pokemon/132/"},"slot":1},{"pokemon":{"name":"eevee","url":"https://pokeapi.co/api/v2/pokemon/133/"},"slot":1},{"pokemon":{"name":"porygon","url":"https://pokeapi.co/api/v2/pokemon/137/"},"slot":1},{"pokemon":{"name":"snorlax","url":"https://pokeapi.co/api/v2/pokemon/143/"},"slot":1},{"pokemon":{"name":"sentret","url":"https://pokeapi.co/api/v2/pokemon/161/"},"slot":1},{"pokemon":{"name":"furret","url":"https://pokeapi.co/api/v2/pokemon/162/"},"slot":1},{"pokemon":{"name":"hoothoot","url":"https://pokeapi.co/api/v2/pokemon/163/"},"slot":1},{"pokemon":{"name":"noctowl","url":"https://pokeapi.co/api/v2/pokemon/164/"},"slot":1},{"pokemon":{"name":"igglybuff","url":"https://pokeapi.co/api/v2/pokemon/174/"},"slot":1},{"pokemon":{"name":"aipom","url":"https://pokeapi.co/api/v2/pokemon/190/"},"slot":1},{"pokemon":{"name":"girafarig","url":"https://pokeapi.co/api/v2/pokemon/203/"},"slot":1},{"pokemon":{"name":"dunsparce","url":"https://pokeapi.co/api/v2/pokemon/206/"},"slot":1},{"pokemon":{"name":"teddiursa","url":"https://pokeapi.co/api/v2/pokemon/216/"},"slot":1},{"pokemon":{"name":"ursaring","url":"https://pokeapi.co/api/v2/pokemon/217/"},"slot":1},{"pokemon":{"name":"porygon2","url":"https://pokeapi.co/api/v2/pokemon/233/"},"slot":1},{"pokemon":{"name":"stantler","url":"https://pokeapi.co/api/v2/pokemon/234/"},"slot":1},{"pokemon":{"name":"smeargle","url":"https://pokeapi.co/api/v2/pokemon/235/"},"slot":1},{"pokemon":{"name":"miltank","url":"https://pokeapi.co/api/v2/pokemon/241/"},"slot":1},{"pokemon":{"name":"blissey","url":"https://pokeapi.co/api/v2/pokemon/242/"},"slot":1},{"pokemon":{"name":"zigzagoon","url":"https://pokeapi.co/api/v2/pokemon/263/"},"slot":1},{"pokemon":{"name":"linoone","url":"https://pokeapi.co/api/v2/pokemon/264/"},"slot":1},{"pokemon":{"name":"taillow","url":"https://pokeapi.co/api/v2/pokemon/276/"},"slot":1},{"pokemon":{"name":"swellow","url":"https://pokeapi.co/api/v2/pokemon/277/"},"slot":1},{"pokemon":{"name":"slakoth","url":"https://pokeapi.co/api/v2/pokemon/287/"},"slot":1},{"pokemon":{"name":"vigoroth","url":"https://pokeapi.co/api/v2/pokemon/288/"},"slot":1},{"pokemon":{"name":"slaking","url":"https://pokeapi.co/api/v2/pokemon/289/"},"slot":1},{"pokemon":{"name":"whismur","url":"https://pokeapi.co/api/v2/pokemon/293/"},"slot":1},{"pokemon":{"name":"loudred","url":"https://pokeapi.co/api/v2/pokemon/294/"},"slot":1},{"pokemon":{"name":"exploud","url":"https://pokeapi.co/api/v2/pokemon/295/"},"slot":1},{"pokemon":{"name":"azurill","url":"https://pokeapi.co/api/v2/pokemon/298/"},"slot":1},{"pokemon":{"name":"skitty","url":"https://pokeapi.co/api/v2/pokemon/300/"},"slot":1},{"pokemon":{"name":"delcatty","url":"https://pokeapi.co/api/v2/pokemon/301/"},"slot":1},{"pokemon":{"name":"spinda","url":"https://pokeapi.co/api/v2/pokemon/327/"},"slot":1},{"pokemon":{"name":"swablu","url":"https://pokeapi.co/api/v2/pokemon/333/"},"slot":1},{"pokemon":{"name":"zangoose","url":"https://pokeapi.co/api/v2/pokemon/335/"},"slot":1},{"pokemon":{"name":"castform","url":"https://pokeapi.co/api/v2/pokemon/351/"},"slot":1},{"pokemon":{"name":"kecleon","url":"https://pokeapi.co/api/v2/pokemon/352/"},"slot":1},{"pokemon":{"name":"starly","url":"https://pokeapi.co/api/v2/pokemon/396/"},"slot":1},{"pokemon":{"name":"staravia","url":"https://pokeapi.co/api/v2/pokemon/397/"},"slot":1},{"pokemon":{"name":"staraptor","url":"https://pokeapi.co/api/v2/pokemon/398/"},"slot":1},{"pokemon":{"name":"bidoof","url":"https://pokeapi.co/api/v2/pokemon/399/"},"slot":1},{"pokemon":{"name":"bibarel","url":"https://pokeapi.co/api/v2/pokemon/400/"},"slot":1},{"pokemon":{"name":"ambipom","url":"https://pokeapi.co/api/v2/pokemon/424/"},"slot":1},{"pokemon":{"name":"buneary","url":"https://pokeapi.co/api/v2/pokemon/427/"},"slot":1},{"pokemon":{"name":"lopunny","url":"https://pokeapi.co/api/v2/pokemon/428/"},"slot":1},{"pokemon":{"name":"glameow","url":"https://pokeapi.co/api/v2/pokemon/431/"},"slot":1},{"pokemon":{"name":"purugly","url":"https://pokeapi.co/api/v2/pokemon/432/"},"slot":1},{"pokemon":{"name":"happiny","url":"https://pokeapi.co/api/v2/pokemon/440/"},"slot":1},{"pokemon":{"name":"chatot","url":"https://pokeapi.co/api/v2/pokemon/441/"},"slot":1},{"pokemon":{"name":"munchlax","url":"https://pokeapi.co/api/v2/pokemon/446/"},"slot":1},{"pokemon":{"name":"lickilicky","url":"https://pokeapi.co/api/v2/pokemon/463/"},"slot":1},{"pokemon":{"name":"porygon-z","url":"https://pokeapi.co/api/v2/pokemon/474/"},"slot":1},{"pokemon":{"name":"regigigas","url":"https://pokeapi.co/api/v2/pokemon/486/"},"slot":1},{"pokemon":{"name":"arceus","url":"https://pokeapi.co/api/v2/pokemon/493/"},"slot":1},{"pokemon":{"name":"patrat","url":"https://pokeapi.co/api/v2/pokemon/504/"},"slot":1},{"pokemon":{"name":"watchog","url":"https://pokeapi.co/api/v2/pokemon/505/"},"slot":1},{"pokemon":{"name":"lillipup","url":"https://pokeapi.co/api/v2/pokemon/506/"},"slot":1},{"pokemon":{"name":"herdier","url":"https://pokeapi.co/api/v2/pokemon/507/"},"slot":1},{"pokemon":{"name":"stoutland","url":"https://pokeapi.co/api/v2/pokemon/508/"},"slot":1},{"pokemon":{"name":"pidove","url":"https://pokeapi.co/api/v2/pokemon/519/"},"slot":1},{"pokemon":{"name":"tranquill","url":"https://pokeapi.co/api/v2/pokemon/520/"},"slot":1},{"pokemon":{"name":"unfezant","url":"https://pokeapi.co/api/v2/pokemon/521/"},"slot":1},{"pokemon":{"name":"audino","url":"https://pokeapi.co/api/v2/pokemon/531/"},"slot":1},{"pokemon":{"name":"minccino","url":"https://pokeapi.co/api/v2/pokemon/572/"},"slot":1},{"pokemon":{"name":"cinccino","url":"https://pokeapi.co/api/v2/pokemon/573/"},"slot":1},{"pokemon":{"name":"deerling","url":"https://pokeapi.co/api/v2/pokemon/585/"},"slot":1},{"pokemon":{"name":"sawsbuck","url":"https://pokeapi.co/api/v2/pokemon/586/"},"slot":1},{"pokemon":{"name":"bouffalant","url":"https://pokeapi.co/api/v2/pokemon/626/"},"slot":1},{"pokemon":{"name":"rufflet","url":"https://pokeapi.co/api/v2/pokemon/627/"},"slot":1},{"pokemon":{"name":"braviary","url":"https://pokeapi.co/api/v2/pokemon/628/"},"slot":1},{"pokemon":{"name":"meloetta-aria","url":"https://pokeapi.co/api/v2/pokemon/648/"},"slot":1},{"pokemon":{"name":"bunnelby","url":"https://pokeapi.co/api/v2/pokemon/659/"},"slot":1},{"pokemon":{"name":"diggersby","url":"https://pokeapi.co/api/v2/pokemon/660/"},"slot":1},{"pokemon":{"name":"fletchling","url":"https://pokeapi.co/api/v2/pokemon/661/"},"slot":1},{"pokemon":{"name":"litleo","url":"https://pokeapi.co/api/v2/pokemon/667/"},"slot":2},{"pokemon":{"name":"pyroar","url":"https://pokeapi.co/api/v2/pokemon/668/"},"slot":2},{"pokemon":{"name":"furfrou","url":"https://pokeapi.co/api/v2/pokemon/676/"},"slot":1},{"pokemon":{"name":"helioptile","url":"https://pokeapi.co/api/v2/pokemon/694/"},"slot":2},{"pokemon":{"name":"heliolisk","url":"https://pokeapi.co/api/v2/pokemon/695/"},"slot":2},{"pokemon":{"name":"pikipek","url":"https://pokeapi.co/api/v2/pokemon/731/"},"slot":1},{"pokemon":{"name":"trumbeak","url":"https://pokeapi.co/api/v2/pokemon/732/"},"slot":1},{"pokemon":{"name":"toucannon","url":"https://pokeapi.co/api/v2/pokemon/733/"},"slot":1},{"pokemon":{"name":"yungoos","url":"https://pokeapi.co/api/v2/pokemon/734/"},"slot":1},{"pokemon":{"name":"gumshoos","url":"https://pokeapi.co/api/v2/pokemon/735/"},"slot":1},{"pokemon":{"name":"stufful","url":"https://pokeapi.co/api/v2/pokemon/759/"},"slot":1},{"pokemon":{"name":"bewear","url":"https://pokeapi.co/api/v2/pokemon/760/"},"slot":1},{"pokemon":{"name":"oranguru","url":"https://pokeapi.co/api/v2/pokemon/765/"},"slot":1},{"pokemon":{"name":"type-null","url":"https://pokeapi.co/api/v2/pokemon/772/"},"slot":1},{"pokemon":{"name":"silvally","url":"https://pokeapi.co/api/v2/pokemon/773/"},"slot":1},{"pokemon":{"name":"komala","url":"https://pokeapi.co/api/v2/pokemon/775/"},"slot":1},{"pokemon":{"name":"drampa","url":"https://pokeapi.co/api/v2/pokemon/780/"},"slot":1},{"pokemon":{"name":"meloetta-pirouette","url":"https://pokeapi.co/api/v2/pokemon/10018/"},"slot":1},{"pokemon":{"name":"kangaskhan-mega","url":"https://pokeapi.co/api/v2/pokemon/10039/"},"slot":1},{"pokemon":{"name":"audino-mega","url":"https://pokeapi.co/api/v2/pokemon/10069/"},"slot":1},{"pokemon":{"name":"pidgeot-mega","url":"https://pokeapi.co/api/v2/pokemon/10073/"},"slot":1},{"pokemon":{"name":"lopunny-mega","url":"https://pokeapi.co/api/v2/pokemon/10088/"},"slot":1},{"pokemon":{"name":"rattata-alola","url":"https://pokeapi.co/api/v2/pokemon/10091/"},"slot":2},{"pokemon":{"name":"raticate-alola","url":"https://pokeapi.co/api/v2/pokemon/10092/"},"slot":2},{"pokemon":{"name":"raticate-totem-alola","url":"https://pokeapi.co/api/v2/pokemon/10093/"},"slot":2},{"pokemon":{"name":"gumshoos-totem","url":"https://pokeapi.co/api/v2/pokemon/10121/"},"slot":1}]}"#;

#[test]
fn type_from_id() {
    test_from_id::<Type>(1, NORMAL_JSON);
}

#[test]
fn type_from_name() {
    test_from_name::<Type>("normal", NORMAL_JSON);
}

static EN_JSON: &str = r#"{"id":9,"iso3166":"us","iso639":"en","name":"en","names":[{"language":{"name":"ja-Hrkt","url":"https://pokeapi.co/api/v2/language/1/"},"name":"英語"},{"language":{"name":"fr","url":"https://pokeapi.co/api/v2/language/5/"},"name":"Anglais"},{"language":{"name":"de","url":"https://pokeapi.co/api/v2/language/6/"},"name":"Englisch"},{"language":{"name":"en","url":"https://pokeapi.co/api/v2/language/9/"},"name":"English"}],"official":true}"#;

#[test]
fn language_from_id() {
    test_from_id::<Language>(9, EN_JSON);
}

#[test]
fn language_from_name() {
    test_from_name::<Language>("en", EN_JSON);
}
